// In version 1.04, I added the functionality of imputing marker file one by one.


/* GIGI: Genotype Imputation Given Inheritance
    Copyright (C) 2013 Charles Y K Cheung

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *GIGI is completely free for academic and non-commercial use.
 *GIGI
 * GIGI.cpp
 *  Last modified: July 14, 2013
 *
 *
 * This program imputes genotypes in pedigree data. Refer to the documentation for usage.
 *
 * @author Charles Cheung
 *
 *Usage:
 * To run GIGI:
 * ./GIGI PARAM_FILE, where the param file is formatted as:
 *
 *
// use g++
//gcc 4.3 specific stricter condition
*/

#include <time.h>
#include <sstream>
#include <map>
#include <list>

#include <cstdlib>
#include <string.h>

#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <math.h>

//#include "/acct/cykc/analysis/traitSimulation/scripts/misc/Mersenne/MersenneTwister.h"
#include "Mersenne/MersenneTwister.h"

//#include <sstream>
using namespace std;

static int MAX_NUM_PEOPLE_IN_PED = 5000;

/**
 * Data structure storing the Pedigree Meiosis Information
 * @see data file for a pedigree meiosis file, or see ()
 */
typedef struct {
	string id;
	string pid;
	string mid;
	int compid;
	int patMeio;
	int matMeio;
} PedMeioInfo;


/**
 * Data structure storing the Pedigree Meiosis Information
 * OSCOR object class  --implemented with int
 */
class OSCOR {
public:
	OSCOR(int num_Iterations, int num_Meiosis) {
		n_it = num_Iterations;
		n_Meiosis = num_Meiosis;

		initialValue = new int*[num_Iterations];
		for (int i = 0; i < num_Iterations; i++) {
			initialValue[i] = new int[num_Meiosis];
			for (int j = 0; j < num_Meiosis; j++) {
				initialValue[i][j] = -1;
			}
		}

		numRecomb = new int*[num_Iterations];
		for (int i = 0; i < num_Iterations; i++) {
			numRecomb[i] = new int[num_Meiosis];
			for (int j = 0; j < num_Meiosis; j++) {
				numRecomb[i][j] = -1;
			}
		}

		recombPos = new int**[num_Iterations];
		for (int i = 0; i < num_Iterations; i++) {
			recombPos[i] = new int*[num_Meiosis];
		}

	}

	//destructor
	~OSCOR() {

		//cout<< "recombPos at last position is " <<recombPos[n_it-1][n_Meiosis-1] <<endl;
		//delete[] recombPos[n_it-1][n_Meiosis-1]; //no idea why it isn't read!

		//for(int i=0; i< (n_it); i++){
		////cout << "i=" <<i <<"\n";
		//for(int j=0; j< (n_Meiosis); j++){
		////cout<< "r=" <<recombPos[i][j] << ": " ;
		//for(int k=0; k< numRecomb[i][j];k++){
		//  //cout << recombPos[i][j][k] <<" ";
		//  }
		//  delete[] recombPos[i][j];    //THIS LINE CAUSES ERROR
		//}
		//delete[] recombPos[i];
		////cout<<endl;
		//}
		//delete[] recombPos;


		for (int i = 0; i < n_it; i++) {
			delete[] initialValue[i];
			delete[] numRecomb[i];

			for (int j = 0; j < n_Meiosis; j++) {
				delete[] recombPos[i][j];
			}
			delete[] recombPos[i];
		}

		delete[] initialValue;
		delete[] numRecomb;
		delete[] recombPos;

		//cout <<"Still run!\n";
		//exit(0);
	}

	void writeLine(int cur_it, int cur_m, int firstPos, int numSwitch,
			int *markerPos) {
		initialValue[cur_it][cur_m] = firstPos;
		numRecomb[cur_it][cur_m] = numSwitch;
		recombPos[cur_it][cur_m] = new int[numSwitch];
		for (int i = 0; i < numSwitch; i++) {
			recombPos[cur_it][cur_m][i] = markerPos[i];
		}
	}

	int getNRecomb(int cur_it, int cur_m) {
		return (numRecomb[cur_it][cur_m]);
	}

	int *getRecombPos(int cur_it, int cur_m) {
		return (recombPos[cur_it][cur_m]);
	}
	int getInitialValue(int cur_it, int cur_m) {
		return (initialValue[cur_it][cur_m]);
	}

private:
	int n_it;
	int n_Meiosis;
	int **initialValue;
	int **numRecomb;
	int ***recombPos;

};

//NEED A DESTRUCTOR
class CodeTable {
public:
	CodeTable() {

	}
private:
	int codeTable;
};


/**
 * Pedigree structure class that contains a list of PediMeioInfo objects. This class is used to access the list of pedigree meioses and provide other functions about the list of objects.
 */
class Ped {
public:
	int n_ind; //number of individuals in the pedigree
	int n_Meio; //number of meiosis in the pedigree
	int startMeio; //the starting meiosis number


	/**add a pedMeioInfo object to the Ped class.
	 *@param info the pedigree meiosis info structure to be added to the Ped object
	 *@post the PedMeioInfo object is added to the Ped class. n_ind and is incremented. startMeio is set when the first meiosis is added.
	 */
	virtual void addMember(PedMeioInfo info)=0;

	/**
	 *@param index index of list of PedMeioInfo objects
	 *@return true if the individual corresponding to the index is a founder: both parents's ids are 0; false otherwise.
	 *@see seasrchIndex()
	 */
	virtual int isFounder(int index)=0; //1 if both paternal id and maternal id are 0; 0 otherwise

	/**
	 *@param index index of list of PedMeioInfo objects
	 *@return the PedMeioInfo of the index corresponding to an individual of interest
	 *@see seasrchIndex()
	 */
	virtual PedMeioInfo getPedMeio(int index)=0; //get the

	/**
	 *@post output the entire PedMeioInfo in the order the objects are added
	 */
	virtual void printPed()=0;

	/**
	 *@post id of the individual
	 *@return the internal index of the ID being queried.
	 */
	virtual int searchIndex(string ID)=0; //returns the first ID matches.. return -1 if ID cannot be matched.

	virtual ~Ped() {
		// cout<<"Destructor : Base"<<endl;
	}
};

/**
 * Implementation of the Pedigree class
 *
 * @pre always assume that the maternal meiosis has a lower number than paternal meiosis.
 * limitation is that it sets pedigree size to be PEDSIZE uniformly -
 * because we won't know the pedigree size before hand, we can't allocate
 * the exact memory for pedigree..
 * current implementation uses a dynamic array - not a linked list
 */
class Ped_Imp: public Ped {
private:
	PedMeioInfo *pedMeio;
	// PedMeioInfo *pedInfo; //dynamic array -
	int PED_SIZE;
public:

	Ped_Imp() {
		n_ind = 0;
		n_Meio = 0;
		startMeio = -1;
		PED_SIZE = MAX_NUM_PEOPLE_IN_PED;
		pedMeio = new PedMeioInfo[PED_SIZE];
	}

	void addMember(PedMeioInfo info) {
		pedMeio[n_ind].compid = info.compid;
		pedMeio[n_ind].id = info.id;
		pedMeio[n_ind].matMeio = info.matMeio;
		pedMeio[n_ind].mid = info.mid;
		pedMeio[n_ind].patMeio = info.patMeio;
		pedMeio[n_ind].pid = info.pid;

		if (pedMeio[n_ind].matMeio != 0) {
			n_Meio++;
			if (startMeio == -1) {
				startMeio = pedMeio[n_ind].matMeio;
			}
		}

		if (pedMeio[n_ind].patMeio != 0) {
			n_Meio++;
			if (startMeio == -1) {
				startMeio = pedMeio[n_ind].patMeio;
			}
		}
		//cout<<"Start meiosis is " <<startMeio<<endl;
		//cout <<"id: " <<pedMeio[n_ind].id <<endl;
		//cout<<"num meiosis is "<<n_Meio <<endl;
		//	cout <<"adding: " <<pedMeio[n_ind].id << endl ;
		n_ind++;
		//cout <<"num ind is "<<n_ind <<endl;
	}

	int isFounder(int index) {
		if ((pedMeio[index].pid.compare("0") == 0)
				&& (pedMeio[index].mid.compare("0") == 0)) {
			return (1);
		}
		return (0);
	}

	PedMeioInfo getPedMeio(int index) {

		return (pedMeio[index]);
	}

	void printPed() {

		cout << "ID\tPID\tMID\tCOMPID\tPATMEIO\tMATMEIO\n";
		int i;
		for (i = 0; i < n_ind; i++) {
			cout << pedMeio[i].id << " " << pedMeio[i].pid << " "
					<< pedMeio[i].mid << " " << pedMeio[i].compid << " "
					<< pedMeio[i].patMeio << " " << pedMeio[i].matMeio << endl;
			//	 cout<< pedInfo[0].id <<endl;
			//	 cout <<"printing: " <<pedMeio[i].id << endl ;

		}

	}

	int searchIndex(string ID) {
		int i;
		for (i = 0; i < PED_SIZE; i++) {
			//if equal
			if (pedMeio[i].id.compare(ID) == 0) {
				return i;
			}
		}
		return -1;
	}

	~Ped_Imp() {
		delete[] pedMeio;
	}
};

/**
 * Class containing a list of pedigrees.
 * When the pedigree list is initialized, the list of pedigree objects are already initialized too (ie. a fixed memory size has been allocated.) - it isn't the best way to initialize these objects... but it won't eat away much memory either.
 */
class PedList {
public:
	int n_ped;

	/**
	 * Get the Pedigree of interest
	 * @param index the index of pedigree of interest
	 * @return the pedigree corresponding to the index
	 */
	virtual Ped *getPed(int index)=0;

	/**
	 * determine whether a new pedigree can be added to the pedigree list
	 * @return
	 */
	virtual int canAddPed()=0;
	virtual ~PedList() {
	}

private:
	virtual void PedList_INIT()=0;
};

/**
 * An implementation of a list of pedigrees
 *
 * One limitation of this implementation is that it pre-specifies
 * the maximum of pedigree when this class is initiated PedList_INIT
 */
class PedList_Imp: public PedList {
public:
	PedList_Imp(int MAX_N) {
		n_ped = 0;
		MAX_N_PED = MAX_N;
		PedList_INIT();
	}

	Ped *getPed(int index) {
		return (&list[index]);
	}

	int canAddPed() {
		int tmp = n_ped;

		if ((tmp + 1) <= MAX_N_PED) {
			n_ped++;
			return (1);
		}
		return 0; //exceed capacity of the number of pedigrees in this implementation
	}

	~PedList_Imp() {
		delete[] list;
	}

private:
	Ped_Imp *list;
	//int *list;
	int MAX_N_PED;
	void PedList_INIT() {
		//MAX_N_PED = MAX_N;
		list = new Ped_Imp[MAX_N_PED];
	}

};

/**
 * Abstract class containing the Meiosis Inheritance Matrix. The data is specified by the number of locus and number of meioses.
 * MIM_INIT should be called to initiate instances of the data.
 *
 * MIM cells should contain either 0 or 1, specifying whether the meiosis is inherited from father or mother.  .. mother or father?
 */
class MIM {
public:
	int n_loci;
	int n_meiosis;

	/**
	 * initialize the MIM storage object.
	 * @param n_me number of meioses
	 * @param n_lo number of locus
	 */
	virtual void MIM_INIT(int n_me, int n_lo)=0;

	/**
	 * @param value the value to be written to the MIM - should be just 0 or 1 (although I didn't have a check for possible permittable value)
	 * @param m_index the meiosis index to be written to
	 * @param l_index the locus index to be written to
	 * @post the value is written to the appropriate cell of the matrix
	 */
	virtual void write2Matrix(int value, int m_index, int l_index) =0; //something to do with int **fgl;

	/**
	 * @param m_index the meiosis index to query
	 * @param l_index the locus index to query
	 * @return the value of the cell of the matrix
	 */
	virtual int getValue(int m_index, int l_index)=0;
	virtual ~MIM() {
	}
};

/**
 * Abstract class containing the list of MIM.. *in the MCMC dump, we may have many iterations (realizations) of that
 *
 */
class MIM_List {
public:
	long n_Iterations;

	/**
	 * @param index the iteration index
	 * @return pointer to the MIM corresponding to the specific index of interest
	 */
	virtual MIM *getMatrix(long index)=0;
	virtual ~MIM_List() {
	}
};

// x
class MIM_int: public MIM {
private:
	//data structure
	int **mim;
public:
	MIM_int() {
	}
	void MIM_INIT(int n_me, int n_lo) { // default constructor
		n_loci = n_lo;
		n_meiosis = n_me;
		mim = new int*[n_meiosis];
		int i = 0;
		for (i = 0; i < n_meiosis; i++) {
			mim[i] = new int[n_loci];
		}
	}

	void write2Matrix(int value, int m_index, int l_index) {
		mim[m_index][l_index] = value;
	}

	int getValue(int m_index, int l_index) {
		return (mim[m_index][l_index]);
	}
	~MIM_int() {
		//	cout<<"run...\n";
		for (int i = 0; i < n_meiosis; i++) {
			delete[] mim[i];
			//mim[i]=0;
		}
		delete[] mim;
		//mim=0;
	}

};

class MIM_List_int: public MIM_List {
private:
	MIM_int *list;

public:
	MIM_List_int(long n, int m, int l) {
		n_Iterations = n;
		list = new MIM_int[n_Iterations];
		long i = 0;
		for (i = 0; i < n_Iterations; i++) {
			list[i].MIM_INIT(m, l);
		}
	}

	MIM *getMatrix(long index) {
		return (&list[index]);
	}

	~MIM_List_int() {
		delete[] list;
	}

};

/**
 * FGL_Matrix is an abstract class storing the genome founder labels. The data is specified by
 * #loci x #individuals x allele 0 or allele 1, as each individual should have a
 * pair of genome founder label at every loci.
 *
 *FGL_Matrix_INIT should be called to initiate the data.
 */
class FGL_Matrix {
public:
	int n_loci; ///< number of loci.
	int n_ind; ///< number of meiosis

	/**
	 * @param n_ind the number of individual to be initialized
	 * @param n_lo the number of locus to be initialized
	 * @post the FGL Matrix data structure containing n individual, n locus, 2 alleles each for each individual locus is initialized t0 0.  //addition on Feb6, 2009 to initialize to 0.
	 */
	virtual void FGL_Matrix_INIT(int n_ind, int n_lo)=0;

	/**
	 * @param value the value of the matrix to be written
	 * @param ind_index index of the individual
	 * @param a_index should be either 0 or 1
	 * @param l_index index of the locus
	 * @return write the appropriate value to the appropriate cell
	 */
	virtual void write2Matrix(int value, int ind_index, int a_index,
			int l_index) =0; //something to do with int **fgl;
	//virtual int foo (void) =0; //something to do with int **fgl;

	/**
	 * @param ind_index index of the individual
	 * @param a_index should be either 0 or 1
	 * @param l_index index of the locus
	 * @return read the value of the cell of interest
	 **/
	virtual int getValue(int ind_index, int a_index, int l_index)=0;

	virtual ~FGL_Matrix() {
		// cout<<"Destructor : Base"<<endl;
	}
};

/**
 * Implementation of FGL_Matrix.
 */
class FGL_Matrix_int: public FGL_Matrix {
private:
	//data structure
	int **fgl;
public:
	FGL_Matrix_int() {
	}
	void FGL_Matrix_INIT(int n_ppl, int n_lo) { // default constructor
		n_loci = n_lo;
		n_ind = n_ppl;
		int n_alleles = n_ind * 2;
		fgl = new int*[n_alleles];
		int i = 0;
		for (i = 0; i < n_alleles; i++) {
			fgl[i] = new int[n_loci];
			//feb6, 2009 addition -- initialize vector to 0s
			for (int j = 0; j < n_loci; j++) {
				fgl[i][j] = 0;
			}

		}
	}

	void write2Matrix(int value, int ind_index, int a_index, int l_index) {
		if ((a_index > 1) || (a_index < 0)) {
			printf("allele index must be 0 or 1");
		}
		fgl[ind_index * 2 + a_index][l_index] = value;
	}

	int getValue(int ind_index, int a_index, int l_index) {
		if ((a_index > 1) || (a_index < 0)) {
			printf("allele index must be 0 or 1");
		}
		return (fgl[ind_index * 2 + a_index][l_index]);
	}
	~FGL_Matrix_int() {
		int n_alleles = n_ind * 2;
		for (int i = 0; i < n_alleles; i++) {
			delete[] fgl[i];
		}
		delete[] fgl;

	}
};

/**
 * An abstract class containing many iterations of Founder Genome Labels.
 */
class FGL_Matrix_List {
public:
	int n_Iterations;
	/**
	 * @return a pointer to the FGL_Matrix so we can manipulate through that
	 */
	virtual FGL_Matrix *getMatrix(int index)=0;
	virtual int writeMatrix(int index, FGL_Matrix *fgl)=0;

	//virtual ~FGL_Matrix_List()=0; // THIS WILLL GIVE A SYNTAX ERROR.. use below will work
	virtual ~FGL_Matrix_List() {
		// cout<<"Destructor : Base"<<endl;
	}

};

/**
 * An implementation of the list of genome founder label matrices
 */
class FGL_Matrix_List_int: public FGL_Matrix_List {
private:
	FGL_Matrix_int *list;

public:
	FGL_Matrix_List_int(int n, int n_ind, int l) {
		n_Iterations = n;
		list = new FGL_Matrix_int[n_Iterations];
		int i = 0;
		for (i = 0; i < n_Iterations; i++) {
			list[i].FGL_Matrix_INIT(n_ind, l);
		}
	}

	FGL_Matrix *getMatrix(int index) {
		return (&list[index]);
	}

	int writeMatrix(int index, FGL_Matrix *fgl) {
		if ((list[index].n_ind == fgl->n_ind) && (list[index].n_loci
				== fgl->n_loci)) {

			for (int i = 0; i < list[index].n_ind; i++) {
				for (int j = 0; j < list[index].n_loci; j++) {
					list[index].write2Matrix(fgl->getValue(i, 0, j), i, 0, j);
					list[index].write2Matrix(fgl->getValue(i, 1, j), i, 1, j);
				}
			}

			return (1);
		} else {
			return (0);
		}

	}

	~FGL_Matrix_List_int() {
		delete[] list;
	}

};

/**
 * Generic class of the IBD-related matrices.
 * This is a memory-efficient way [n*(n-1)/2 cells needed] of storing the top triangle of a SYMMETRIC matrix in
 * which we have a common value in the diagonal element.
 * We can reproduce the full matrix.
 */
class TopTri_Matrix {
public:
	int n_ind;

	/**
	 * initialize the matrix
	 */
	void TopTri_Matrix_INIT(int n, double diag) {
		n_ind = n;
		diagValue = diag; //by default
		IBD = new double[n_ind * (n_ind - 1) / 2];
	}

	/**
	 * change the diagonal value of the matrix
	 */
	void changeDiagValue(double value) {
		diagValue = value;
	}

	/**
	 * @pre Requires  i (row index) to be less than  j (column index) (strictly)
	 * @post write the value to the appropriate cell of interest
	 */
	void writeMatrix(double value, int i, int j) {
		if (i >= j) {
			printf("i must be smaller than or larger than j. Do nothing.\n");
		} else {
			IBD[i * n_ind - i * (i + 1) / 2 + (j - i) - 1] = value;
		}

	}

	/**
	 * @return read the matrix of the cell of interest
	 */
	double readMatrix(int i, int j) {
		if (i < j) {
			return (IBD[i * n_ind - i * (i + 1) / 2 + (j - i) - 1]);
		} else if (i == j) {
			return (diagValue); //?
		} else {
			return (IBD[j * n_ind - j * (j + 1) / 2 + (i - j) - 1]);
		}
	}

	~TopTri_Matrix() {
		delete[] IBD;
	}
protected:
	double *IBD; //efficient storage.
private:
	double diagValue; //actually this should be a private variable, since I can change the value here...

};

//=================================================================================================


/**
 * Read the Pedigree info data from a file and create an instance of PedList containing a list of pedigree data
 * stored in the Ped class.
 * @pre Note it assumes the data to be read is already sorted.
 * @pre ie.  assume component starts with 1.
 * @pre ie. assume top down pedigree data (sorted) & meiosis labels in consecutive order.
 * @pre ie. assumes maternal meiosis number is less than paternal meiosis number if a given individual has both non-zero maternal and paternal meiosis
 * @pre sorted by component and meiosis ordering. (Note: name (ID) does not have to be sorted.)

 * num_Ped should be known.
 *
 * The file format to be read in should be in the format:
 *
 name      name.pa      name.ma Compnt pat.meio mat.meio
 1             0              0       1      0      0
 2             0              0       1      0      0
 3              1              2      1      2      1
 4              1              2      1      4      3
 *
 *
 * @param pedigreeFile the file name of the pedigree meiosis file
 * @param num_Ped the number of pedigrees to be read in
 * @return the pointer to the PedList data which contains the pedigree meiosis information read in from the pedigree meiosis file
 */
PedList *readPedigrees(const char *pedigreeFile, int num_Ped) {

	char buffer[100]; // One extra for null char.
	ifstream inFile;
	inFile.open(pedigreeFile);
	if (!inFile) {
		cout << "Unable to open the pedigree file";
		exit(1); // terminate with error
	}

	PedList_Imp *myPedList = new PedList_Imp(num_Ped); //should be PedList

	//Reading file:
	//header
	inFile >> buffer;
	//cout<<buffer <<endl;
	inFile >> buffer;
	inFile >> buffer;
	inFile >> buffer;
	inFile >> buffer;
	inFile >> buffer;
	//cout <<buffer <<endl;
	//
	int cur = 0;
	Ped *curPed;
	int withinCapacity = 0;

	PedMeioInfo tmpInfo;
	//cout << "list of individuals to read in:\n";
	while (inFile >> tmpInfo.id >> tmpInfo.pid >> tmpInfo.mid >> tmpInfo.compid
			>> tmpInfo.patMeio >> tmpInfo.matMeio) {
		//cout<<"RUN \n";
		//inFile >> tmpInfo.id;
		//inFile >> tmpInfo.pid;
		//inFile >> tmpInfo.mid;
		//inFile >> tmpInfo.compid;
		//inFile >> tmpInfo.patMeio;
		//inFile >> tmpInfo.matMeio;
//		cout << tmpInfo.id << "\t";
		//try adding a pedigree to the pedigree list
		//if new family is present
		if (tmpInfo.compid != cur) { //assume the file component is ordered 1,2,3,4,5,6,..
			//cout <<"cur is " <<cur <<endl;
			//cout <<"tmp compid is " <<tmpInfo.compid <<endl;
			withinCapacity = myPedList->canAddPed(); //try adding it to the new pedigree
			if (withinCapacity) {
				curPed = myPedList->getPed(cur);
				cur++; //from now on is the next number up, so the next line from the same pedigree is read, the above two if statements will not be run, but the below if statement will be run instead.
			}
		}
		if (withinCapacity) {
			curPed->addMember(tmpInfo);

		} else {
			cout << "Pedigree capacity exceeded....\n";
			exit(0);
		}

	}

	inFile.close();
//	cout << "\n";
	return (myPedList);
}

/**
 * check that the appropriate number of lines are read
 * Read the raw IV file component by component  - it stores the compressed IVs
 */
OSCOR *readOSCOR_byComponent(const char *OSCORFile, long num_Iterations,
		int num_Meiosis, ifstream *oscorStream, long *readPos) {

	//cout<<"Before allocation.\n";
	OSCOR *oscor = new OSCOR(num_Iterations, num_Meiosis);
	//read file and expand into MIM_List
	//int firstIteration = 1;
	int ind_Index;
	int cur;
	int m_cur = 0;
	int index = 0;
	MIM *curMIM;
	int tmp, firstPos, numSwitch, switchCur;
	int *markerPos;
	long totalLines = num_Meiosis * num_Iterations;

	oscorStream->seekg(*readPos, ios::beg);
	long curPos = 0;
	for (cur = 0; cur < totalLines; cur++) {
		if (m_cur == num_Meiosis) {
			index++; //iteration index
			m_cur = 0;
			//firstIteration = 0;
		}

		//Read line
		*oscorStream >> ind_Index;
	//cout<<cur+1 <<":" << ind_Index <<endl;
		*oscorStream >> tmp;
		*oscorStream >> firstPos;
		*oscorStream >> numSwitch;
		markerPos = new int[numSwitch];
		for (switchCur = 0; switchCur < numSwitch; switchCur++) {
			*oscorStream >> markerPos[switchCur];
		}
		//writeLine
		oscor->writeLine(index, m_cur, firstPos, numSwitch, markerPos);
		m_cur++;
		//	cout<<"Position: " << inFile.tellg() <<endl;
		//free memory:
		delete[] markerPos;
		curPos = oscorStream->tellg();
		if(curPos == -1){
			cout<<"The number of lines in this file ("<<cur <<") is less than "<<num_Iterations <<"*"<<num_Meiosis
					<<"=" <<num_Iterations*num_Meiosis <<" (#Sampled IVs*#Meioses).\n";
			cout<<"Are you sure that the number of sampled IVs is correct?\n";
			cout<<"Exit with error.\n";
			exit(0);
		}
	}

	*readPos = oscorStream->tellg();
	return (oscor);
}

//============================================================================================================================

/**
 * The old obsolete file to read in the IV file
 */
OSCOR *readOSCOR(const char *OSCORFile, long num_Iterations, int num_Meiosis) {

	ifstream inFile;
	inFile.open(OSCORFile);
	if (!inFile) {
		cout << "Unable to open the meiosis indicator matrix file";
		exit(1); // terminate with error
	}
	//cout<<"Before allocation.\n";
	OSCOR *oscor = new OSCOR(num_Iterations, num_Meiosis);
	//read file and expand into MIM_List
	//int firstIteration = 1;
	long readPos = 0;
	int ind_Index;
	int cur, value;
	int m_cur = 0;
	int index = 0;
	MIM *curMIM;
	int tmp, firstPos, numSwitch, switchCur;
	int *markerPos;
	long totalLines = num_Meiosis * num_Iterations;

	inFile.seekg(readPos, ios::beg);

	for (cur = 0; cur < totalLines; cur++) {
		if (m_cur == num_Meiosis) {
			index++; //iteration index
			m_cur = 0;
			//firstIteration = 0;
		}

		//Read line
		inFile >> ind_Index;
		inFile >> tmp;
		inFile >> firstPos;
		inFile >> numSwitch;
		markerPos = new int[numSwitch];
		for (switchCur = 0; switchCur < numSwitch; switchCur++) {
			inFile >> markerPos[switchCur];
		}

		//writeLine
		oscor->writeLine(index, m_cur, firstPos, numSwitch, markerPos);

		m_cur++;
		//	cout<<"Position: " << inFile.tellg() <<endl;
		//free memory:
		delete[] markerPos;

	}

	readPos = inFile.tellg();

	inFile.close();
	return (oscor);
}

//========================================================================================

//readMIM_partial
/**
 * This is a modification of the readMIM and allow us to read only the MIM of a few positions...
 *
 * Read the MIM file and return a LIST (iterations) of the Meiosis Indicator Matrices
 * the MIM list should be specified by num_Iterations*number of meiosis*number of markers(locus)
 * @pre It assumes that the MIM file is sorted BY COMPONENT. The PedList data should be of the right format, which means that the original pedigree meiosis file must contain the right format. Also see format.
 * @see readPedigrees()
 *
 * @param MIMFile - the Meiosis indicator matrix file to be processed.
 * @param readPos - starting position [not the starting line number] of the file to be read.
 * @param markersToKeep array must be sorted - marker index starts at 1, which corresponds to keeping the first marker in the data; the first marker does not start at 0.
 * @param keep_N_Markers should correspond to the length of markersToKeep
 *
 * @return a pointer to the MIM_List data AND the read position after processing through the component-/pedigree-related information
 *
 * The FORMAT of the data to read in should conform to the format:
 * no header. each row contains several space-delimited columns:
 *
 * For each row:
 * individual index	first unlinked marker (to be ignored)	first marker meiosis	number of switches	recombination positions
 *
 ** ie.
 1 0 1 10
 1 1 2 14 18
 0 0 1 7
 0 1 0
 * ...
 *
 * @format of the MIM file
 * The organization of the rows proceeds as follows:
 * Component 1 Iteration 1 Meiosis 1
 * Component 1 Iteration 1 Meiosis 2
 * ...
 * Component 1 Iteration 1 Meiosis m1 (assume component 1 contains a total of m1 meiosis)
 * Component 1 Iteration 2 Meiosis 1
 * Component 1 Iteration 2 Meiosis 2
 * ...
 * Component 1 Iteration 1 Meiosis m1
 * ...
 * ...
 * Component 1 Iteration N Meiosis m1 (assume EACH component contains a total of N iterations)
 * Component 2 Iteration 1 Meiosis 1
 * Component 2 Iteration 1 Meiosis 2
 * ...
 *
 * ..
 * ...
 * Component k Iteration N Meiosis mk (assume the file contains a total of k pedigrees)
 *
 */

// assume the next set of index we are interseted in are to the right of the previous set. Only the last index of the old set is allowed to overlap with the new set
//MIM_List *readMIM_partial(OSCOR *oscor, int *markersToKeep, int keep_N_Markers, long num_Iterations, int num_Meiosis, int num_markers, MIM_List *latestMIM) {
MIM_List *readMIM_partial(OSCOR *oscor, long num_Iterations, int num_Meiosis,
		int readNMarkers, MIM_List *previousMiml, int new_startingIndex,
		int old_startingIndex) {
	//cout <<"old starting index " << old_startingIndex <<endl;
	//cout <<"new starting index " << new_startingIndex <<endl;
	int isBegin = 0;
	if (previousMiml == NULL) {
		isBegin = 1;
		previousMiml = new MIM_List_int(num_Iterations, num_Meiosis, 1);
		for (int cur_it = 0; cur_it < num_Iterations; cur_it++) {
			MIM *previousMIM = previousMiml->getMatrix(cur_it);
			for (int cur_m = 0; cur_m < num_Meiosis; cur_m++) {
				int value = oscor->getInitialValue(cur_it, cur_m);
				previousMIM->write2Matrix(value, cur_m, 0);
			}
		}

	}

	MIM_List *miml =
			new MIM_List_int(num_Iterations, num_Meiosis, readNMarkers);
	//read file and expand a specific region into MIM_List
	int lastMarker = previousMiml->getMatrix(0)->n_loci - 1; //the index of the last marker in the MIM_List structure

	int old_endingIndex = old_startingIndex + lastMarker;

	if (old_endingIndex > new_startingIndex) {
		cout
				<< "Error. The last index of the previous miml is not allowed to be larger than the first index of the new miml in this implementation.\n";
		exit(0);
	}

	//	for(int i=0; i < num_Iterations; i++){
	//		for(int j=0; j<num_Meiosis; j++){
	//			cout << latestMiml->getMatrix(i)->getValue(j, 0) <<" ";
	//		}
	//		cout <<endl;
	//	}


	//cout << "lastMarker " <<lastMarker <<endl;

	//cout << "old ending index " << old_endingIndex <<endl;
	//cout << "new starting index " << new_startingIndex <<endl;
	//cout << "readNMarkers " << readNMarkers <<endl;

	for (int cur_it = 0; cur_it < num_Iterations; cur_it++) {
		MIM *myMIM = miml->getMatrix(cur_it);
		MIM *previousMIM = previousMiml->getMatrix(cur_it);

		for (int cur_m = 0; cur_m < num_Meiosis; cur_m++) {
			int numRecomb = oscor->getNRecomb(cur_it, cur_m);
			int *recombPos = oscor->getRecombPos(cur_it, cur_m);
			int value = previousMIM->getValue(cur_m, lastMarker);
			//start reading
			int cur_read = 0;
			//if the first index of the new miml overlaps with the last index of the previous miml
			if (new_startingIndex == old_endingIndex) {
				myMIM->write2Matrix(value, cur_m, cur_read);
				cur_read++;
			}
			int cur_index = old_endingIndex;
			while (cur_read < readNMarkers) {
				cur_index++; //probe the next one
				for (int r = 0; r < numRecomb; r++) {
					if ((cur_index + 1) == recombPos[r]) {
						//if( (cur_index+1) == recombPos[r]){
						//flip inheritance vector
						if (value == 0) {
							value = 1;
						} else {
							value = 0;
						}
						break;
					}
				} //r
				//check whether in range of interseted position:
				if (cur_index >= new_startingIndex) {
					myMIM->write2Matrix(value, cur_m, cur_read); //write value
					cur_read++;
				}
			}//while still have markers to read
		}//cur_m
	}//cur_it


	//if I have to create the Miml at the first position, I would delete it. Otherwise, I don't want to delete the object
	if (isBegin) {
		delete previousMiml;
	}

	return (miml);

}

//=========================================================================================


/**
 * Read the MIM file and return a LIST (iterations) of the Meiosis Indicator Matrices
 * the MIM list should be specified by num_Iterations*number of meiosis*number of markers(locus)
 * @pre It assumes that the MIM file is sorted BY COMPONENT. The PedList data should be of the right format, which means that the original pedigree meiosis file must contain the right format. Also see format.
 * @see readPedigrees()
 *
 * @param MIMFile - the Meiosis indicator matrix file to be processed.
 * @param readPos - starting position [not the starting line number] of the file to be read.
 * @param markersToKeep array must be sorted - marker index starts at 1, which corresponds to keeping the first marker in the data; the first marker does not start at 0.
 * @param keep_N_Markers should correspond to the length of markersToKeep
 *
 * @return a pointer to the MIM_List data AND the read position after processing through the component-/pedigree-related information
 *
 * The FORMAT of the data to read in should conform to the format:
 * no header. each row contains several space-delimited columns:
 *
 * For each row:
 * individual index	first unlinked marker (to be ignored)	first marker meiosis	number of switches	recombination positions
 *
 ** ie.
 1 0 1 10
 1 1 2 14 18
 0 0 1 7
 0 1 0
 * ...
 *
 * @format of the MIM file
 * The organization of the rows proceeds as follows:
 * Component 1 Iteration 1 Meiosis 1
 * Component 1 Iteration 1 Meiosis 2
 * ...
 * Component 1 Iteration 1 Meiosis m1 (assume component 1 contains a total of m1 meiosis)
 * Component 1 Iteration 2 Meiosis 1
 * Component 1 Iteration 2 Meiosis 2
 * ...
 * Component 1 Iteration 1 Meiosis m1
 * ...
 * ...
 * Component 1 Iteration N Meiosis m1 (assume EACH component contains a total of N iterations)
 * Component 2 Iteration 1 Meiosis 1
 * Component 2 Iteration 1 Meiosis 2
 * ...
 *
 * ..
 * ...
 * Component k Iteration N Meiosis mk (assume the file contains a total of k pedigrees)
 *
 */
MIM_List *readMIM(const char *MIMFile, int *markersToKeep, int keep_N_Markers,
		long *readPos, long num_Iterations, int num_Meiosis, int num_markers) {

	ifstream inFile;
	inFile.open(MIMFile);
	if (!inFile) {
		cout << "Unable to open the meiosis indicator matrix file";
		exit(1); // terminate with error
	}
	//cout<<"Before allocation.\n";
	MIM_List *miml = new MIM_List_int(num_Iterations, num_Meiosis,
			keep_N_Markers);
	//read file and expand into MIM_List
	//cout<<"After allocation.\n";

	//cout<<"Position: " << inFile.tellg() <<endl;
	//int curInd =0;
	int ind_Index;
	int cur, value;
	int m_cur = 0;
	int index = 0;
	MIM *curMIM;
	int tmp, firstPos, numSwitch, switchCur;
	int *markerPos;
	long totalLines = num_Meiosis * num_Iterations;

	//cout << totalLines << endl;
	inFile.seekg(*readPos, ios::beg);
	//for(cur=0; cur< 20; cur++){
	for (cur = 0; cur < totalLines; cur++) {
		//cout<<cur <<" ";
		if (m_cur == num_Meiosis) {
			index++; //iteration index
			m_cur = 0;
		}
		curMIM = miml->getMatrix(index);

		//Read line
		inFile >> ind_Index;
		inFile >> tmp;
		inFile >> firstPos;
		inFile >> numSwitch;
		markerPos = new int[numSwitch];
		//cout <<ind_Index <<" ";
		//cout << tmp <<" " <<firstPos <<" " <<numSwitch <<": ";
		for (switchCur = 0; switchCur < numSwitch; switchCur++) {
			inFile >> markerPos[switchCur];
			//cout <<markerPos[switchCur] <<" ";
		}
		//cout <<endl;

		//===============VALIDITY CHECK==============================//
		//expect a bunch of the same number because the first meiosis
		//should be the same for each component
		//		if(m_cur ==0){
		//			cout<< ind_Index << " ";
		//		}
		//===========================================================//
		//Write MIM
		int curKeepIndex;
		int curMarkerPos = 0;
		for (curKeepIndex = 0; curKeepIndex < keep_N_Markers; curKeepIndex++) {
			//	cout <<markersToKeep[curKeepIndex] <<endl;


			while (curMarkerPos < numSwitch) {
				if (markersToKeep[curKeepIndex] >= markerPos[curMarkerPos]) {
					curMarkerPos++;
				} else {
					break;
				}
			}

			value = curMarkerPos % 2;

			//flip bit
			if (firstPos == 1) {
				value = (value + 1) % 2;
			}

			curMIM->write2Matrix(value, m_cur, curKeepIndex);

			//cout << curMIM->getValue(m_cur, curKeepIndex) <<" ";
		}
		//cout << endl;


		m_cur++;
		//	cout<<"Position: " << inFile.tellg() <<endl;

		//free memory:
		delete[] markerPos;

	}

	//cout<<"After reading.\n";

	*readPos = inFile.tellg();
	//cout <<"New position: " << *readPos <<endl;


	/*
	 inFile >> tmp;
	 inFile >> firstPos;
	 inFile >> numSwitch;
	 cout << tmp <<" " <<firstPos <<" " <<numSwitch <<endl;
	 inFile >>tmp;
	 cout <<tmp;
	 */

	inFile.close();
	return (miml);

}

FGL_Matrix_List *readFGL(const char *FGLFILE, int *markersToKeep,
		int keep_N_Markers, long *readPos, long num_Iterations,
		int num_markers, Ped *pedigree, int *startLabel) {

	cout << " readFGL....\n";

	//Because the input right now also outputs the meiosis that was omitted in the Meiotic indicator file, the number of meiosis is defined to be the #of nonfounder *2.
	//note that if the consistent way is used, the assignment of founder labels would be more complicated, as the ones who have only one meiosis in the output should get the meiosis from an arbitrary meiosis from his/her parent.
	int num_Meiosis = 0;
	int num_Founders = 0;
	for (int i = 0; i < pedigree->n_ind; i++) {
		if (pedigree->isFounder(i) == 0) {
			num_Meiosis += 1;
		} else {
			num_Founders += 1;
		}
	}
	num_Meiosis = num_Meiosis * 2;
	//

	//declare variable:
	int pedSize = pedigree->n_ind;
	int curIteration;
	int cur, curLocus;
	int curLabel = 0; //keep track of the current label to be assigned to the founders
	int curMeiosis = pedigree->startMeio;
	int startMeiosis = pedigree->startMeio; //the starting meiosis number of the pedigree.
	int meiosisValue, fglValue;

	//Read file
	ifstream inFile;
	inFile.open(FGLFILE);
	if (!inFile) {
		cout << "Unable to open FGL file";
		cout << FGLFILE << endl;
		exit(1); // terminate with error
	}
	//cout<<"Before allocation.\n";
	cout << "# ind: " << pedigree->n_ind << endl;
	cout << "# meiosis: " << num_Meiosis << endl;

	//create a fgl object with #iterations, # people in pedigree, and # loci
	FGL_Matrix_List *gml = new FGL_Matrix_List_int(num_Iterations,
			pedigree->n_ind, keep_N_Markers);

	//read file and expand into MIM_List
	//cout<<"After allocation.\n";

	//cout<<"Position: " << inFile.tellg() <<endl;
	//int curInd =0;
	string ind_Index;
	int value;
	int m_cur = 0;
	int index = 0;
	MIM *curMIM;
	int tmp, firstPos, numSwitch, switchCur;
	int *markerPos;
	int *FGLswitch;
	long totalLines = num_Meiosis * num_Iterations;

	//cout << totalLines << endl;
	inFile.seekg(*readPos, ios::beg);
	//for(cur=0; cur< 20; cur++){

	//first iteration
	FGL_Matrix *fglm = gml->getMatrix(0);

	//setup the structure which keeps track of the meiosis which were entered
	int meiosis[pedigree->n_ind];
	int meiCur = 0;
	//initialize
	for (meiCur = 0; meiCur < pedigree->n_ind; meiCur++) {
		meiosis[meiCur] = 0;
	}

	for (cur = 0; cur < totalLines; cur++) {

		if (m_cur == num_Meiosis) {
			index++; //iteration index
			m_cur = 0; //reset cur indicator
			fglm = gml->getMatrix(index); //get the next iteration
			//initialize
			for (meiCur = 0; meiCur < pedigree->n_ind; meiCur++) {
				meiosis[meiCur] = 0;
			}

		}

		//Read line
		inFile >> ind_Index;
		inFile >> tmp;
		inFile >> firstPos;
		inFile >> numSwitch;
		markerPos = new int[numSwitch + 1];
		FGLswitch = new int[numSwitch + 1];
		//num switch is the last index of the arrays above..

		markerPos[0] = 1;
		FGLswitch[0] = firstPos;

		//cout <<ind_Index <<" ";
		//cout << tmp <<" " <<firstPos <<" " <<numSwitch <<": ";
		for (switchCur = 1; switchCur < (numSwitch + 1); switchCur++) {
			inFile >> markerPos[switchCur];
			inFile >> FGLswitch[switchCur];
			//cout <<markerPos[switchCur] <<" ";
			//cout <<FGLswitch[switchCur] <<" ";
		}

		//cout <<endl;

		//...................................//


		//if (cur ==0){
		//cout <<ind_Index <<" ";
		//cout << tmp <<" " <<firstPos <<" " <<numSwitch <<": ";
		//}

		//Approach:

		//after this, fill in the empty cells. check which meiosis are missing.  find the first. starting assign new labels?
		// fill in the rest, if isn't from the new founders.. errors


		//1. check name. find person's index.
		//cout<<"Searching for Index: " <<ind_Index <<endl;
		int pedIndex = pedigree->searchIndex(ind_Index);

		//2. get FGLs values; write locus
		//marker counting begins at 1, actually.
		int curLo = 0;
		int expandFGL[num_markers];
		int curSwitch = 0;
		for (curLo = 0; curLo < num_markers; curLo++) {
			if (curSwitch < numSwitch) {
				if (curLo >= (markerPos[curSwitch + 1] - 1)) {
					curSwitch++;
				}
			}
			expandFGL[curLo] = FGLswitch[curSwitch];
			// cout <<expandFGL[curLo] <<" ";
		}
		//cout <<endl;

		//cout<<"pedIndex is " <<pedIndex <<endl;
		int meiosisIndex = meiosis[pedIndex];
		meiosis[pedIndex]++;

		for (curLo = 0; curLo < keep_N_Markers; curLo++) {
			// -1 because the input marker index starts at 1 instead of 0.
			//cout << markersToKeep[curLo] <<", " <<pedIndex << ", " <<meiosisIndex <<", " << curLo <<endl;


			//for checking purpose only
			//       	if((meiosisIndex !=0)&&(meiosisIndex!=1)){
			//           cout<<markersToKeep[curLo] <<", " <<pedIndex <<", " <<meiosisIndex <<", " <<curLo <<endl;
			//        exit(0);
			//        }


			fglm->write2Matrix(expandFGL[markersToKeep[curLo] - 1], pedIndex,
					meiosisIndex, curLo);
		}

		//exit(0);
		//..........//

		m_cur++;

		//cout<<cur <<" ";
		if (m_cur == num_Meiosis) {
			//check progress:
			//cout<<"=====================\n";
			int curLabel = *startLabel;

			//cout<<"CURLABEL: "<<*startLabel <<endl;
			for (int i = 0; i < pedigree->n_ind; i++) {
				PedMeioInfo tmpID = pedigree->getPedMeio(i);
				for (int j = 0; j < 2; j++) {
					// cout<< tmpID.id <<":";
					int isImpute = 0;
					if (fglm->getValue(i, j, 0) == 0) {
						isImpute = 1;
					}
					for (int k = 0; k < keep_N_Markers; k++) {

						if (isImpute == 1) {
							fglm->write2Matrix(curLabel, i, j, k);
						}
						//  	cout<< fglm->getValue(i, j, k);
						//	cout<<",";
					}
					if (isImpute == 1) {
						curLabel++;
					}
					//	cout <<endl;
				}
			}
			//cout<<"=====================\n";

			//cout<<"bye\n";
			//

		}

		//	cout<<"Position: " << inFile.tellg() <<endl;

		//free memory:
		delete[] markerPos;
		delete[] FGLswitch;

	}
	cout << "After reading.\n";

	//exit(0);
	//cout<< num_Meiosis <<endl;

	//exit(0);

	*readPos = inFile.tellg();
	//formala for updating the next founder label:
	*startLabel = *startLabel + num_Founders * 2;

	//cout <<pedSize <<endl;
	//cout<<num_Meiosis <<endl;
	//cout << *startLabel <<endl;

	inFile.close();
	return (gml);

}

//assuming Haldane map function with no interference, odd number of recombinations takes the rate of Poisson(-2x)
//http://www.stat.washington.edu/thompson/Genetics/6.1_haldane.html
//-100 is the value used to indicate the flanking marker is unavailable
//NEED TO VERIFY WHETHER the d1== -100 case is right
//if d2 == -100, we only use the left flanking marker
//need to test this function.
double calcWeight(int m1, int m2, double d1, double d2, double pos) {
	double p;
	double x = (pos - d1) / 100;
	double y = (d2 - pos) / 100;
	double r_x = (1 - exp(-2 * x)) / 2;
	double r_y = (1 - exp(-2 * y)) / 2;
	double r_xy = (1 - exp(-2 * (x + y))) / 2;

	if ((d1 <= pos) && (pos < d2)) {
		if ((m1 == 0) && (m2 == 0)) {
			p = r_x * r_y / (1 - r_xy);
		} else if ((m1 == 0) && (m2 == 1)) {
			p = r_x * (1 - r_y) / (r_xy);
		} else if ((m1 == 1) && (m2 == 0)) {
			p = (1 - r_x) * r_y / (r_xy);
		} else if ((m1 == 1) && (m2 == 1)) {
			p = (1 - r_x) * (1 - r_y) / (1 - r_xy);
		} else {
			cout << "error. m1 and m2 should be 0 or 1.\n";
		}
	}
	//else if((d1<pos) && (d2 == -100)){
	else if (d2 == -100) {
		//cout<<"You reached the case where there is only the left flanking marker.\n";
		if (m1 == 0) {
			p = r_x;
		} else if (m1 == 1) {
			p = (1 - r_x);
		} else {
			cout << "error. m1 should be 0 or 1.\n";
		}
	} else if (d1 == -100) {
		if (m2 == 0) {
			p = r_y;
		} else if (m2 == 1) {
			p = (1 - r_y);
		} else {
			cout << "error. m2 should be 0 or 1.\n";
		}
	} else {
		cout << "error. " << m1 << ", " << m2 << ", " << d1 << ", " << d2
				<< ", " << pos
				<< ": the desired position should be between the position of the first and second marker. \n";

	}
	return p;
}

void print_estMeiosis(MIM_List *miml) {

	MIM *mim = miml->getMatrix(0);
	int num_loci = mim->n_loci;
	int num_meiosis = mim->n_meiosis;
	int num_iterations = miml->n_Iterations;

	for (int m = 0; m < num_meiosis; m++) {

		double recombination[num_loci];
		for (int i = 0; i < num_loci; i++) {
			recombination[i] = 0;
		}
		for (int it = 0; it < num_iterations; it++) {
			mim = miml->getMatrix(it);
			int cur = mim->getValue(m, 0);
			for (int loci = 0; loci < num_loci; loci++) {
				if (cur != mim->getValue(m, loci)) {
					cur = mim->getValue(m, loci);
					recombination[loci] += 1;
				}
			}
		}

		for (int i = 0; i < num_loci; i++) {
			recombination[i] = recombination[i] / num_iterations;
		}

		cout << "m" << m << " ";
		for (int l = 0; l < num_loci; l++) {
			cout << recombination[l] << " ";
		}
		cout << endl;
	}

}

/**
 * This function creates the MIM (IV) at an arbitrary position, given the framework IVs
 */
//1. inheritance pattern
//Original goal:
//for each person in the pedigree, and for each copy of this person's DNA, assign the probability that it comes from the maternal side (0) or paternal side (1) based on the available inheritance pattern informations (flanking inheritance pattern information)
//altogether, this would reconstruct the probability of the pattern of gene flow at position p
//use Monte Carlo Sampling to sample the inheritance pattern of each nonfounder using the probability of gene flow at position p for each person

//Implementation
//make the look up table of the probability of inheriting from 1 (father) given the marker is either (0,0), (0,1), (1,0), (1,1) based on the Haldane map function : 4 x # position matrix

//use Monte Carlo Sampling to sample the inheritance pattern of each nonfounder using the lookup table

MIM_List *MIM_pos(MIM_List *miml, int num_Iterations, int num_Pos,
		int num_Meiosis, MTRand *mtrand1, double *positions, double *markerPos,
		int numMarkers) {
	if (numMarkers <= 0) {
		cout << "number of markers should not be less than or equal to 0.\n";
		exit(0);
	}

	int num_Samples = num_Iterations;
	MIM_List *miml_pos = new MIM_List_int(num_Samples, num_Meiosis, num_Pos);

	MIM *curMIM;
	MIM *mim;

	// and used with
	//double a = mtrand1(); // ??????


	//Weight matrix
	double weightProb[4][num_Pos];
	double d1, d2;
	int markerIndex;
	markerIndex = 0;
	d2 = markerPos[markerIndex];
	for (int j = 0; j < num_Pos; j++) {

		//case: -2 flanking markers: d1 assigned , d2 assigned
		//      -only the left flanking marker: d1 assigned, d2 = -100
		//      - only the right flanking marker: d1=-100, d2 assigned
		//search for a position that is strictly to the right of the position of interest
		while ((d2 <= positions[j]) && (markerIndex < numMarkers)) {
			markerIndex++;
			d2 = markerPos[markerIndex];
		}
		//if the position of the leftmost flanking marker is to the right of the position of interest
		if (markerIndex == 0) {
			d1 = -100;

		} else {
			d1 = markerPos[markerIndex - 1];
		}

		//case of having only the left flanking marker
		if (markerIndex == numMarkers) {
			d2 = -100;
			//cout<<"inheritance vector will be calculated from only the left flanking marker\n";
		}

		// NEED TO ADJUST CALCWEIGHT for the case where only d1 is available.. or only d2 is available
		weightProb[0][j] = calcWeight(0, 0, d1, d2, positions[j]);
		weightProb[1][j] = calcWeight(0, 1, d1, d2, positions[j]);
		weightProb[2][j] = calcWeight(1, 0, d1, d2, positions[j]);
		weightProb[3][j] = calcWeight(1, 1, d1, d2, positions[j]);

		//iterate each sample
		for (int index = 0; index < num_Samples; index++) {
			curMIM = miml_pos -> getMatrix(index);
			mim = miml -> getMatrix(index); // now num_Samples = numIterations

			for (int m_cur = 0; m_cur < num_Meiosis; m_cur++) {

				int m1, m2;
				if (d1 != -100) {
					m1 = mim-> getValue(m_cur, (markerIndex - 1));
				} else {
					m1 = 0; //it doesn't matter because I THINK calcProb does not depend on it - NEED TO VERIFY
				}

				//having the left marker only
				if (d2 != -100) {
					m2 = mim-> getValue(m_cur, (markerIndex));
				} else {
					m2 = 0; //it doesn't matter because I THINK calcProb does not depend on it - NEED TO VERIFY
				}

				int col;
				if (m1 == 0 && m2 == 0) {
					col = 0;
				} else if (m1 == 0 && m2 == 1) {
					col = 1;
				} else if (m1 == 1 && m2 == 0) {
					col = 2;
				} else if (m1 == 1 && m2 == 1) {
					col = 3;
				}

				double probability = weightProb[col][j];
				double ranNum = mtrand1->rand();
				int value;
				if (ranNum <= probability) {
					value = 1;
				} else {
					value = 0;
				}

				curMIM->write2Matrix(value, m_cur, j);
			} //m_cur


		} //iteration

	} //j
	return (miml_pos);
}



/**
 * THIS FUNCTION IS OBSOLETE - replaced by MIM_pos
 */
//this implementation already added the case when only the left is available - although not tested yet; new implementation just makes this code cleaner
//1. inheritance pattern
//Original goal:
//for each person in the pedigree, and for each copy of this person's DNA, assign the probability that it comes from the maternal side (0) or paternal side (1) based on the available inheritance pattern informations (flanking inheritance pattern information)
//altogether, this would reconstruct the probability of the pattern of gene flow at position p
//use Monte Carlo Sampling to sample the inheritance pattern of each nonfounder using the probability of gene flow at position p for each person

//Implementation
//make the look up table of the probability of inheriting from 1 (father) given the marker is either (0,0), (0,1), (1,0), (1,1) based on the Haldane map function : 4 x # position matrix

//use Monte Carlo Sampling to sample the inheritance pattern of each nonfounder using the lookup table
MIM_List *oldMIM_pos(MIM_List *miml, int num_Iterations, int num_Pos,
		int num_Meiosis, int seed, double *positions, double *markerPos,
		int numMarkers) {

	if (numMarkers <= 0) {
		cout << "number of markers should not be less than or equal to 0.\n";
		exit(0);
	}

	//Weight matrix
	double weightProb[4][num_Pos];
	double d1, d2;
	int markerIndex;
	markerIndex = 0;
	d2 = markerPos[markerIndex];
	for (int j = 0; j < num_Pos; j++) {

		//case: 2 flanking markers: d1 assigned , d2 assigned
		//      only the left flanking marker: d1 assigned, d2 = -100
		//      only the right flanking marker: d1=-100, d2 assigned
		//search for a position that is strictly to the right of the position of interest
		while ((d2 <= positions[j]) && (markerIndex < numMarkers)) {
			markerIndex++;
			//                     if(markerIndex >=  numMarkers){  //greater than or equal to.. so the next time it tries...
			//                        d2= -100;
			// d1 = markerPos[markerIndex -1];
			//			break;
			//                      }
			d2 = markerPos[markerIndex];
		}

		//if the position of the leftmost flanking marker is to the right of the position of interest
		if (markerIndex == 0) {
			d1 = -100;
		} else {
			d1 = markerPos[markerIndex - 1];
		}

		//case of having only the left flanking marker
		if (markerIndex == numMarkers) {
			d2 = -100;
			//cout<<"inheritance vector will be calculated from only the left flanking marker\n";
		}

		//
		// NEED TO ADJUST CALCWEIGHT for the case where only d1 is available.. or only d2 is available
		weightProb[0][j] = calcWeight(0, 0, d1, d2, positions[j]);
		weightProb[1][j] = calcWeight(0, 1, d1, d2, positions[j]);
		weightProb[2][j] = calcWeight(1, 0, d1, d2, positions[j]);
		weightProb[3][j] = calcWeight(1, 1, d1, d2, positions[j]);

		//            cout << d1 <<", " << d2 << ", " << positions[j] <<": " <<weightProb[0][j]
		//	      <<" " <<weightProb[1][j] <<" " <<weightProb[2][j] <<" " <<weightProb[3][j] <<"\n";
	} //j

	//Checks
	//cout << " =================================================================" <<endl;
	//cout << " === Weighting Probability === " <<endl;

	//for(int b=0; b<num_Pos; b++){
	//cout << positions[b] <<" ";
	//}
	//cout << endl;

	//for(int a=0; a< 4; a++){
	//for(int b=0; b<num_Pos; b++){
	//    cout << weightProb[a][b] <<" ";
	//}
	//cout <<endl;
	//}
	////////////////////////////////////////////////////////////////////////////////////////


	int num_Samples = num_Iterations;

	MIM_List *miml_pos = new MIM_List_int(num_Samples, num_Meiosis, num_Pos);

	//miml_pos = new MIM_List_int(num_Samples, num_Meiosis, num_Pos);
	//cout << "miml_pos update:" <<miml_pos <<endl;

	MIM *curMIM;
	MIM *mim;

	MTRand mtrand1;

	// and used with
	double a = mtrand1(); // ??????
	mtrand1.seed(seed);

	//for(int index=1; index < 2; index++){
	for (int index = 0; index < num_Samples; index++) {
		curMIM = miml_pos -> getMatrix(index);
		mim = miml -> getMatrix(index); // now num_Samples = numIterations
		double d1, d2;
		int markerIndex;
		markerIndex = 0;
		d2 = markerPos[markerIndex];

		//		  cout << "d2 " <<d2 <<endl;
		for (int j = 0; j < num_Pos; j++) {
			// ?
			while ((d2 <= positions[j]) && (markerIndex < numMarkers)) {
				markerIndex++;
				d2 = markerPos[markerIndex];
			}

			//if the position of the leftmost flanking marker is to the right of the position of interest
			if (markerIndex == 0) {
				d1 = -100;
			} else {
				d1 = markerPos[markerIndex - 1];
			}

			//case of having only the left flanking marker
			if (markerIndex == numMarkers) {
				d2 = -100;
			}

			//

			//cout <<"marker index " << markerIndex <<"\n";
			for (int m_cur = 0; m_cur < num_Meiosis; m_cur++) {

				int m1, m2;
				if (d1 != -100) {
					m1 = mim-> getValue(m_cur, (markerIndex - 1));
				} else {
					m1 = 0; //it doesn't matter because I THINK calcProb does not depend on it - NEED TO VERIFY
				}

				//having the left marker only
				if (d2 != -100) {
					m2 = mim-> getValue(m_cur, (markerIndex));
				} else {
					m2 = 0; //it doesn't matter because I THINK calcProb does not depend on it - NEED TO VERIFY
				}

				//if(j==13){
				//	 cout<<"position: " <<positions[j] <<endl;
				//	 cout<<"Marker index -1: "<< (markerIndex - 1) <<": " <<markerPos[markerIndex-1] <<endl;
				//cout <<m_cur << " ---  m1: " <<m1 <<"  m2:" <<m2 <<endl;
				//cout <<m1 <<" ";
				//}

				int col;
				if (m1 == 0 && m2 == 0) {
					col = 0;
				} else if (m1 == 0 && m2 == 1) {
					col = 1;
				} else if (m1 == 1 && m2 == 0) {
					col = 2;
				} else if (m1 == 1 && m2 == 1) {
					col = 3;
				}

				double probability = weightProb[col][j];
				double ranNum = mtrand1.rand();
				int value;
				if (ranNum <= probability) {
					value = 1;
				} else {
					value = 0;
				}

				curMIM->write2Matrix(value, m_cur, j);
			}

		}
		//cout <<"c";

		//if(index > 3){ exit(0);}
		//testing//////////////////////////////
		/*
		 if(index < 10){
		 for(int a=0; a<20; a++){
		 cout << mim->getValue(0, a) <<" ";
		 }
		 cout<<endl;
		 for(int a=0; a<200; a++){
		 cout << curMIM->getValue(0, a) <<" ";
		 }
		 cout<<endl;
		 }
		 */
		///////////////////////////////////////

	}

	return (miml_pos);
}



//LAST EDITED ON OCT 27, 2009 - CHANGE - LABEL STARTS AT 1 INSTEAD OF 0.
/**
 * Using pedigree information and corresponding meiosis to assign Founder Genome Label to every individuals
 * in the pedigree. We assume top down pedigree data (sorted in a way that the genome founder label of
 * nonfounders can be inferred). (Note: name (ID) does not have to be sorted.)
 * We also assume meiosis comes in consecutive order: curStartMeiosis, curStartMeiosis + 1, curStartMeiosis + 2,
 *  etc...
 *
 * @param pedigree the pedigree to be processed
 * @param miml mim vectors of iterations
 *
 * Assume 0=inherit from mom, 1=inherit from dad
 */
FGL_Matrix_List *MIM2FGL(MIM_List *miml, Ped *pedigree) {

	//create a fgl object with #iterations, # people in pedigree, and # loci
	FGL_Matrix_List *fgl = new FGL_Matrix_List_int(miml->n_Iterations,
			pedigree->n_ind, miml->getMatrix(0)->n_loci);

	//declare variable:
	int pedSize = pedigree->n_ind;
	int n_loci = miml->getMatrix(0)->n_loci;

	int curIteration;
	int cur, curLocus;
	int curLabel = 1; //keep track of the current label to be assigned to the founders
	int curMeiosis = pedigree->startMeio;
	int startMeiosis = pedigree->startMeio; //the starting meiosis number of the pedigree.
	int meiosisValue, fglValue;

	for (cur = 0; cur < pedSize; cur++) {
		// cout<<"still run\n";
		//processing all markers at each pedigree

		//if pedigree is a founder, assign 2 new labels
		if (pedigree->isFounder(cur)) {
			//cout<<"what's up?\n";
			for (curIteration = 0; curIteration < miml->n_Iterations; curIteration++) {
				FGL_Matrix *fglm = fgl->getMatrix(curIteration);
				for (curLocus = 0; curLocus < n_loci; curLocus++) {
					fglm->write2Matrix(curLabel, cur, 0, curLocus); //writing to the 1st allele
				}
			}
			//after writing the labels, we can increment the curLabel for the next founder
			curLabel++;

			for (curIteration = 0; curIteration < miml->n_Iterations; curIteration++) {
				FGL_Matrix *fglm = fgl->getMatrix(curIteration);
				for (curLocus = 0; curLocus < n_loci; curLocus++) {
					fglm->write2Matrix(curLabel, cur, 1, curLocus); //writing to the 2nd allele
				}
			}
			//after writing the labels, we can increment the curLabel for the next founder
			curLabel++;
			//cout<<"what's up is completed?\n";
		}

		//else, inherits the label from parents
		else {
			//cout<<"RUNNING ELSE\n";

			//Note: an individual MIGHT NOT HAVE 2 MEIOSIS.
			//My solution: if we can verify with mat/pat meiosis, look up FGL, else,
			//we will assume it comes from the parent's mother.. person with only 1
			//meiosis is there because the other meiosis should be DIRECTLY from some other founder anyway..
			//THIS IS OUR ASSUMPTION.
			//we would assign a new label..
			PedMeioInfo current_Ind = pedigree->getPedMeio(cur); //retrieve the current pedigree info

			//We now have to process through two meioses: one from mom and one from dad:
			//First if statement is for maternal meiosis
			int momIndex = pedigree->searchIndex(current_Ind.mid); //find the pedigree index associated with ID=tmp.mid
			if (momIndex == -1) {
				cout
						<< "ERROR...CANNOT FIND INDEX. MAYBE THE PEDIGREE IS NOT SORTED CORRECTLY.\n";
			}
			//cout <<"mom index is " <<indexMatch <<endl;
			if (current_Ind.matMeio == curMeiosis) {
				//cout<<"RUNNING MATMEIO\n"; //cout<< curMeiosis <<endl;
				//cout <<tmp.mid <<", " <<tmp.pid <<endl;


				for (curIteration = 0; curIteration < miml->n_Iterations; curIteration++) {
					FGL_Matrix *fglm = fgl->getMatrix(curIteration); //obtain the current FGL matrix
					MIM *mim = miml->getMatrix(curIteration); //obtain the current Meiosis Inheritance Matrix
					//cout<<"curLocus" <<curLocus <<", " <<"n_loci" <<n_loci <<"curMeiosis" <<curMeiosis <<endl;
					for (curLocus = 0; curLocus < n_loci; curLocus++) { //process through all locus
						int curMeiosisIndex = curMeiosis - startMeiosis; //the current meiosis index
						meiosisValue = mim->getValue(curMeiosisIndex, curLocus);
						//cout<<meiosisValue <<" ";
						fglValue = fglm->getValue(momIndex, meiosisValue,
								curLocus); //get FGL from mother
						//cout<<fglValue << " " ;
						fglm->write2Matrix(fglValue, cur, 0, curLocus); //write to maternal allele
						//the above two lines is consistent with the convention that 0-from mother, 1-from father
					}
				}
				curMeiosis++; //update so next time we will processing a new meiosis.. ie. dad's

			} else {
				//cout<<"Maternal's Meiosis does not match... perhaps it is missing or something is wrong.\n";
				//If it is not there because we don't care about this one, we will assume that it is inherited
				//from mother's mother
				//- if we just assign a new label, this will be wrong, because
				//the IBD between the parent and offspring is NOT 0
				for (curIteration = 0; curIteration < miml->n_Iterations; curIteration++) {
					FGL_Matrix *fglm = fgl->getMatrix(curIteration);
					for (curLocus = 0; curLocus < n_loci; curLocus++) {
						fglValue = fglm->getValue(momIndex, 0, curLocus); //get FGL from mother
						fglm->write2Matrix(fglValue, cur, 0, curLocus);
					}
				}
				//curLabel++; -if we do it this way, it will be wrong.

			}

			//Second if statement is for paternal meiosis
			int dadIndex = pedigree->searchIndex(current_Ind.pid); //find the index associated with dad
			if (dadIndex == -1) {
				cout
						<< "ERROR...CANNOT FIND INDEX. MAYBE THE PEDIGREE IS NOT SORTED CORRECTLY.\n";
			}
			//cout <<"dad index is " <<indexMatch <<endl;
			if (current_Ind.patMeio == curMeiosis) {
				//cout<<"RUNNING PATMEIO\n"; //cout<< curMeiosis <<endl;

				for (curIteration = 0; curIteration < miml->n_Iterations; curIteration++) {
					FGL_Matrix *fglm = fgl->getMatrix(curIteration);
					MIM *mim = miml->getMatrix(curIteration);
					// cout <<"num meiosis: " <<mim->n_meiosis <<endl;
					for (curLocus = 0; curLocus < n_loci; curLocus++) {
						int curMeiosisIndex = curMeiosis - startMeiosis;
						meiosisValue = mim->getValue(curMeiosisIndex, curLocus); //optain the meiosis value-0 or 1
						fglValue = fglm->getValue(dadIndex, meiosisValue,
								curLocus); //obtain the FGL from dad's side //0 = from mom 1 = from dad
						fglm->write2Matrix(fglValue, cur, 1, curLocus); //write the FGL to the dad's allele
					}
				}
				curMeiosis++;
				//cout <<"STILL OKAY\n";
			}

			else {
				//cout<<"Paternal's Meiosis does not match... perhaps it is missing or something is wrong.\n";
				//If it is not there because we don't care about this one, we will assume that it is inherited
				//from father's mother
				//- if we just assign a new label, this will be wrong, because
				//the IBD between the two individuals is NOT 0
				for (curIteration = 0; curIteration < miml->n_Iterations; curIteration++) {
					FGL_Matrix *fglm = fgl->getMatrix(curIteration);
					for (curLocus = 0; curLocus < n_loci; curLocus++) {
						fglValue = fglm->getValue(dadIndex, 0, curLocus); //get FGL from mother
						fglm->write2Matrix(fglValue, cur, 1, curLocus);
					}
				}
				//curLabel++;	-if we do it this way, it will be wrong.
			} //closing else for paternal's meiosis doesn't match.

			//We just finished processing the two meiosis for a nonfounder individual
		} //closing else - nonfounder or founder

		//we have just looped through all pedigree members
	} //closing for- loop through all pedigree members

	//returning the pointer to the fgl matrix list
	return (fgl);
}

/**
 * Read the FGL matrices (the data containing many iterations of the genome founder labels) and
 * return the estimates of kinship coefficient matrix list, P(IBD=2) matrix list, P(IBD=1)matrix list
 * of EACH marker from the FGL matrices.
 * ie. kinship[1] contains the kinship matrix of the second marker originally defined in the FGL matrices.
 *
 * note that the matrix is provided by FGL_Matrix (which is not necessarily sorted by name id but should be
 * sorted to preserve the meiosis-FGL determination ordering)
 *
 * Calculation of Kinship: 0.25*freq(FGL of allele i from ind1 == FGL of allele k from ind 2)
 * 						  +0.25*freq(FGL of allele j from ind1 == FGL of allele k from ind 2)
 * 						  +0.25*freq(FGL of allele i from ind1 == FGL of allele l from ind 2)
 * 						  +0.25*freq(FGL of allele j from ind1 == FGL of allele l from ind 2)
 *
 * Calculation of P(IBD=2): freq( (FGL i==FGL k AND FGL j == FGL l) OR (FGL i==FGL l AND FGL j == FGL k ))
 *
 * ...
 */
void compute_mibd_Optimized(TopTri_Matrix *IBDtwo, TopTri_Matrix *IBDone,
		FGL_Matrix_List *fglm, int n_loci, int n_ind) {

	int cur;
	//calculate IBD
	int num_Iterations = fglm->n_Iterations;

	cout << num_Iterations << endl;

	int l;
	int oneA, oneB, twoA, twoB, ind1, ind2;
	//switching the order of the loops may affect performance significantly!!!

	//Initialize the values of the matrices first
	for (ind1 = 0; ind1 < n_ind; ind1++) {
		for (ind2 = (ind1 + 1); ind2 < n_ind; ind2++) {
			for (l = 0; l < n_loci; l++) {
				IBDtwo[l].writeMatrix(0, ind1, ind2);
				IBDone[l].writeMatrix(0, ind1, ind2);
			}
		}
	}

	for (cur = 0; cur < num_Iterations; cur++) {
		for (ind1 = 0; ind1 < n_ind; ind1++) {
			for (ind2 = (ind1 + 1); ind2 < n_ind; ind2++) {
				for (l = 0; l < n_loci; l++) {
					double IBD_two = 0;
					double IBD_one = 0;
					oneA = fglm->getMatrix(cur)->getValue(ind1, 0, l);
					oneB = fglm->getMatrix(cur)->getValue(ind1, 1, l);
					twoA = fglm->getMatrix(cur)->getValue(ind2, 0, l);
					twoB = fglm->getMatrix(cur)->getValue(ind2, 1, l);

					if (((oneA == twoA) && (oneB == twoB)) || ((oneA == twoB)
							&& (oneB == twoA))) {
						IBD_two = IBDtwo[l].readMatrix(ind1, ind2) + 1;
						IBDtwo[l].writeMatrix(IBD_two, ind1, ind2);

					}
					if (((oneA == twoA) && (oneB != twoB)) || ((oneA == twoB)
							&& (oneB != twoA)) || ((oneA != twoA) && (oneB
							== twoB)) || ((oneA != twoB) && (oneB == twoA))) {
						IBD_one = IBDone[l].readMatrix(ind1, ind2) + 1;
						IBDone[l].writeMatrix(IBD_one, ind1, ind2);
					}

				}
			}
		}
	}
	for (l = 0; l < n_loci; l++) {
		for (ind1 = 0; ind1 < n_ind; ind1++) {
			for (ind2 = (ind1 + 1); ind2 < n_ind; ind2++) {
				//cout <<IBDtwo[l].readMatrix(ind1, ind2) <<" and " <<IBDone[l].readMatrix(ind1, ind2) <<endl;
				double IBD_two = IBDtwo[l].readMatrix(ind1, ind2)
						/ num_Iterations;
				IBDtwo[l].writeMatrix(IBD_two, ind1, ind2);
				double IBD_one = IBDone[l].readMatrix(ind1, ind2)
						/ num_Iterations;
				IBDone[l].writeMatrix(IBD_one, ind1, ind2);

				//cout <<num_Iterations <<endl;
				//cout<<"so: " <<IBD_two <<" and " <<IBD_one <<endl;
			}
		}
	}

	//VALIDITY CHECK//
	//- IT SEEMS THAT THE ONLY PROBLEM WE ARE HAVING IS DUE
	//TO FLOATING POINT DIFFERENCE
	//double kinship = 0.25 * AA + 0.25 * AB + 0.25 * BA + 0.25 * BB;
	//double kinshipX2 = kinship*2;
	//if(kinshipX2 != IBD){
	//cout <<"For individual 1: " <<ind1 <<" and individual 2: " <<ind2 <<endl;
	//cout <<"IBD one: " <<IBD_one <<", IBD two:" << IBD_two <<endl;
	//	cout << kinshipX2 <<" vs. " <<IBD <<endl;
	//	cout << "kinship and IBD output aren't the same. Exit now\n";
	//}

	//	cout<<ind1 <<" and " <<ind2 <<"- AA:"<<AA <<", AB" <<AB <<", BA:" <<BA <<", BB" <<BB
	//				<<"\t IBD=" <<IBD <<" wrote: " << mibd_List[l].readMatrix(ind1, ind2) <<endl;


	//validity check--uncomment otherwise -notice this gives us the kinship matrix
	//	double kinship=0.5*IBD_two + 0.25*IBD_one;  //
	//	IBDone[l].writeMatrix(kinship, ind1, ind2);
	//IBDone[l].changeDiagValue(0.5);
	////////////////////////////////////////////


	//return mibd_List;

}

/**
 * Write the IBD matrix out in Solar Compatible format: (UNSORTED VERSION)
 *ie.
 * CHROM,LOCATION,ID1,ID2,IBD,D7
 *
 **/
void writeIBDMatrix(TopTri_Matrix *ibdMatList, int n_locus, int chromNum,
		int location) {
	int i, j, k;
	double value;
	int n_ind = ibdMatList[0].n_ind;
	//C++ style write:
	ofstream myfile("example.txt");
	if (myfile.is_open()) {
		myfile << "CHROM,LOCATION, ID1, ID2,IBD\n";
		for (k = 0; k < n_locus; k++) {
			for (i = 0; i < n_ind; i++) {
				for (j = i + 1; j < n_ind; j++) {
					value = ibdMatList[k].readMatrix(i, j);
					myfile << chromNum << location << "ID1" << "ID2" << value
							<< endl;
				}
			}
		}
	} else {
		cout << "Can't Open file for writing\n";
	}
	myfile.close();

}

/**
 * write P(IBD=1) and P(IBD=2) out to a file.
 *
 */
void writeIBDonetwo(const char *outFile, TopTri_Matrix *ibdOne,
		TopTri_Matrix *ibdTwo, int num_Markers, int pedSize, int pedIndex,
		Ped *pedigree, int IS_BEGINNING) {
	ofstream myfile;
	//ofstream out("testingFile.txt");

	//if(IS_BEGINNING){
	myfile.open(outFile);
	//}
	//else{
	//	myfile.open(outFile, ios::app);
	//}

	if (!myfile) {
		cout << "Cannot open file.\n";
		exit(1);
	}

	if (myfile.is_open()) {
		if (IS_BEGINNING) {
			myfile << "PedIndex,MarkerIndex,Ind1ID,Ind2ID,IBD1,IBD2\n";
		}

		for (int cur = 0; cur < num_Markers; cur++) {
			for (int ind1 = 0; ind1 < pedSize; ind1++) {
				for (int ind2 = ind1; ind2 < pedSize; ind2++) {
					myfile << pedIndex << "," << cur << ","
							<< pedigree->getPedMeio(ind1).id << ","
							<< pedigree->getPedMeio(ind2).id << ","
							<< ibdOne[cur].readMatrix(ind1, ind2) << ","
							<< ibdTwo[cur].readMatrix(ind1, ind2) << "\n";
				}
			}
		}
	}
	myfile.close();
}

void remove_carriage_return(std::string& line)
{
    if (*line.rbegin() == '\r')
    {
        line.erase(line.length() - 1);
    }
}


/**
 * Read the parameter file of GIGI
 * This version fixes readImputeParameterFile_v0_05()
 * The problem was that
 *
 * "If your inputFile has Windows-style line-breaks (CRLF) but your program is
 * running on a system that has Unix-like line-breaks (LF), then std::getline() will
 * leave the CR at the end of your string and you'll need to trim it."
 *
 */
void readImputeParameterFile_GIGI(char *imputeParFileName,
		char **pedigreeFilePtr, int *num_Ped, char **denseMarkersFilePtr,
		char **MIMFilePtr, long *num_Iterations, char **markerPanelPosFilePtr,
		char **denseMarkersPosFilePtr, char **afFilePtr, int *seed,
		char **genotypeFilePtr, int *method, double *threshold1,
		double *threshold2, char **probFilePtr, char **consistencyFilePtr) {

	char *pedigreeFile, *MIMFile;
	int *markersToKeep;
	double *markerPos, *positions;
	char *d, *a, *h, *g;
	ifstream inFile;
	inFile.open(imputeParFileName);
	if (!inFile) {
		cout << "Unable to open the imputation parameter file. Exit now.\n";
		exit(1); // terminate with error
	}
	string line;

	//pedigree file
	getline(inFile, line);
	remove_carriage_return(line);
	pedigreeFile = new char[line.length() + 1];
	strcpy(pedigreeFile, line.c_str());

	//num pedigree
	getline(inFile, line);
	remove_carriage_return(line);
	char *num_PedStr = new char[line.length() + 1];
	strcpy(num_PedStr, line.c_str());
	*num_Ped = atoi(num_PedStr);

	//dense marker file
	getline(inFile, line);
	remove_carriage_return(line);
	d = new char[line.length() + 1];
	strcpy(d, line.c_str());

	//inheritance vector file
	getline(inFile, line);
	remove_carriage_return(line);
	MIMFile = new char[line.length() + 1];
	strcpy(MIMFile, line.c_str());

	//num iterations
	getline(inFile, line);
	remove_carriage_return(line);
	char *num_IterationsStr = new char[line.length() + 1];
	strcpy(num_IterationsStr, line.c_str());
	*num_Iterations = atol(num_IterationsStr);

	//positions of markers used for inheritance vector computation
	getline(inFile, line);
	remove_carriage_return(line);
	char *markerPanelPosFile = new char[line.length() + 1];
	strcpy(markerPanelPosFile, line.c_str());

	//number of panel markers
	//getline(inFile, line);
	//char *numPanelMarkersStr = new char[line.length()+1];
	//strcpy(numPanelMarkersStr, line.c_str() );
	//*numPanelMarkers = atoi(numPanelMarkersStr);

	//positions of markers used for dense markers
	getline(inFile, line);
	remove_carriage_return(line);
	char *denseMarkersPosFile = new char[line.length() + 1];
	strcpy(denseMarkersPosFile, line.c_str());

	//number of dense markers
	//getline (inFile,line);
	//char *numDenseMarkersStr = new char[line.length()+1];
	//strcpy ( numDenseMarkersStr, line.c_str() );
	//*numDenseMarkers = atoi (numDenseMarkersStr);

	//allele frequency file of dense markers
	getline(inFile, line);
	remove_carriage_return(line);
	a = new char[line.length() + 1];
	strcpy(a, line.c_str());

	//seed
	getline(inFile, line);
	remove_carriage_return(line);
	char *seedStr = new char[line.length() + 1];
	strcpy(seedStr, line.c_str());
	*seed = atoi(seedStr);

	//output haplotype file
	//getline (inFile,line);
	//    h = new char[line.length()+1];
	//	    strcpy ( h, line.c_str() );

	//output genotype file
	getline(inFile, line);
	remove_carriage_return(line);
	g = new char[line.length() + 1];
	strcpy(g, line.c_str());

	//method
	getline(inFile, line);
	remove_carriage_return(line);
	char *methodStr = new char[line.length() + 1];
	strcpy(methodStr, line.c_str());
	*method = atoi(methodStr);

	//threshold1
	getline(inFile, line);
	remove_carriage_return(line);
	char *t1Str = new char[line.length() + 1];
	strcpy(t1Str, line.c_str());
	*threshold1 = atof(t1Str);

	//threshold2
	getline(inFile, line);
	remove_carriage_return(line);
	char *t2Str = new char[line.length() + 1];
	strcpy(t2Str, line.c_str());
	*threshold2 = atof(t2Str);

	//cout << line.c_str() <<endl;
	//
	getline(inFile,line);
	remove_carriage_return(line);
	char *probFile = new char[line.length() + 1];
	strcpy(probFile, line.c_str());



	getline(inFile, line);
	remove_carriage_return(line);
	char *consistencyFile  = new char[line.length() + 1];
	strcpy(consistencyFile, line.c_str());

	*pedigreeFilePtr = pedigreeFile;
	*denseMarkersFilePtr = d;
	*MIMFilePtr = MIMFile;
	*markerPanelPosFilePtr = markerPanelPosFile;
	*denseMarkersPosFilePtr = denseMarkersPosFile;
	*afFilePtr = a;
	//*hapFilePtr = h;
	*genotypeFilePtr = g;

	*probFilePtr = probFile;
	*consistencyFilePtr = consistencyFile;
	//delete?
	delete[] num_PedStr;
	delete[] num_IterationsStr;
	delete[] seedStr;
	delete[] methodStr;
	delete[] t1Str;
	delete[] t2Str;
	//delete []numDenseMarkersStr;
	//delete []numPanelMarkersStr;

	inFile.close();

}



/**
 * The new parameter file format
 *
 * Read the parameter file of GIGI
 *
 */
void readImputeParameterFile_GIGI_v1_01(char *imputeParFileName,
		char **pedigreeFilePtr, int *num_Ped, char **denseMarkersFilePtr,
		char **MIMFilePtr, long *num_Iterations, char **markerPanelPosFilePtr,
		char **denseMarkersPosFilePtr, char **afFilePtr, int *seed,
		char **genotypeFilePtr, int *method, double *threshold1,
		double *threshold2, char **probFilePtr, char **consistencyFilePtr) {

	char *pedigreeFile, *MIMFile;
	int *markersToKeep;
	double *markerPos, *positions;
	char *d, *a, *h, *g;
	ifstream inFile;
	inFile.open(imputeParFileName);
	if (!inFile) {
		cout << "Unable to open the imputation parameter file. Exit now.\n";
		exit(1); // terminate with error
	}
	string line;

	//pedigree file
	getline(inFile, line);
	remove_carriage_return(line);
	pedigreeFile = new char[line.length() + 1];
	strcpy(pedigreeFile, line.c_str());

	//inheritance vector file
	getline(inFile, line);
	remove_carriage_return(line);
	MIMFile = new char[line.length() + 1];
	strcpy(MIMFile, line.c_str());

	//num iterations
	getline(inFile, line);
	remove_carriage_return(line);
	char *num_IterationsStr = new char[line.length() + 1];
	strcpy(num_IterationsStr, line.c_str());
	*num_Iterations = atol(num_IterationsStr);

	//positions of markers used for inheritance vector computation
	getline(inFile, line);
	remove_carriage_return(line);
	char *markerPanelPosFile = new char[line.length() + 1];
	strcpy(markerPanelPosFile, line.c_str());

	//positions of markers used for dense markers
	getline(inFile, line);
	remove_carriage_return(line);
	char *denseMarkersPosFile = new char[line.length() + 1];
	strcpy(denseMarkersPosFile, line.c_str());


	//dense marker file
	getline(inFile, line);
	remove_carriage_return(line);
	d = new char[line.length() + 1];
	strcpy(d, line.c_str());


	//allele frequency file of dense markers
	getline(inFile, line);
	remove_carriage_return(line);
	a = new char[line.length() + 1];
	strcpy(a, line.c_str());

	getline(inFile, line);
	remove_carriage_return(line);
	char *methodStrNew = new char[line.length() + 1];
	strcpy(methodStrNew, line.c_str());
	  // parse the line into blank-delimited tokens
	    int n = 0; // a for-loop index

	    // array to store memory addresses of the tokens in buf
	    const char* token[3] = {}; // initialize to 0

	    // parse the line: the format should be
	    // method t1 t2
	    // method=1 for most probable calling and method=2 for confidence-based threshold calling
	    //use threhold1=0.8 by default and threshold2=0.9 by default, but allow the user to set
	    //threshold1 and threshold2 if t1 and t2 are provided... if only t1 is provided, use threshold1=0.8 and threshold2=1-threshold1/2
	    token[0] = strtok(methodStrNew, " "); // first token
	    *method = atoi(token[0]);
		*threshold1 = 0.8; //default
		*threshold2 = 0.9; //default

	    if (token[0]) // zero if line is blank
	    {
	     //read the second token
	     token[1] = strtok(0, " "); // subsequent tokens
		 if (token[1]){
		  *threshold1=atof(token[1]);
		  *threshold2 = (1 + *threshold1)/2; //set threshold2 by default
		 }
	    }
	    if (token[1]) // zero if line is blank
	    {
	      //read the third token
	      token[2] = strtok(0, " "); // subsequent tokens
	      if (token[2]){
	       *threshold2=atof(token[2]);
	      }
	    }
	    if(! (*method == 1 | *method==2)){
	    	cout<<"Call method should be either 1 or 2. Exit with error.\n";
	    	exit(0);
	    }

	    if(*threshold1 >= 1 | *threshold1 < 0.5){
	    	cout<<"threshold 1 should be a number greater than or equal to 0.5 and less than 1. Exit with error.\n";
	    	exit(0);
	    }
	    if(*threshold2 >= 1 | *threshold2 < 0.5){
	    	cout<<"threshold 2 should be a number greater than or equal to 0.5 and less than 1. Exit with error.\n";
	    	exit(0);
	    }


	*pedigreeFilePtr = pedigreeFile;
	*denseMarkersFilePtr = d;
	*MIMFilePtr = MIMFile;
	*markerPanelPosFilePtr = markerPanelPosFile;
	*denseMarkersPosFilePtr = denseMarkersPosFile;
	*afFilePtr = a;
	//*hapFilePtr = h;
	*genotypeFilePtr = g;
	//delete?
	delete[] num_IterationsStr;
	delete[] methodStrNew;

	inFile.close();

}





/**
 * OBSOLETE - this version does not work in all platforms
 */
void readImputeParameterFile_v0_05(char *imputeParFileName,
		char **pedigreeFilePtr, int *num_Ped, char **denseMarkersFilePtr,
		char **MIMFilePtr, long *num_Iterations, char **markerPanelPosFilePtr,
		char **denseMarkersPosFilePtr, char **afFilePtr, int *seed,
		char **genotypeFilePtr, int *method, double *threshold1,
		double *threshold2, char **probFilePtr, char **consistencyFilePtr) {

	char *pedigreeFile, *MIMFile;
	int *markersToKeep;
	double *markerPos, *positions;
	char *d, *a, *h, *g;
	ifstream inFile;
	inFile.open(imputeParFileName);
	if (!inFile) {
		cout << "Unable to open the imputation parameter file. Exit now.\n";
		exit(1); // terminate with error
	}
	string line;

	//pedigree file
	getline(inFile, line);
	pedigreeFile = new char[line.length() + 1];
	strcpy(pedigreeFile, line.c_str());

	//num pedigree
	getline(inFile, line);
	char *num_PedStr = new char[line.length() + 1];
	strcpy(num_PedStr, line.c_str());
	*num_Ped = atoi(num_PedStr);

	//dense marker file
	getline(inFile, line);
	d = new char[line.length() + 1];
	strcpy(d, line.c_str());

	//inheritance vector file
	getline(inFile, line);
	MIMFile = new char[line.length() + 1];
	strcpy(MIMFile, line.c_str());

	//num iterations
	getline(inFile, line);
	char *num_IterationsStr = new char[line.length() + 1];
	strcpy(num_IterationsStr, line.c_str());
	*num_Iterations = atol(num_IterationsStr);

	//positions of markers used for inheritance vector computation
	getline(inFile, line);
	char *markerPanelPosFile = new char[line.length() + 1];
	strcpy(markerPanelPosFile, line.c_str());

	//number of panel markers
	//getline(inFile, line);
	//char *numPanelMarkersStr = new char[line.length()+1];
	//strcpy(numPanelMarkersStr, line.c_str() );
	//*numPanelMarkers = atoi(numPanelMarkersStr);

	//positions of markers used for dense markers
	getline(inFile, line);
	char *denseMarkersPosFile = new char[line.length() + 1];
	strcpy(denseMarkersPosFile, line.c_str());

	//number of dense markers
	//getline (inFile,line);
	//char *numDenseMarkersStr = new char[line.length()+1];
	//strcpy ( numDenseMarkersStr, line.c_str() );
	//*numDenseMarkers = atoi (numDenseMarkersStr);

	//allele frequency file of dense markers
	getline(inFile, line);
	a = new char[line.length() + 1];
	strcpy(a, line.c_str());

	//seed
	getline(inFile, line);
	char *seedStr = new char[line.length() + 1];
	strcpy(seedStr, line.c_str());
	*seed = atoi(seedStr);

	//output haplotype file
	//getline (inFile,line);
	//    h = new char[line.length()+1];
	//	    strcpy ( h, line.c_str() );

	//output genotype file
	getline(inFile, line);
	g = new char[line.length() + 1];
	strcpy(g, line.c_str());

	//method
	getline(inFile, line);
	char *methodStr = new char[line.length() + 1];
	strcpy(methodStr, line.c_str());
	*method = atoi(methodStr);

	//threshold1
	getline(inFile, line);
	char *t1Str = new char[line.length() + 1];
	strcpy(t1Str, line.c_str());
	*threshold1 = atof(t1Str);

	//threshold2
	getline(inFile, line);
	char *t2Str = new char[line.length() + 1];
	strcpy(t2Str, line.c_str());
	*threshold2 = atof(t2Str);

	//cout << line.c_str() <<endl;
	//
	char *probFile;
	const char *defaultProbFile = "imputed_probabilities.txt";
	if (getline(inFile, line) == NULL) {
		strncpy(probFile, defaultProbFile, strlen(defaultProbFile) + 1);
		probFile[strlen(defaultProbFile) + 1 - 1] = '\0';
		cout << "Probability file not specified. use DEFAULT: " << probFile
				<< "\n";
	} else {
		probFile = new char[line.length() + 1];
		strcpy(probFile, line.c_str());
	}

	char *consistencyFile;
	const char *defaultconsistencyFile = "consistency.txt";
	if (getline(inFile, line) == NULL) {
		strncpy(consistencyFile, defaultconsistencyFile, strlen(
				defaultconsistencyFile) + 1);
		consistencyFile[strlen(defaultconsistencyFile) + 1 - 1] = '\0';
		cout << "consistency file not specified. use DEFAULT: "
				<< consistencyFile << "\n";
	} else {
		consistencyFile = new char[line.length() + 1];
		strcpy(consistencyFile, line.c_str());
		//cout<< consistencyFile <<"\n";
	}

	*pedigreeFilePtr = pedigreeFile;
	*denseMarkersFilePtr = d;
	*MIMFilePtr = MIMFile;
	*markerPanelPosFilePtr = markerPanelPosFile;
	*denseMarkersPosFilePtr = denseMarkersPosFile;
	*afFilePtr = a;
	//*hapFilePtr = h;
	*genotypeFilePtr = g;

	*probFilePtr = probFile;
	*consistencyFilePtr = consistencyFile;
	//delete?
	delete[] num_PedStr;
	delete[] num_IterationsStr;
	delete[] seedStr;
	delete[] methodStr;
	delete[] t1Str;
	delete[] t2Str;
	//delete []numDenseMarkersStr;
	//delete []numPanelMarkersStr;

	//others?
}


/**
 * reads 1 iteration
 * This reads True FGL of this format:
 * ie.
 * ID maternal/paternal FGL_m1 FGL_m2 FGL_m3 ...
 * 101 0 1 1 1
 * 101 1 2 2 2
 * 102 0 3 3 3
 * 102 1 4 4 4
 * 201 0 1 1 2
 * 201 0 3 3 3
 */
FGL_Matrix_List *readTrueFGLM_v2(char *truthFGLFile, Ped *pedigree,
		int numMarkers) {

	int numInd = pedigree->n_ind;

	FGL_Matrix_List *testAlleleTrueFGLM = new FGL_Matrix_List_int(1,
			pedigree->n_ind, numMarkers);

	ifstream inFile;
	inFile.open(truthFGLFile);
	if (!inFile) {
		cout << "Unable to open the true FGL file. Exit now.\n";
		exit(1); // terminate with error
	}

	char *pch;
	string line;
	char *line_c;

	FGL_Matrix *trueFGL = testAlleleTrueFGLM->getMatrix(0);

	for (int indexNumber = 0; indexNumber < pedigree->n_ind * 2; indexNumber++) {
		if (getline(inFile, line) != NULL) {

			if (line.empty()) {
				//cout<<"empty!\n";
				//just ignore empty line
			} else {
				int curMarker = 0;
				char *markersToKeepStr = new char[line.length() + 1];
				strcpy(markersToKeepStr, line.c_str());
				pch = strtok(markersToKeepStr, " ");
				cout << pch << endl;
				int indexNumber = pedigree->searchIndex(pch);
				cout << "The index number is " << indexNumber << endl;

				//read matPat
				pch = strtok(NULL, " ");
				int matPat = atoi(pch);

				//read marker
				for (curMarker = 0; curMarker < numMarkers; curMarker++) {
					//printf ("%s\n",pch);
					pch = strtok(NULL, " ");
					trueFGL->write2Matrix(atoi(pch), indexNumber, matPat,
							curMarker);
				}
				//cout <<endl;
			} //non-empty line
		} //getline
		else {
			cout
					<< "can't get the next line. maybe file ends earlier than expected for some reason?\n";
		}
	}//for

	inFile.close();

	return (testAlleleTrueFGLM);
}

//FGL is read the same format as marker data
/**
 * This is an extension to readTrueFGL to read multiple expanded FGL in the file
 */
FGL_Matrix_List *readTrueFGLM(char *truthFGLFile, Ped *pedigree,
		int numMarkers, int numIterations) {

	int numInd = pedigree->n_ind;

	FGL_Matrix_List *testAlleleTrueFGLM = new FGL_Matrix_List_int(
			numIterations, pedigree->n_ind, numMarkers);

	cout << testAlleleTrueFGLM->n_Iterations << endl;

	ifstream inFile;
	inFile.open(truthFGLFile);
	if (!inFile) {
		cout << "Unable to open the true FGL file. Exit now.\n";
		exit(1); // terminate with error
	}

	char *pch;
	string line;
	char *line_c;
	getline(inFile, line); //ignore the header

	for (int index = 0; index < numIterations; index++) {
		FGL_Matrix *trueFGL = testAlleleTrueFGLM->getMatrix(index);

		for (int indexNumber = 0; indexNumber < pedigree->n_ind; indexNumber++) {
			if (getline(inFile, line) != NULL) {

				if (line.empty()) {
					//cout<<"empty!\n";
					//just ignore empty line
				} else {
					int curMarker = 0;
					char *markersToKeepStr = new char[line.length() + 1];
					strcpy(markersToKeepStr, line.c_str());
					pch = strtok(markersToKeepStr, " ");
					cout << pch << endl;

					//ignore mother and father
					pch = strtok(NULL, " ");
					pch = strtok(NULL, " ");

					//int indexNumber = pedigree->searchIndex(pch);
					//cout<<"The index number is " <<indexNumber <<endl;

					for (curMarker = 0; curMarker < numMarkers; curMarker++) {
						//printf ("%s\n",pch);
						pch = strtok(NULL, " ");
						// denseMarkers[curMarker] = atoi(pch);
						//cout << atoi(pch) << " ---";
						trueFGL->write2Matrix(atoi(pch), indexNumber, 0,
								curMarker);
						pch = strtok(NULL, " ");
						trueFGL->write2Matrix(atoi(pch), indexNumber, 1,
								curMarker);
					}
					//cout <<endl;
				} //non-empty line
			} //getline
			else {
				cout
						<< "can't get the next line. maybe file ends earlier than expected for some reason?\n";
			}
		}//for


	}//index
	inFile.close();

	return (testAlleleTrueFGLM);
}

//FGL is read the same format as marker data
/**
 * This is an extension to readTrueFGL to read multiple expanded FGL in the file
 */
FGL_Matrix_List *readTrueFGLM_noHeader(char *truthFGLFile, Ped *pedigree,
		int numMarkers, int numIterations) {

	cout << "NUM ITERATIONS: " << numIterations << endl;
	int numInd = pedigree->n_ind;

	FGL_Matrix_List *testAlleleTrueFGLM = new FGL_Matrix_List_int(
			numIterations, pedigree->n_ind, numMarkers);

	cout << testAlleleTrueFGLM->n_Iterations << endl;

	ifstream inFile;
	inFile.open(truthFGLFile);
	if (!inFile) {
		cout << "Unable to open the true FGL file. Exit now.\n";
		exit(1); // terminate with error
	}

	char *pch;
	string line;
	char *line_c;
	for (int index = 0; index < numIterations; index++) {
		FGL_Matrix *trueFGL = testAlleleTrueFGLM->getMatrix(index);
		for (int indexNumber = 0; indexNumber < pedigree->n_ind; indexNumber++) {
			if (getline(inFile, line) != NULL) {

				if (line.empty()) {
					//cout<<"empty!\n";
					//just ignore empty line
				} else {
					int curMarker = 0;
					char *markersToKeepStr = new char[line.length() + 1];
					strcpy(markersToKeepStr, line.c_str());
					pch = strtok(markersToKeepStr, " ");
					cout << pch << endl;
					int indexNumber = pedigree->searchIndex(pch);
					cout << "The index number is " << indexNumber << endl;

					for (curMarker = 0; curMarker < numMarkers; curMarker++) {
						//printf ("%s\n",pch);
						pch = strtok(NULL, " ");
						// denseMarkers[curMarker] = atoi(pch);
						//cout << atoi(pch) << " ---";
						trueFGL->write2Matrix(atoi(pch), indexNumber, 0,
								curMarker);
						pch = strtok(NULL, " ");
						trueFGL->write2Matrix(atoi(pch), indexNumber, 1,
								curMarker);
					}
					//cout <<endl;
				} //non-empty line
			} //getline
			else {
				cout
						<< "can't get the next line. maybe file ends earlier than expected for some reason?\n";
			}
		}//for


	}//index
	inFile.close();

	return (testAlleleTrueFGLM);
}

FGL_Matrix *readTrueFGL(char *truthFGLFile, Ped *pedigree, int numMarkers) {

	int numInd = pedigree->n_ind;

	FGL_Matrix *trueFGL = new FGL_Matrix_int;

	trueFGL->FGL_Matrix_INIT(numInd, numMarkers);

	ifstream inFile;
	inFile.open(truthFGLFile);
	if (!inFile) {
		cout << "Unable to open the true FGL file. Exit now.\n";
		exit(1); // terminate with error
	}

	char *pch;
	string line;
	char *line_c;
	getline(inFile, line); //ignore the header

	int indexNumber = 0;
	while (getline(inFile, line) != NULL) {

		if (line.empty()) {
			//cout<<"empty!\n";
			//just ignore empty line
		} else {
			// cout << line <<endl;

			int curMarker = 0;
			char *markersToKeepStr = new char[line.length() + 1];
			strcpy(markersToKeepStr, line.c_str());
			pch = strtok(markersToKeepStr, " ");
			cout << pch << endl;

			//ignore mother and father
			pch = strtok(NULL, " ");
			pch = strtok(NULL, " ");

			//int indexNumber = pedigree->searchIndex(pch);
			//cout<<"The index number is " <<indexNumber <<endl;

			for (curMarker = 0; curMarker < numMarkers; curMarker++) {
				//    printf ("%s\n",pch);
				pch = strtok(NULL, " ");
				// denseMarkers[curMarker] = atoi(pch);
				//cout << atoi(pch) << " ---";
				trueFGL->write2Matrix(atoi(pch), indexNumber, 0, curMarker);

				pch = strtok(NULL, " ");
				trueFGL->write2Matrix(atoi(pch), indexNumber, 1, curMarker);

			}
			//cout <<endl;
			indexNumber++;
		} //non-empty line
	} //getline


	inFile.close();

	for (int i = 0; i < indexNumber; i++) {
		cout << pedigree->getPedMeio(i).id << ": ";
		for (int j = 0; j < numMarkers; j++) {
			cout << trueFGL->getValue(i, 0, j) << "," << trueFGL->getValue(i,
					1, j) << " ";
		}
		cout << "\n=======================================\n";
	}

	return (trueFGL);
}

/**
 * Read dense markers into FGL - note that the reading.. assumes same order as in the pedigree

 * WRONG! not everybody is typed for the dense markers.. only a selected few are!, so what that means is, I  have to match the correct id
 *
 */
//also need to match the id
FGL_Matrix *readDenseMarkers(char *denseMarkersFile, Ped *pedigree,
		int *observed, int num_denseMarkers) {

	int verbose = 0;

	char *pch;

	ifstream inFile;
	inFile.open(denseMarkersFile);
	if (!inFile) {
		cout << "Unable to open the dense marker file. Exit now.\n";
		cout << denseMarkersFile << endl;
		exit(1); // terminate with error
	}
	string line;
	char *line_c;

	int foundLine = 0;
	//int num_denseMarkers;


	FGL_Matrix *denseMarkers = new FGL_Matrix_int;

	//while(  (foundLine==0)   &&(getline(inFile, line)!=NULL)  ){
	//	line_c = new char[line.length()+1];
	//	strcpy ( line_c, line.c_str() );
	//	pch = strstr (line_c,"set marker data");
	//found string
	// if(pch!=NULL){
	//    num_denseMarkers = atoi (line.substr(16).c_str());
	//    foundLine = 1;
	//  }
	//}

	//if(foundLine==1){
	//cout<<pedigree->n_ind <<", " <<num_denseMarkers <<endl;
	denseMarkers->FGL_Matrix_INIT(pedigree->n_ind, num_denseMarkers);

	while (getline(inFile, line) != NULL) {

		if (line.empty()) {
			//cout<<"empty!\n";
			//just ignore empty line
		} else {
			// cout << line <<endl;

			//int *denseMarkers = new int [num_denseMarkers*2];	//just to play safe.

			int curMarker = 0;
			char *markersToKeepStr = new char[line.length() + 1];
			strcpy(markersToKeepStr, line.c_str());
			pch = strtok(markersToKeepStr, " ");
			//cout <<pch <<endl;

			int indexNumber = pedigree->searchIndex(pch);
			cout << "index number is " << indexNumber << endl;
			observed[indexNumber] = 1;
			if (indexNumber == -1) {
				cout << "Can't match the individual to the pedigree file\n";
			}

			if (verbose) {
				cout << "The index number of " << pch << " is " << indexNumber
						<< endl;
			}

			for (curMarker = 0; curMarker < num_denseMarkers; curMarker++) {

				//    printf ("%s\n",pch);
				pch = strtok(NULL, " ");
				// denseMarkers[curMarker] = atoi(pch);
				//cout << atoi(pch) << " ---";
				denseMarkers->write2Matrix(atoi(pch), indexNumber, 0, curMarker);

				pch = strtok(NULL, " ");
				denseMarkers->write2Matrix(atoi(pch), indexNumber, 1, curMarker);

			}
			//cout <<endl;

		} //non-empty line
	} //getline

	//}//foundLine==1

	inFile.close();

	if (verbose) {
		for (int i = 0; i < pedigree->n_ind; i++) {
			cout << pedigree->getPedMeio(i).id << ": ";
			for (int j = 0; j < num_denseMarkers; j++) {
				cout << denseMarkers->getValue(i, 0, j) << "/"
						<< denseMarkers->getValue(i, 1, j) << " ";
			}
			//cout<<"yo\n";
			cout << endl;
		}
	}
	//

	return (denseMarkers);
}





/**
 * read in the header line
 * get the index of people
 */
ifstream *initializeBEAGLE_denseMarkersStream(char *denseMarkersFile,
		int **personIndexPtr, Ped *pedigree, int *numObserved) {

	ifstream *inFile = new ifstream;

	cout << endl;
	cout << "Initialize BEAGLE dense marker stream.\n";
	//stream for dense marker file

	inFile->open(denseMarkersFile);
	if (!(*inFile)) {
		cout << "Unable to open the dense marker file. Exit now.\n";
		cout << denseMarkersFile << endl;
		exit(1); // terminate with error
	}

	int verbose = 0;
	char *pch;
	char *pch2;

	//inFile->seekg(*readPos, ios::beg);
	string line;
	char *line_c;
	//while(getline(*inFile,line)!=NULL ){
	getline(*inFile, line);

	if (line.empty()) {
		//just ignore empty line
		cout << "the file is empty.\n";
	} else {

		char *lineStr = new char[line.length() + 1];

		//count the number of observed people
		char *copyStr = new char[line.length() + 1];
		strcpy(copyStr, line.c_str());
		pch = strtok(copyStr, " "); //ignore the first column
//assume the second column doesn't have interesting thing
//		pch = strtok(NULL, " "); //ignore the second column

		while (pch != NULL) {
			pch = strtok(NULL, " "); //the first person
			pch = strtok(NULL, " ");
			if (pch != NULL) {
				(*numObserved)++;
			}
		}

//		cout << "Number of observed individuals is " << *numObserved << endl;
		int *personIndex = new int[*numObserved];
		//----------------------------------------

		strcpy(lineStr, line.c_str());
		pch = strtok(lineStr, " "); //ignore the first column
		//cout << pch << endl;
//assume only 1 column.
//		pch = strtok(NULL, " "); //ignore the second column
//		cout << pch << endl;
		for (int i = 0; i < *numObserved; i++) {
			pch = strtok(NULL, " "); //the first person
			pch2 = strtok(NULL, " "); //the second allele of the same person
			int compare2strings = strcmp(pch, pch2);
			if (compare2strings == 0) {
				int indexNumber = pedigree->searchIndex(pch);
				if (verbose) {
					cout << "The index number of " << pch << " is "	<< indexNumber << endl;
				}
				personIndex[i] = indexNumber;
//cout <<"indexNumber is = " <<indexNumber <<endl;
				if (indexNumber == -1) {
					cout
							<< "Can't match the ID of the person to a person in the pedigree.\n";
				}
			} else {
				cout
						<< "The pair of ID that identifies the two alleles from same person is not the same. Check the header of the marker file.\n";
			}

		}//for

		*personIndexPtr = personIndex;

	} //non-empty line
	//update the latest read position
	//}  //getline


	return (inFile);
}

/**
 * Read BEAGLE formatted dense marker
 *
 * Input: fileStream
 *
 */
FGL_Matrix *readBEAGLE_denseMarkers_oneMarker(ifstream *inFile, Ped *pedigree,
		int *observedIndex, int numObserved, char **markerNamePtr) {
	//cout<<"number of dense markers is: " << num_denseMarkers <<endl;
	int num_denseMarkers = 1;
	int curMarker = 0;

	int verbose = 0;
	char *pch;
	char *pch2;

	FGL_Matrix *denseMarkers = new FGL_Matrix_int;
	denseMarkers->FGL_Matrix_INIT(pedigree->n_ind, num_denseMarkers);

	string line;

	getline(*inFile, line);
//cout<<line <<endl;
	char *nameChar = new char[30];

//cout<<"hihi\n";
	if (line.empty()) {
		//just ignore empty line
		cout
				<< "the dense marker file has an empty line. Something is wrong. \n";
	} else {
		char *lineStr = new char[line.length() + 1];

		strcpy(lineStr, line.c_str());
		pch = strtok(lineStr, " "); //ignore the first column
		//cout<<pch<<endl;

//Assume we only have 1 column
//		pch = strtok(NULL, " "); //ignore the second column
		//cout<<pch<<endl;

		strcpy(nameChar, pch);

//cout<<"yoyo\n";
//cout<<"number observed = " <<numObserved <<endl;
		for (int i = 0; i < numObserved; i++) {
			pch = strtok(NULL, " "); //the first person

//cout<< observedIndex[i] <<endl;
//cout<<pch <<" " <<endl;

			denseMarkers->write2Matrix(atoi(pch), observedIndex[i], 0, curMarker);
			pch2 = strtok(NULL, " "); //the second allele of the same person
//cout<<pch2 <<" " <<endl;
			denseMarkers->write2Matrix(atoi(pch2), observedIndex[i], 1, curMarker);
		}
	delete lineStr;
	}

//cout<<"well\n";
//exit(0);
	*markerNamePtr = nameChar;
	//delete nameChar;
//cout<<"marker name ptr is: " << *markerNamePtr;

	//update the latest read position
	//}  //getline
	//cout << endl;
	if (verbose) {
		for (int i = 0; i < pedigree->n_ind; i++) {
			cout << pedigree->getPedMeio(i).id << ": ";
			for (int j = 0; j < num_denseMarkers; j++) {
				cout << denseMarkers->getValue(i, 0, j) << "/"
						<< denseMarkers->getValue(i, 1, j) << " ";
			}
			cout << endl;
		}
		//cout << "readpos=" << *readPos <<endl;
	}
//exit(0);
	//cout<< inFile->tellg() <<endl;

	return (denseMarkers);
}

/**
 * Close the denseMarkersStream.
 */
void closeBEAGLE_denseMarkersStream(ifstream *inFile) {
	inFile->close();
	delete inFile;
}

/**
 * added error check: if can't locate the person, print error message and quit.
 *
 *read one component of dense marker
 * Read dense markers into FGL - note that the reading.. assumes same order as in the pedigree
 * WRONG! not everybody is typed for the dense markers.. only a selected few are!, so what that means is, I  have to match the correct id
 */
//also need to match the id
FGL_Matrix *readDenseMarkers_byComponent(char *denseMarkersFile, Ped *pedigree,
		int num_denseMarkers) {
	//cout<<"number of dense markers is: " << num_denseMarkers <<endl;
	//cout << "Read dense markers from: ";
	int verbose = 0;
	char *pch;
char *ind_ID;
	//stream for dense marker file
	ifstream *inFile = new ifstream;
	inFile->open(denseMarkersFile);
	if (!(*inFile)) {
		cout << "Unable to open the dense marker file. Exit now.\n";
		cout << denseMarkersFile << endl;
		exit(1); // terminate with error
	}
	//inFile->seekg(*readPos, ios::beg);
	string line;
	char *line_c;

	int foundLine = 0;
	FGL_Matrix *denseMarkers = new FGL_Matrix_int;
	denseMarkers->FGL_Matrix_INIT(pedigree->n_ind, num_denseMarkers);

	while (getline(*inFile, line) != NULL) {

		if (line.empty()) {
			//just ignore empty line
		} else {

			int curMarker = 0;
			char *markersToKeepStr = new char[line.length() + 1];
			strcpy(markersToKeepStr, line.c_str());
			ind_ID = strtok(markersToKeepStr, " ");

			int indexNumber = pedigree->searchIndex(ind_ID);
			//cout<<"index number is " <<indexNumber <<endl;

			if (verbose) {
				cout << "The index number of " << ind_ID << " is " << indexNumber
						<< endl;
			}

			if (indexNumber == -1) {
				cout <<" Can't match individual " << ind_ID << " in the genotype file "<<denseMarkersFile << " to an ID in the pedigree.\n";
				cout<<"Make sure that the IDs match!\n";
				cout<<"Exit with error.\n";
				exit(0);
				//cout <<"Can't match the individual to the pedigree file\n";
				//it will signal to stop reading. Moreover, it will lead the cursor back to the previous ending position.
				//inFile->seekg(*readPos, ios::beg);
				//break;
			} else {
//				cout << pch << "\t";
				for (curMarker = 0; curMarker < num_denseMarkers; curMarker++) {
					pch = strtok(NULL, " ");
					if(pch == NULL){
						cout <<"The number of columns in the genotype file does not match the number of dense markers.("<<num_denseMarkers <<")\n";
						cout<<"Please check that the line starting with ID=" << ind_ID <<" contains 1+" <<"(2*"<<num_denseMarkers <<")= "<<1+(2*num_denseMarkers)  <<" columns.\n";
						cout<<"Exit with error.\n";
						exit(0);
					}
					denseMarkers->write2Matrix(atoi(pch), indexNumber, 0, curMarker);

					pch = strtok(NULL, " ");
					if(pch == NULL){
						cout <<"The number of columns in the genotype file does not match the number of dense markers.("<<num_denseMarkers <<")\n";
						cout<<"Please check that the line starting with ID=" << ind_ID <<" contains 1+" <<"(2*"<<num_denseMarkers <<")= "<<1+(2*num_denseMarkers)  <<" columns.\n";
						cout<<"Exit with error.\n";
						exit(0);
					}
					denseMarkers->write2Matrix(atoi(pch), indexNumber, 1, curMarker);
				}
				//*readPos = inFile->tellg();

			}

		} //non-empty line
		//update the latest read position
	} //getline
	//cout << endl;
	if (verbose) {

		for (int j = 0; j < num_denseMarkers; j++) {
			cout << "marker " << j << endl;
			for (int i = 0; i < pedigree->n_ind; i++) {
				cout << pedigree->getPedMeio(i).id << ": ";
				cout << denseMarkers->getValue(i, 0, j) << "/"
						<< denseMarkers->getValue(i, 1, j) << " ";
				cout << endl;
			}
		}
		//cout << "readpos=" << *readPos <<endl;
	}
	//cout<< inFile->tellg() <<endl;
	inFile->close();
	delete inFile;
	return (denseMarkers);
}

//http://www.linuxselfhelp.com/HOWTO/C++Programming-HOWTO-7.html
void Tokenize(const string& str, vector<string>& tokens,
		const string& delimiters = " ") {
	// Skip delimiters at beginning.
	string::size_type lastPos = str.find_first_not_of(delimiters, 0);
	// Find first "non-delimiter".
	string::size_type pos = str.find_first_of(delimiters, lastPos);

	while (string::npos != pos || string::npos != lastPos) {
		// Found a token, add it to the vector.
		tokens.push_back(str.substr(lastPos, pos - lastPos));
		// Skip delimiters.  Note the "not_of"
		lastPos = str.find_first_not_of(delimiters, pos);
		// Find next "non-delimiter"
		pos = str.find_first_of(delimiters, lastPos);
	}
}

void readAllelicTypeCount(char *afFile, int num_denseMarkers,
		int *countAllelicType) {
	ifstream inFile;
	inFile.open(afFile);
	if (!inFile) {
		cout << "Unable to open the allele frequency file. Exit now.\n";
		exit(1); // terminate with error
	}

	char *pch;
	string line;
	char *line_c;

	//int *countAllelicType = new int[num_denseMarkers];

	for (int curLine = 0; curLine < num_denseMarkers; curLine++) {
		// cout << line <<endl;
		getline(inFile, line);
		char *markersToKeepStr = new char[line.length() + 1];
		strcpy(markersToKeepStr, line.c_str());
		pch = strtok(markersToKeepStr, " ");

		int num_allelictypes = 0;
		while (pch != NULL) {
			num_allelictypes++;
			//		  atoi(pch)
			//cout <<pch <<endl;
			pch = strtok(NULL, " ");
		}
		countAllelicType[curLine] = num_allelictypes;
		delete markersToKeepStr;
	}

	inFile.close();

}

/** read the allele frequency file
 *
 */
void readAF(char *afFile, int num_denseMarkers, double **alleleFreq,
		int *countAllelicType) {

	//Ask whether the allele frequency vector is available - we can only make a probabilistic inference if it is available
	//A. load the allele frequency in [be flexible in the number of allelic types]
	//char *afFile = "/storage/Dropbox/RA/traitSimulation/temp/dense_af.txt";
	ifstream inFile;
	char *pch;
	string line;

/*
	inFile.open(afFile);
	if (!inFile) {
		cout << "Unable to open the allele frequency file. Exit now.\n";
		exit(1); // terminate with error
	}

	char *pch;
	string line;
	char *line_c;

	//int *countAllelicType = new int[num_denseMarkers];

	for (int curLine = 0; curLine < num_denseMarkers; curLine++) {
		// cout << line <<endl;
		getline(inFile, line);
		char *markersToKeepStr = new char[line.length() + 1];
		strcpy(markersToKeepStr, line.c_str());
		pch = strtok(markersToKeepStr, " ");

		int num_allelictypes = 0;
		while (pch != NULL) {
			num_allelictypes++;
			//		  atoi(pch)
			//cout <<pch <<endl;
			pch = strtok(NULL, " ");
		}
		countAllelicType[curLine] = num_allelictypes;
	}

	inFile.close();
*/

	inFile.open(afFile);
	if (!inFile) {
		cout << "Unable to open the allele frequency file. Exit now.\n";
		exit(1); // terminate with error
	}

	// alleleFreq = new double*[ (num_denseMarkers) ];
	//for(int cur_marker = 0 ; cur_marker < num_denseMarkers; cur_marker++){
	//	alleleFreq[cur_marker] = new double[countAllelicType[cur_marker]];
	//}

	for (int curLine = 0; curLine < num_denseMarkers; curLine++) {
		// cout << line <<endl;
		getline(inFile, line);
		char *markersToKeepStr = new char[line.length() + 1];
		strcpy(markersToKeepStr, line.c_str());
		pch = strtok(markersToKeepStr, " ");

		int cur_a = 0;
		double sumAF=0;
		while (pch != NULL) {
			alleleFreq[curLine][cur_a] = atof(pch);
			sumAF+=alleleFreq[curLine][cur_a];
			pch = strtok(NULL, " ");
			cur_a++;
		}
		if( (sumAF-1) > 0.0000001){

			cout.precision(10);
			cout<< "Allele frequencies of marker " << curLine+1 <<" does not sum to 1. Instead, it sums to "<<sumAF <<"\n";
			for(int i=0; i < countAllelicType[curLine]; i++){
				cout<< alleleFreq[curLine][i] <<" ";
			}
			cout<< endl;
			cout <<"Exit with error.\n";
			exit(0);
		}
		delete markersToKeepStr;
	}

	inFile.close();

	/*
	 for(int curLine=0; curLine < num_denseMarkers; curLine++){
	 for(int cur_a=0; cur_a < countAllelicType[curLine]; cur_a++){
	 cout << alleleFreq[curLine][cur_a] << " ";
	 }
	 cout <<endl;
	 }
	 exit(0);
	 */
}


//optimized version:
//check a single marker specified by the curMarkerIndex with a FGL - 1 iteration
/**
 * Main routine used to calculate the likelihood of observed data given an ibd-graph
 */
double calculate_graph_likelihood(FGL_Matrix *denseMarkers, Ped *pedigree,
		FGL_Matrix *fglmatrix, int cur_marker, double *alleleFreq, int DEBUG,
		int *num_graphs, double ***likelihoodPtr, int *inconsistent,
		double *likelihood_resolved, int **resolvedPtr, int **visitedPtr,
		int ***fglAlleleMatchPtr, int CAN_CUT_SHORT) {

	double **likelihood;
	int ***fglAlleleTable;
	int **fglAlleleMatch;
	int *resolved; //-1 if FGL is unresolved.. if it is resolved, it will be assigned the allele
	int *visited; //in the IBDG routine, we will associate -2 with unobserved FGL, -1 with unvisited - which may be because it has already been resolved or we have no data to suggest which allele it could be

	*num_graphs = 0;
	*inconsistent = 0; //indicator of whether the FGL is consistent with the observed data
	//int DEBUG = 0;
	int verbose = 0;

	int displayError = 0;
	int displayOutput = 0;

	int allObservedFGLResolved = 0;

	int numInd = pedigree->n_ind;

	int num_founders = 0;
	for (int i = 0; i < numInd; i++) {
		if (pedigree->isFounder(i) == 1) {
			num_founders++;
		}
	}

	int NUM_FGLs = num_founders * 2; // ? how can i figure that out? - number of founders x2
	resolved = new int[NUM_FGLs];
	for (int cur_FGL = 0; cur_FGL < NUM_FGLs; cur_FGL++) {
		resolved[cur_FGL] = -1;
	}

	int *observedFGL = new int[NUM_FGLs];
	for (int cur_FGL = 0; cur_FGL < NUM_FGLs; cur_FGL++) {
		observedFGL[cur_FGL] = 0;
	}

	for (int cur_ind = 0; cur_ind < numInd; cur_ind++) {
		int isObserved = (denseMarkers->getValue(cur_ind, 0, cur_marker) != 0)
				&& (denseMarkers->getValue(cur_ind, 1, cur_marker) != 0);

		if (isObserved == 1) {

			observedFGL[fglmatrix->getValue(cur_ind, 0, 0) - 1] = 1;
			observedFGL[fglmatrix->getValue(cur_ind, 1, 0) - 1] = 1;
			int locus0 = denseMarkers->getValue(cur_ind, 0, cur_marker);
			//if homozygous
			if (denseMarkers->getValue(cur_ind, 0, cur_marker)
					== denseMarkers->getValue(cur_ind, 1, cur_marker)) {
				//note: for each locus, we assume it is either genotyped at both alleles, or completely missing
				//if it is not homozygote for missing!
				if (locus0 != 0) {
					//given we already resolved, the resolved allele is not equal to what we see from the homozygous marker
					if ((resolved[fglmatrix->getValue(cur_ind, 0, 0) - 1] != -1)
							&& (resolved[fglmatrix->getValue(cur_ind, 0, 0) - 1]
									!= locus0)) {
						*inconsistent = 1;
						//return(*inconsistent);
						if (displayError == 1) {
							cout
									<< "*inconsistency with previous assignment at the same FGL "
									<< fglmatrix->getValue(cur_ind, 0, 0)
									<< " and marker " << cur_marker << endl;
							cout << "->overwrite anyway\n";
							cout << "marker index: " << cur_marker << "\n";
						}//displayError
						break;
					}
					if ((resolved[fglmatrix->getValue(cur_ind, 1, 0) - 1] != -1)
							&& (resolved[fglmatrix->getValue(cur_ind, 1, 0) - 1]
									!= locus0)) {
						*inconsistent = 1;
						//return(*consistent);
						if (displayError == 1) {
							cout
									<< "*inconsistency with previous assignment at the same FGL "
									<< fglmatrix->getValue(cur_ind, 1, 0)
									<< " and marker " << cur_marker << endl;
							cout << "->overwrite anyway\n";
							cout << "marker index: " << cur_marker << "\n";
						}//displayError
						break;
					}
					//write to the resolved FGL table
					int one = fglmatrix->getValue(cur_ind, 0, 0);
					int two = fglmatrix->getValue(cur_ind, 1, 0);
					resolved[one - 1] = locus0;
					resolved[two - 1] = locus0;
					if (DEBUG) {
						cout << " at marker index " << cur_marker
								<< ", resolved FGLs " << one << " and " << two
								<< " belonging to " << pedigree->getPedMeio(
								cur_ind).id << endl;
					}

				} //if it is not homozygous for missing
			} //if homozygous
		}
		//if(*inconsistent ==1){
		//cout<<"need to break!\n";
		//break;
		//}
	} //iteration of each person

	//cout<<"inconsistent = " << *inconsistent <<endl;
	//for(int i=0; i < NUM_FGLs; i++){
	//	cout << i << ": " <<resolved[i] <<endl;
	//}
	int isUpdating = 1;
	while (isUpdating == 1) {
		isUpdating = 0; //reset update flag to 0
		for (int cur_ind = 0; cur_ind < numInd; cur_ind++) {
			int isObserved = (denseMarkers->getValue(cur_ind, 0, cur_marker)
					!= 0) && (denseMarkers->getValue(cur_ind, 1, cur_marker)
					!= 0);
			if (isObserved == 1) {

				//looking up the FGLs of the person of current interest
				int one = fglmatrix->getValue(cur_ind, 0, 0);
				int two = fglmatrix->getValue(cur_ind, 1, 0);
				if (DEBUG) {
					cout << "We observe marker " << cur_marker << " from "
							<< pedigree->getPedMeio(cur_ind).id << " with FGL "
							<< one << " and " << two << endl;
				}

				//if both alleles have been resolved - simply check if they agree with the data
				if ((resolved[one - 1] != -1) && (resolved[two - 1] != -1)) {
					if (((denseMarkers->getValue(cur_ind, 0, cur_marker)
							== resolved[one - 1]) && (denseMarkers->getValue(
							cur_ind, 1, cur_marker) == resolved[two - 1]))
							|| ((denseMarkers->getValue(cur_ind, 1, cur_marker)
									== resolved[one - 1])
									&& (denseMarkers->getValue(cur_ind, 0,
											cur_marker) == resolved[two - 1]))) {
						//then that's fine
					} else {
						if (displayError) {
							cout << "***Conflict for " << pedigree->getPedMeio(
									cur_ind).id << " at marker index "
									<< cur_marker << endl;
							//for(int in=0; in <  pedigree->n_ind; in++){
							//cout<<pedigree->getPedMeio(in).id <<": FGL:" <<one <<"/" <<two <<"  marker:" << denseMarkers->getValue(in,0, cur_marker) <<"," <<denseMarkers->getValue(in,1, cur_marker) <<endl;
							//}
						}
						*inconsistent = 1;
						//return(*inconsistent);
						break;
					}

				}

				//if one of the alleles has been solved BUT NOT BOTH, we try to resolve the other
				else if (((resolved[one - 1] != -1)
						|| (resolved[two - 1] != -1)) && !((resolved[one - 1]
						!= -1) && (resolved[two - 1] != -1))) {
					//find the other one to resolve
					if (resolved[one - 1] != -1) {
						if (denseMarkers->getValue(cur_ind, 0, cur_marker)
								== resolved[one - 1]) {
							resolved[two - 1] = denseMarkers->getValue(cur_ind,
									1, cur_marker);
						} else {
							resolved[two - 1] = denseMarkers->getValue(cur_ind,
									0, cur_marker);
						}
					}
					//resolved[cur_marker][one] is not -1
					else {
						if (denseMarkers->getValue(cur_ind, 0, cur_marker)
								== resolved[two - 1]) {
							resolved[one - 1] = denseMarkers->getValue(cur_ind,
									1, cur_marker);
						} else {
							resolved[one - 1] = denseMarkers->getValue(cur_ind,
									0, cur_marker);
						}
					}

					//if after resolving, BOTH ARE RESOLVED - which will be
					if ((resolved[one - 1] != -1) && (resolved[two - 1] != -1)) {
						isUpdating = 1;
						if (DEBUG) {
							cout << "->Resolved! " << pedigree->getPedMeio(
									cur_ind).id << "\n";
						}
					} else {
						cout
								<< "Unanticipated error in matching the other allele while updating the deterministic part of the FGL-allele table\n";
						exit(0);
					}
				}

			} //only try to resolve if this person is observed
			if (*inconsistent == 1) {
				//cout<<"should have broken already ";
				break;
			}
		}//for this iteration on all individuals

		if (*inconsistent == 1) {
			//cout<<"have not broken out of the while loop yet. now break ";
			break;
		}
	}//while loop ended for updating

	*likelihood_resolved = 1;

	//if all FGLs were resolved OR incopatibility is deected, we will return - we don't need to process the IBDG routine.

	//determine if all FGLs are already resolved, given that the data is sinconsistent with the FGLs so-far
	allObservedFGLResolved = 1;
	if (*inconsistent == 0) {
		for (int cur_FGL = 0; cur_FGL < NUM_FGLs; cur_FGL++) {
			if (observedFGL[cur_FGL] == 1) {
				if (resolved[cur_FGL] == -1) {
					allObservedFGLResolved = 0;
					break;
				}
			}
		}
	}

	if (*inconsistent == 1) {
		*likelihood_resolved = 0;
	}
	//calculate the likelihood of the resolved FGLs
	else {
		for (int cur_FGL = 0; cur_FGL < NUM_FGLs; cur_FGL++) {
			if (observedFGL[cur_FGL] == 1) {
				if (resolved[cur_FGL] != -1) {
					*likelihood_resolved *= alleleFreq[resolved[cur_FGL] - 1];
					//cout<< resolved[cur_FGL] << "and af=" << alleleFreq[resolved[cur_FGL] -1] <<" and partial likelihood = " << *likelihood_resolved <<"\n";
				}
			}
		}
	}

	delete[] observedFGL;

	if (*inconsistent == 1 || allObservedFGLResolved == 1) {
		//deallocate variables first
		//delete []observedFGL; -delete at the end
		//return(*inconsistent);
		//cout << *likelihood_resolved <<endl;
		if (DEBUG) {
			cout << "observed likelihood=" << *likelihood_resolved << endl;
		}

		//to quit immediately if everyone already resolved. -- for error detection
		if (CAN_CUT_SHORT == 1) {
			*resolvedPtr = resolved;
			return (*likelihood_resolved); //current implementation: uncommented it
		}

		//return(0);
		//cout<<"hi ya\n";
	}
	//else{
	//cout<<"not all resolved yet\n";
	//}

	//--------------------------------------------------------------------

	//IBDG routine:
	//adjacency list  - FGL minus 1 for indexing purpose
	list<int> *adjList = new list<int> [NUM_FGLs]; //note: this variable only add connections between observed FGLs, and it doesn't add all the connections - just enough so that every observed FGL will be traversed.
	list<int> *trackIndList = new list<int> [NUM_FGLs];
	list<int> observedIndList;

	for (int cur_ind = 0; cur_ind < pedigree->n_ind; cur_ind++) {
		int allele1 = denseMarkers->getValue(cur_ind, 0, cur_marker);
		int allele2 = denseMarkers->getValue(cur_ind, 1, cur_marker);
		int isObserved = (allele1 != 0) && (allele2 != 0);
		if (isObserved == 1) {
			observedIndList.push_back(cur_ind);

			int one = fglmatrix->getValue(cur_ind, 0, 0);
			int two = fglmatrix->getValue(cur_ind, 1, 0);

			if (DEBUG) {
				cout << "person " << pedigree->getPedMeio(cur_ind).id << "("
						<< cur_ind << ") : " << denseMarkers->getValue(cur_ind,
						0, cur_marker) << ", " << denseMarkers->getValue(
						cur_ind, 1, cur_marker) << "\t" << "FGL: " << one - 1
						<< "/" << two - 1 << endl;
			}
			//cout << resolved[one -1] <<" and " <<resolved[two -1] <<endl;
			if ((resolved[one - 1] == -1) && (resolved[two - 1] == -1)) { //they weren't resolved.
				//if both FGLs were observed already but have different genotypes:
				bool isDup = false;
				list<int>::iterator t1 = trackIndList[one - 1].begin();
				for (list<int>::iterator iter = adjList[one - 1].begin(); iter
						!= adjList[one - 1].end(); ++iter) {
					if (*iter == (two - 1)) {
						int ind = *t1;
						int ind_a1 = denseMarkers->getValue(ind, 0, cur_marker);
						int ind_a2 = denseMarkers->getValue(ind, 1, cur_marker);
						//check whether the duplicated set of FGLs contained the same observed alleles
						//pull out the pair of alleles...
						if ((ind_a1 == allele1 && ind_a2 == allele2) || (ind_a1
								== allele2 && ind_a2 == allele1)) {
							isDup = true;
							//  cout <<"dup = true\n";
						} else {
							*inconsistent = 1; //should return right away also.
							*likelihood_resolved = 0;
							//						cout<<"weird, not taken care of yet\n";

							//							cout << "past ind_a1/ind_a2=" << ind_a1 << "/"
							//									<< ind_a2 << ", and current alleles="
							//									<< allele1 << "/" << allele2 << "\n";
							//							cout << "decrepancy noted.\n";

							//						for(int i=0; i< pedigree->n_ind; i++){
							//							 one = fglmatrix->getValue(i, 0, 0);
							//							 two = fglmatrix->getValue(i, 1, 0);
							//				cout << "person " << pedigree->getPedMeio(i).id << "("
							//						<< i << ") : " << denseMarkers->getValue(i,
							//						0, cur_marker) << ", " << denseMarkers->getValue(
							//						i, 1, cur_marker) << "\t" << "FGL: " << one - 1
							//						<< "/" << two - 1   << endl;
							//						}
							//						for(int i=0; i < NUM_FGLs; i++){
							//							cout << "FGL" << i <<": " << resolved[i] <<endl;
							//						}
							//						exit(0);


						}

					}
					t1++;
				}
				if (isDup == false) {
					//cout<<"add to adjacency and individual tracking list\n";
					//add to adjacency list
					adjList[one - 1].push_back(two - 1);
					adjList[two - 1].push_back(one - 1);

					//add to individual tracking list
					trackIndList[one - 1].push_back(cur_ind);
					trackIndList[two - 1].push_back(cur_ind);
				}
			}//if both FGLs are NOT resolved - purpose: only add non-resolved FGLs to this IBDG
		} //if the person is observed
	}

	//FOR DEBUGGING
	if (DEBUG) {
		cout
				<< "Printing out the unresolved FGLs that would be put on a graph:\n";
		//fgl begins at 1
		for (int cur_fgl = 0; cur_fgl < NUM_FGLs; cur_fgl++) {
			list<int>::iterator iter;
			cout << cur_fgl << ": ";
			for (iter = adjList[cur_fgl].begin(); iter
					!= adjList[cur_fgl].end(); ++iter) {
				cout << *iter << " ";
			}
			cout << endl;
		}
		cout << "====================\n";
		//FOR DEBUGGING
	}

	//determine the partitions
	//fgl has been subtracted by 1.
	visited = new int[NUM_FGLs]; //store which graph it belongs to
	for (int i = 0; i < NUM_FGLs; i++) {
		visited[i] = -1;//-1 = unvisited
	}

	int graph_label = -1; //graph label starts at 0

	list<list<int> > graphList;
	list<list<int> > preList;
	list<list<int> > fromList;

	//process fgl 1 by 1. if the current FGL hasn't been put to a graph yet, first check if it has any
	for (int cur_fgl = 0; cur_fgl < NUM_FGLs; cur_fgl++) {

		//if haven't visited, attempt to visit
		if (visited[cur_fgl] == -1) {
			//start graph at this index
			int numElement = adjList[cur_fgl].size();
			//if it is a part of a non-empty graph, determine all the elements of the graph; keep track of the preceeding element
			if (numElement > 0) {
				//Breadth-first traversal

				//start recording:
				list<int> curQueue;
				list<int> saveQueue;
				list<int> preStep;
				list<int> fromInd;

				graph_label++; //label of the next graph

				visited[cur_fgl] = graph_label;

				curQueue.push_back(cur_fgl);
				saveQueue.push_back(cur_fgl);
				//    // preStep.push_back(-1); //that means it is the root  - no need
				//    //fromInd.push_back(-1); //uninteresting, root  --- no need
				while (!curQueue.empty()) {
					int curElement = curQueue.front();
					curQueue.pop_front();

					list<int>::iterator iter = adjList[curElement].begin();
					list<int>::iterator iter2 =
							trackIndList[curElement].begin(); //parallel list
					//for(iter=adjList[curElement ].begin(); iter != adjList[curElement].end(); ++iter){
					for (int i = 0; i < adjList[curElement].size(); i++) {

						if (visited[*iter] == -1) {
							curQueue.push_back(*iter);
							saveQueue.push_back(*iter);
							preStep.push_back(curElement);
							fromInd.push_back(*iter2);
							visited[*iter] = graph_label;
						}
						++iter;
						++iter2;

					}//close for

				}//close while
				//now we have finished making the graph, we should add it to the list of graph
				graphList.push_back(saveQueue); //maybe this won't work.
				preList.push_back(preStep);
				fromList.push_back(fromInd);

			} //close number of element greater than 0

			//we just didn't observe any data for it
			else {
				//if not resolved yet...
				if (resolved[cur_fgl] == -1) {
					visited[cur_fgl] = -2;
				}
			}

		}//close if visit
	}//close for loop


	//after this function, we have assigned a graph number to every ambiguous FGL
	//for(int i=0; i< NUM_FGLs; i++){
	//cout <<"fgl index "  <<i <<": " <<  visited[i] <<endl;

	//}

	// 1. Enqueue the root node.
	//2. Dequeue a node and examine it.
	//     * If the element sought is found in this node, quit the search and return a result.
	//    * Otherwise enqueue any successors (the direct child nodes) that have not yet been discovered.
	//3. If the queue is empty, every node on the graph has been examined – quit the search and return "not found".
	//4. Repeat from Step 2.

	//list<int> saveQueue = graphList.front();
	*num_graphs = graphList.size();
	list<list<int> >::iterator myGList = graphList.begin();
	list<list<int> >::iterator myPList = preList.begin();
	list<list<int> >::iterator myFList = fromList.begin();

	if (DEBUG) {
		//Print graphs queue
		//YOU REALLY WANT TO ITERATE THE GRAPH. YOU DON'T WANT TO ERASE THE GRAPH.
		cout << "Total number of graphs is " << graphList.size() << endl;
		for (int curGraph = 0; curGraph < *num_graphs; curGraph++) {
			cout << "Graph " << curGraph + 1 << "\n";

			cout << "G: ";
			for (list<int>::iterator iter = myGList->begin(); iter
					!= myGList->end(); ++iter) {
				cout << *iter << " ";
			}
			cout << endl;

			cout << "P: ";
			for (list<int>::iterator iter = myPList->begin(); iter
					!= myPList->end(); ++iter) {
				cout << *iter << " ";
			}
			cout << endl;

			cout << "F: ";
			for (list<int>::iterator iter = myFList->begin(); iter
					!= myFList->end(); ++iter) {
				cout << *iter << " ";
			}
			cout << endl;

			++myGList;
			++myPList;
			++myFList;
		}
		cout
				<< "=============================================================================\n";
	}

	//C.for each partition,
	//enumerate the FGL labels efficiently and attach a probability to each
	myFList = fromList.begin();
	myGList = graphList.begin();
	myPList = preList.begin();

	//there are two possibilities in each subgraph. i want to store both possibilities and attach a probability to each combination
	fglAlleleTable = new int**[*num_graphs]; //each cell records the allelic type of a particular FGL. each sequence (row) forms a possibility
	list<list<int> >::iterator myGList2 = graphList.begin();
	for (int i = 0; i < *num_graphs; i++) {

		fglAlleleTable[i] = new int*[2];

		fglAlleleTable[i][0] = new int[myGList2->size()];
		fglAlleleTable[i][1] = new int[myGList2->size()];
		for (int j = 0; j < myGList2->size(); j++) {
			fglAlleleTable[i][0][j] = -1;
			fglAlleleTable[i][1][j] = -1;
		}
		myGList2++;
	}

	fglAlleleMatch = new int*[2];
	for (int i = 0; i < 2; i++) {
		fglAlleleMatch[i] = new int[NUM_FGLs];
		for (int j = 0; j < NUM_FGLs; j++) {
			fglAlleleMatch[i][j] = -1;
		}
	}

	likelihood = new double*[*num_graphs];
	for (int i = 0; i < *num_graphs; i++) {
		likelihood[i] = new double[2];
		likelihood[i][0] = 1;
		likelihood[i][1] = 1;
	}
	int success1, success2;
	for (int curGraph = 0; curGraph < *num_graphs; curGraph++) {

		map<int, int> fgl_index_table;
		int cur_label = 0;
		for (list<int>::iterator iter = myGList->begin(); iter
				!= myGList->end(); ++iter) {
			fgl_index_table[*iter] = cur_label;
			cur_label++;
		}

		if (DEBUG) {
			cout << "Graph " << curGraph + 1 << "\n";
			cout << "My F List has size " << myFList->size() << endl;
			cout << "Print FGL_INDEX_TABLE\n";
			for (list<int>::iterator iter = myGList->begin(); iter
					!= myGList->end(); ++iter) {
				cout << *iter << " - " << fgl_index_table[*iter] << endl;
			}
		}

		//enumerate joint FGL label

		int *tmpLabel = new int[myGList->size()];

		//for a diploid chromosome, there are only 2 possibilities for the allele FGL placement
		for (int curPerm = 0; curPerm < 2; curPerm++) {
			list<int>::iterator fit = myFList->begin();
			list<int>::iterator pit = myPList->begin();

			if (curPerm == 0) {
				//base on the first label
				//cout << denseMarkers->getValue(*fit, 0, cur_marker);  //condition on it being the first allele
				//cout<<" ";
				//cout << denseMarkers->getValue(*fit, 1, cur_marker);
				//cout<<" ";
				tmpLabel[0] = denseMarkers->getValue(*fit, 0, cur_marker);
				tmpLabel[1] = denseMarkers->getValue(*fit, 1, cur_marker);

			} else {
				//cout << denseMarkers->getValue(*fit, 1, cur_marker);  //condition on it being the first allele
				//cout<<" ";
				//cout << denseMarkers->getValue(*fit, 0, cur_marker);
				//cout<<" ";
				tmpLabel[0] = denseMarkers->getValue(*fit, 1, cur_marker);
				tmpLabel[1] = denseMarkers->getValue(*fit, 0, cur_marker);

			}

			//cout <<"before the while loop\n";


			//doesn't work
			//if(fit== *(myFList->end())  ){
			//	cout<<"graph done\n";
			//}
			int curF_index = 1;
			int success = 1;
			while (curF_index < myFList->size()) {
				fit++;
				pit++;
				int a1 = denseMarkers->getValue(*fit, 0, cur_marker);
				int a2 = denseMarkers->getValue(*fit, 1, cur_marker);
				int otherAllele = tmpLabel[fgl_index_table[*pit]]; //retrieve the previously determined allele
				if (otherAllele == a1) {
					//cout << a2;
					//cout <<" ";

					tmpLabel[curF_index + 1] = a2;
				} else if (otherAllele == a2) {
					//cout << a1;
					//cout <<" ";

					tmpLabel[curF_index + 1] = a1;
				} else {
					if (DEBUG) {
						cout << "Impossible configuration\n";
						cout << "other allele: " << otherAllele << endl;
						cout << "a1: " << a1 << endl;
						cout << "a2: " << a2 << endl;
						cout << "person " << *fit << "("
								<< pedigree->getPedMeio(*fit).id << ")" << endl;
					}
					success = 0;
					break;
					//exit(0);
				}
				curF_index++;
			}
			//cout << endl;

			if (success) {

				//when assigning alleles to FGLs, I ignored some connections in the IBDG. This routine is to make sure that all connections are competible with the FGL-alleles assignment.
				//for each individual
				for (list<int>::iterator iter = observedIndList.begin(); iter
						!= observedIndList.end(); ++iter) {
					//cout<< *iter <<" " ;
					int ind = *iter;
					int allele1 = denseMarkers->getValue(ind, 0, cur_marker);
					int allele2 = denseMarkers->getValue(ind, 1, cur_marker);
					int one = fglmatrix->getValue(ind, 0, 0) - 1;
					int two = fglmatrix->getValue(ind, 1, 0) - 1;

					int oneIndex = -1;
					int twoIndex = -1;
					int theIndex = 0;
					//inefficient already.. better to have a map
					for (list<int>::iterator gFGL = myGList->begin(); gFGL
							!= myGList->end(); ++gFGL) {
						if (*gFGL == one) {
							oneIndex = theIndex;
						}
						if (*gFGL == two) {
							twoIndex = theIndex;
						}
						theIndex++;
					}
					if (oneIndex != -1 && twoIndex != -1) {
						if ((tmpLabel[oneIndex] == allele1
								&& tmpLabel[twoIndex] == allele2)
								|| (tmpLabel[oneIndex] == allele2
										&& tmpLabel[twoIndex] == allele1)) {
							//it is okay do nothing
						} else {
							success = 0;
							likelihood[curGraph][curPerm] = 0; //bug fix on Jan4, 2011 - because back then it forgot to check that
						}
					}
				}
				//cout<<endl;

			}

			if (success) {
				//at this point, write tmpLabel to fglAlleleTable because such configuration is a possibility
				for (int i = 0; i < myGList->size(); i++) {
					fglAlleleTable[curGraph][curPerm][i] = tmpLabel[i];
				}

				int curMatch = 0;
				for (list<int>::iterator iter = myGList->begin(); iter
						!= myGList->end(); ++iter) {
					fglAlleleMatch[curPerm][*iter] = tmpLabel[curMatch];
					curMatch++;
				}
				//attach a probability
				for (int i = 0; i < myGList->size(); i++) {
					likelihood[curGraph][curPerm]
							= likelihood[curGraph][curPerm]
									* alleleFreq[fglAlleleTable[curGraph][curPerm][i]
											- 1]; //type index starts at 1, but storage index starts at 0
				}
				if (DEBUG) {
					cout << "success! print ";
					for (int i = 0; i < myGList->size(); i++) {
						cout << fglAlleleTable[curGraph][curPerm][i] << " ";
					}
					cout << endl;
				}
				if (DEBUG) {
					cout << " - likelihood of the graph is "
							<< likelihood[curGraph][curPerm] << endl;
				}
			} else {
				likelihood[curGraph][curPerm] = 0;
			}

			if (curPerm == 0) {
				success1 = success;
			} else {
				success2 = success;
			}

		} //for the two possibilities

		if (success1 == 0 && success2 == 0) {
			*inconsistent = 1;
		}

		//cout<<"-----------------------------------\n";
		++myFList;
		++myGList;
		++myPList;

		delete[] tmpLabel;

	}// next graph

	//deallocate variables
	delete[] adjList;
	delete[] trackIndList;

	for (int i = 0; i < *num_graphs; i++) {
		for (int j = 0; j < 2; j++) {
			delete[] fglAlleleTable[i][j];
		}
		delete[] fglAlleleTable[i];
	}
	delete[] fglAlleleTable;

	*likelihoodPtr = likelihood;

	*fglAlleleMatchPtr = fglAlleleMatch;
	*visitedPtr = visited;
	*resolvedPtr = resolved;

	//*graphListPtr = graphList;
	//current, when inconsistency is detected, remaining code might still run. I want to make it terminate immediately and properly...
}


/**
 * Calculate the likelihood of observed genotypes of a marker given the FGL
 */
double calculate_observed_likelihood(FGL_Matrix *denseMarkers, Ped *pedigree,
		FGL_Matrix *fglmatrix, int cur_marker, double *alleleFreq, int DEBUG) {

	int CAN_CUT_SHORT = 1;

	double ***likelihoodPtr = new double**;
	int ***fglAlleleMatchPtr = new int**;
	int **resolvedPtr = new int*;
	int **visitedPtr = new int*;

	int num_graphs;
	int inconsistent = 0;
	double likelihood_resolved = 1;

	//list<list<int> > *graphListPtr = new list<list<int> >;
	calculate_graph_likelihood(denseMarkers, pedigree, fglmatrix, cur_marker,
			alleleFreq, DEBUG, &num_graphs, likelihoodPtr, &inconsistent,
			&likelihood_resolved, resolvedPtr, visitedPtr, fglAlleleMatchPtr,
			CAN_CUT_SHORT); // resolved is -1 if it is not resolved and set to the allele code if it is resolved. visited is -1 if it is deterministic, -2 if it is unobserved, 0,1,2 if the FGL belongs to graph 1,2,or 3.
	// calculate_graph_likelihood(denseMarkers, pedigree, fglmatrix, cur_marker, alleleFreq, DEBUG, &num_graphs, likelihoodPtr,
	// fglAlleleTablePtr, &inconsistent, &likelihood_resolved, &allObservedFGLResolved,
	//resolvedPtr, visitedPtr , graphListPtr, fglAlleleMatchPtr); // resolved is -1 if it is not resolved and set to the allele code if it is resolved. visited is -1 if it is deterministic, -2 if it is unobserved, 0,1,2 if the FGL belongs to graph 1,2,or 3.
	//next change is to make sure that if every FGL is already resolved, i

	int all_determined = num_graphs == 0; //num_graphs = 0 implies that all observed alleles are matched to FGLs are determined

	double **likelihood = *likelihoodPtr;
	int **fglAlleleMatch = *fglAlleleMatchPtr;
	int *resolved = *resolvedPtr;
	int *visited = *visitedPtr;
	//list<list<int> > graphList = *graphListPtr;

	double likelihood_observed = likelihood_resolved;

	//calculate the likelihood
	for (int i = 0; i < num_graphs; i++) {
		double sumLikelihood = likelihood[i][0] + likelihood[i][1];
		if (DEBUG) {
			cout << "likelihood of graph " << i << " is " << sumLikelihood
					<< endl;
		}
		likelihood_observed *= sumLikelihood;
	}

	//de-alllocate variables
	if (all_determined == 0) {
		for (int i = 0; i < 2; i++) {
			delete[] fglAlleleMatch[i];
		}
		delete[] fglAlleleMatch;

		for (int i = 0; i < num_graphs; i++) {
			delete[] likelihood[i];
		}
		delete[] likelihood;
		delete[] visited; //should delete - currently just testing
	}
	delete[] resolved;

	if (DEBUG) {
		cout << "observed likelihood: " << likelihood_observed << "\n";
	}

	delete fglAlleleMatchPtr;
	delete likelihoodPtr;
	delete resolvedPtr;
	delete visitedPtr;

	return (likelihood_observed);
}

int codeFunction(int d1, int d2) {
	int b, s;
	//output from large to small
	if (d1 > d2) {
		b = d1;
		s = d2;
	} else {
		b = d2;
		s = d1;
	}
	return ((b - 1) * b / 2 + (s - 1));

}


//for version 1.04 of GIGI
//modificiation of the impute_genotypeProb to not require the initialization of the consistency matrix
//instead, it summarizes the number of consistent iterations
void impute_genotypeProbMod(FGL_Matrix *denseMarkers, Ped *pedigree,
		FGL_Matrix_List *fglm, double *alleleFreq, int countAllelicType,
		MTRand *mtrand1, double ***g_dist, int *numconsistency, int cur_marker) {
	int DEBUG = 0;
	int num_Iterations = fglm->n_Iterations;
	int numInd = pedigree->n_ind;

	int num_consistent_iterations = 0;

	for (int cur_it = 0; cur_it < num_Iterations; cur_it++) {
		//cout<<"cur_it = " <<cur_it <<"\n";
		FGL_Matrix *fglmatrix = fglm->getMatrix(cur_it);

		//if(cur_it==0){
		//for(int i=0; i<numInd; i++){
		//cout << "person " <<i  <<": "  << fglmatrix->getValue(i,0,0) <<"," <<fglmatrix->getValue(i,1,0) <<endl;
		//}
		//}

		int num_graphs = 0;
		int inconsistent = 0;
		double likelihood_resolved = 1;

		double ***likelihoodPtr = new double**;
		int ***fglAlleleMatchPtr = new int**;
		int **resolvedPtr = new int*;
		int **visitedPtr = new int*;
		//list<list<int> > *graphListPtr = new list<list<int> >;

		int CAN_CUT_SHORT = 0; //can't

		//cout<<"before calculating likelihood\n";
		calculate_graph_likelihood(denseMarkers, pedigree, fglmatrix,
				cur_marker, alleleFreq, DEBUG, &num_graphs, likelihoodPtr,
				&inconsistent, &likelihood_resolved, resolvedPtr, visitedPtr,
				fglAlleleMatchPtr, CAN_CUT_SHORT); // resolved is -1 if it is not resolved and set to the allele code if it is resolved. visited is -1 if it is deterministic, -2 if it is unobserved, 0,1,2 if the FGL belongs to graph 1,2,or 3.

		//calculate_graph_likelihood(denseMarkers, pedigree, fglmatrix, cur_marker, alleleFreq, DEBUG, &num_graphs, likelihoodPtr,
		//fglAlleleTablePtr, &inconsistent, &likelihood_resolved, &allObservedFGLResolved,
		//resolvedPtr, visitedPtr , graphListPtr, fglAlleleMatchPtr); // resolved is -1 if it is not resolved and set to the allele code if it is resolved. visited is -1 if it is deterministic, -2 if it is unobserved, 0,1,2 if the FGL belongs to graph 1,2,or 3.
		//cout<<"after calculating likelihood\n";
		//  calculate_graph_likelihood(denseMarkers, pedigree, fglmatrix, cur_marker, alleleFreq, DEBUG, &num_graphs, likelihoodPtr, fglAlleleTablePtr, &consistent, &likelihood_resolved, &allObservedFGLResolved
		//, resolvedPtr, visitedPtr, graphList);

		numconsistency[cur_marker] += inconsistent==0;
		double **likelihood = *likelihoodPtr;
		int **fglAlleleMatch = *fglAlleleMatchPtr;
		int *resolved = *resolvedPtr;
		int *visited = *visitedPtr;

		//list<list<int> > graphList = *graphListPtr;


		//impute:
		//D.finally, we do the imputation probabilistically...(output)
		//so in each iteration, we end up with several probabilities locus by locus {ie. AA, Aa, aA, aa}
		// use the probabilities as weights to calculate the  [estimate] of probabilitiy distribution of imputed genotypes for each individual
		//and we can choose to report the probability distribution or the most likely state

		//TESTING!!!!!! - DELETE WHEN /* */ is removed!
		//if(inconsistent==0){
		//num_inconsistent_iterations++;
		//}

		//only impute if observed markers are consistent with the FGLg
		if (inconsistent == 0) {
			//cout<<"inconsistent ==0\n";
			num_consistent_iterations++;
			for (int cur_ind = 0; cur_ind < numInd; cur_ind++) {
				//cout<<"imputing: cur_ind " << cur_ind <<endl;

				//int one = fglmatrix->getValue(cur_ind, 0, cur_marker);
				//int two = fglmatrix->getValue(cur_ind, 1, cur_marker);
				int one = fglmatrix->getValue(cur_ind, 0, 0); //fglm now only has 1 marker
				int two = fglmatrix->getValue(cur_ind, 1, 0);

				//cout <<resolved[one -1 ] <<"," << resolved[two -1 ] <<endl;

				if (DEBUG) {
					cout << "currently processing: " << pedigree->getPedMeio(
							cur_ind).id << ", whose first FGL is " << one
							<< ", second FGL is " << two << endl;
				}

				//both already resolved
				if (((resolved[one - 1] != -1) && (resolved[two - 1] != -1))) {
					if (DEBUG) {
						cout << pedigree->getPedMeio(cur_ind).id
								<< "->both resolved\n";
						cout << " --> " << resolved[one - 1] << " "
								<< resolved[two - 1] << endl;
					}

					//alleles need to be labeled as 1,2,3,4,5,.... in consecutive order
					//since it has been resolved, get it from the resolved table, NOT from the markers...
					int d1 = resolved[one - 1];
					int d2 = resolved[two - 1];
					int code = codeFunction(d1, d2);
					//cout<< "the code is: " <<code <<".  d1 is " <<d1 <<" and d2 is " <<d2 <<endl;
					g_dist[cur_ind][cur_marker][code] += 1; //add prob 1 as weight
				}
				//or less than 2 resolved
				else {
					if (DEBUG) {
						cout << "visited[one-1]=" << visited[one - 1] << endl;
						cout << "visited[two-1]=" << visited[two - 1] << endl;
					}
					//determined = -1 , in a graph, or singleton -2?
					if ((visited[one - 1] == visited[two - 1]) && (visited[one
							- 1] >= 0)) {
						if (DEBUG) {
							cout << pedigree->getPedMeio(cur_ind).id
									<< "->not resolved, both from same graph\n";
						}
						//from same graph
						//	cout<<"from same graph\n";
						//list<list<int> >::iterator myGList = graphList.begin();

						////roll to the right graph
						//for(int i=0; i< visited[one-1];i++){
						//++myGList;
						//}

						//list<int>::iterator iter=myGList->begin();
						int code;
						//int index1, index2, code;
						//int index=0;
						//for(list<int>::iterator iter=myGList->begin(); iter != myGList->end(); ++iter){
						////			cout << *iter <<" ";
						//if(*iter == (one-1)){
						//index1 = index;
						//}
						//if((*iter==(two-1))){
						//index2 = index;
						//}
						//index++;
						//}
						//cout<<endl;
						//if(DEBUG){
						//cout <<"index 1: "<<index1 <<" and index 2:" <<index2 <<endl;
						//}


						//both permutations would give the same genotype
						//int old_d1 = fglAlleleTable[visited[one-1]][0][index1];
						//int old_d2 = fglAlleleTable[visited[one-1]][0][index2];
						//int old_d1_2 = fglAlleleTable[visited[one-1]][1][index1];
						//int old_d2_2 = fglAlleleTable[visited[one-1]][1][index2];

						int d1 = fglAlleleMatch[0][one - 1];
						int d2 = fglAlleleMatch[0][two - 1];
						int d1_2 = fglAlleleMatch[1][one - 1];
						int d2_2 = fglAlleleMatch[1][two - 1];

						if (DEBUG) {
							cout << "d1:" << d1 << " and d2:" << d2 << endl;
							cout << "d1_2:" << d1_2 << " and d2_2:" << d2_2
									<< endl;
						}

						double sumLikelihood = likelihood[visited[one - 1]][0]
								+ likelihood[visited[one - 1]][1];

						if (DEBUG) {
							cout << "likelihood: " << likelihood[visited[one
									- 1]][0] << " and "
									<< likelihood[visited[one - 1]][1] << endl;
						}

						code = codeFunction(d1, d2);
						//add code
						if (likelihood[visited[one - 1]][0] != 0) {
							g_dist[cur_ind][cur_marker][code]
									+= likelihood[visited[one - 1]][0]
											/ sumLikelihood; //in this case we also know the genotype, even though unresolved.
						}

						//configuration 2
						code = codeFunction(d1_2, d2_2);
						//add code
						if (likelihood[visited[one - 1]][1] != 0) {
							g_dist[cur_ind][cur_marker][code]
									+= likelihood[visited[one - 1]][1]
											/ sumLikelihood; //in this case we also know the genotype, even though unresolved.
						}

						if (DEBUG) {
							cout << " add code " << code << " with prob "
									<< likelihood[visited[one - 1]][1]
											/ sumLikelihood << endl;
						}
					}

					else {
						//check what type of FGLs for this person, and an adapter for the probability of possible alleles
						//cout<<"RUN HERE\n";
						if (DEBUG) {
							cout << pedigree->getPedMeio(cur_ind).id
									<< "->not from the same graph \n";
						}

						int index1, index2, code;

						//int a1[countAllelicType[cur_marker]];
						//WHY IS IT ALLOWABLE? I THOUGHT IT MUST BE DECLARED AS A POINTER?
						double probOne[countAllelicType]; //adpater
						double probTwo[countAllelicType]; //adapter

						for (int i = 0; i < countAllelicType; i++) {
							probOne[i] = 0;
							probTwo[i] = 0;
						}

						//cout<<"ONE\n";
						//one
						if (visited[one - 1] >= 0) {
							//list<list<int> >::iterator myGList = graphList.begin();
							////roll to the right graph
							//for(int i=0; i< visited[one-1];i++){
							//++myGList;
							//}
							//list<int>::iterator iter=myGList->begin();
							//int index=0;
							//for(list<int>::iterator iter=myGList->begin(); iter != myGList->end(); ++iter){
							////		cout << *iter <<" ";
							//if(*iter == (one-1)){
							//index1 = index;
							//break;
							//}
							//index++;
							//}
							//cout<<endl;
							//cout<<index1 <<endl;
							double sumLikelihood =
									likelihood[visited[one - 1]][0]
											+ likelihood[visited[one - 1]][1];

							//				probOne[fglAlleleTable[visited[one-1]][0][index1] -1 ] = likelihood[visited[one-1]][0] / sumLikelihood;
							//				probOne[fglAlleleTable[visited[one-1]][1][index1] -1] = likelihood[visited[one-1]][1] / sumLikelihood;
							probOne[fglAlleleMatch[0][one - 1] - 1]
									= likelihood[visited[one - 1]][0]
											/ sumLikelihood;
							probOne[fglAlleleMatch[1][one - 1] - 1]
									= likelihood[visited[one - 1]][1]
											/ sumLikelihood;

							if (likelihood[visited[one - 1]][0]
									!= likelihood[visited[one - 1]][0]
									|| likelihood[visited[one - 1]][1]
											!= likelihood[visited[one - 1]][1]) {

								cout << "different!" << endl;
								exit(0);
							}
							//cout<<"probOne:" <<probOne[fglAlleleTable[visited[one-1]][0][index1]-1]  <<"," <<probOne[fglAlleleTable[visited[one-1]][1][index1]-1]  <<endl; ;
						} //close if
						//deterministic
						else if (visited[one - 1] == -1) {
							probOne[resolved[one - 1] - 1] = 1; //marker -1 offset
						}
						//follow population allele frequency
						else {
							for (int i = 0; i < countAllelicType; i++) {
								probOne[i] = alleleFreq[i];
								//cout << "here: " <<probOne[i] <<" ";
							}
							//cout <<endl;
						}

						//cout<<"TWO\n";
						//two
						if (visited[two - 1] >= 0) {
							//list<list<int> >::iterator myGList = graphList.begin();
							////roll to the right graph
							//for(int i=0; i< visited[two-1];i++){
							//++myGList;
							//}
							//list<int>::iterator iter=myGList->begin();
							//int index=0;
							//for(list<int>::iterator iter=myGList->begin(); iter != myGList->end(); ++iter){
							////cout <<"try " << *iter <<" ";
							//if(*iter == (two-1)){
							//index2 = index;
							//break;
							//}
							//index++;
							//}
							//cout<<endl;
							//cout<<index2 <<endl;
							double sumLikelihood =
									likelihood[visited[two - 1]][0]
											+ likelihood[visited[two - 1]][1];

							//probTwo[fglAlleleTable[visited[two-1]][0][index2] -1] = likelihood[visited[two-1]][0] / sumLikelihood;
							//probTwo[fglAlleleTable[visited[two-1]][1][index2] -1] = likelihood[visited[two-1]][1] / sumLikelihood;
							probTwo[fglAlleleMatch[0][two - 1] - 1]
									= likelihood[visited[two - 1]][0]
											/ sumLikelihood;
							probTwo[fglAlleleMatch[1][two - 1] - 1]
									= likelihood[visited[two - 1]][1]
											/ sumLikelihood;

						} //close if

						//deterministic
						else if (visited[two - 1] == -1) {
							probTwo[resolved[two - 1] - 1] = 1; //marker -1 offset
						}
						//follow population allele frequency
						else {
							for (int i = 0; i < countAllelicType; i++) {
								probTwo[i] = alleleFreq[i];
							}
						}

						//multiply probabilities
						for (int d1 = 1; d1 <= countAllelicType; d1++) {
							for (int d2 = 1; d2 <= countAllelicType; d2++) {
								int code = codeFunction(d1, d2);
								if (probOne[d1 - 1] * probTwo[d2 - 1] != 0) {
									g_dist[cur_ind][cur_marker][code]
											+= probOne[d1 - 1]
													* probTwo[d2 - 1]; //add prob 1 as weight
									//		   cout << g_dist[cur_ind][cur_marker][code] <<" " ;
								}
							}
						}

						if (DEBUG) {
							cout << "--------------------------------\n";
							cout << "FGL index: " << one - 1 << "," << two - 1
									<< "\n";
							cout << "Mar: " << denseMarkers->getValue(cur_ind,
									0, cur_marker) << "/"
									<< denseMarkers->getValue(cur_ind, 1,
											cur_marker) << "\n";
							cout << "Res: " << resolved[one - 1] << ","
									<< resolved[two - 1] << endl;
							cout << visited[one - 1] << " and " << visited[two
									- 1] << endl;
							//cout<<"P1: " << probOne[0] <<"," <<probOne[1] <<endl;
							//cout<<"P2: " << probTwo[0] <<"," <<probTwo[1] <<endl;
							cout << "P1: ";
							for (int type = 0; type < countAllelicType; type++) {
								cout << probOne[type] << ",";
							}
							cout << endl;
							cout << "P2: ";
							for (int type = 0; type < countAllelicType; type++) {
								cout << probTwo[type] << ",";
							}
							cout << endl;

							cout << "--------------------------------\n";
						}
					} //close else

				} //close less than 1 resolved


			} //close individuals loop

		}//inconsistent==0

		//cout<<"hi, de-allocate variables\n";

		//de-alllocate variables
		for (int i = 0; i < 2; i++) {
			delete[] fglAlleleMatch[i];
		}
		delete[] fglAlleleMatch;

		for (int i = 0; i < num_graphs; i++) {
			delete[] likelihood[i];
		}
		delete[] likelihood;
		delete[] visited;

		delete[] resolved;
		delete fglAlleleMatchPtr;
		delete likelihoodPtr;
		delete resolvedPtr;
		delete visitedPtr;

	}//over iterations


	//cout<<"average over iterations\n";

	//Genotype
	//averaging over iterations. dividing by the sum
	if (num_consistent_iterations > 0) {
		for (int per = 0; per < numInd; per++) {
			//cout<< pedigree->getPedMeio(per).id <<" " ;
			int numEntries = countAllelicType * (countAllelicType + 1) / 2;
			for (int cell = 0; cell < numEntries; cell++) {
				g_dist[per][cur_marker][cell] = g_dist[per][cur_marker][cell]
						/ num_consistent_iterations;
				//cout<< g_dist[per][cur_marker][cell] <<" ";
			}
			//cout<<endl;
		}
	}//num_consistent_iterations
	else {
		for (int per = 0; per < numInd; per++) {
			int numEntries = countAllelicType * (countAllelicType + 1) / 2;
			for (int cell = 0; cell < numEntries; cell++) {
				g_dist[per][cur_marker][cell] = 0;
			}
		}

	}
}



//restrict to single marker now

//first check whether both FGLs are resolved? if they are not resolved, this means we generate IBDG and the variable visited would be set.
//the variable visited would stay as -1 if every FGL is resolved. Hence, the use of the visited variable is appropriate only when some FGLs are not resolved, as indicated by first checked by the resolved variable instead of the visited variable
void impute_genotypeProb(FGL_Matrix *denseMarkers, Ped *pedigree,
		FGL_Matrix_List *fglm, double *alleleFreq, int countAllelicType,
		MTRand *mtrand1, double ***g_dist, int **inconsistency, int cur_marker) {

	int DEBUG = 0;
	int num_Iterations = fglm->n_Iterations;
	int numInd = pedigree->n_ind;

	int num_consistent_iterations = 0;

	for (int cur_it = 0; cur_it < num_Iterations; cur_it++) {
		//cout<<"cur_it = " <<cur_it <<"\n";
		FGL_Matrix *fglmatrix = fglm->getMatrix(cur_it);

		//if(cur_it==0){
		//for(int i=0; i<numInd; i++){
		//cout << "person " <<i  <<": "  << fglmatrix->getValue(i,0,0) <<"," <<fglmatrix->getValue(i,1,0) <<endl;
		//}
		//}

		int num_graphs = 0;
		int inconsistent = 0;
		double likelihood_resolved = 1;

		double ***likelihoodPtr = new double**;
		int ***fglAlleleMatchPtr = new int**;
		int **resolvedPtr = new int*;
		int **visitedPtr = new int*;
		//list<list<int> > *graphListPtr = new list<list<int> >;

		int CAN_CUT_SHORT = 0; //can't

		//cout<<"before calculating likelihood\n";
		calculate_graph_likelihood(denseMarkers, pedigree, fglmatrix,
				cur_marker, alleleFreq, DEBUG, &num_graphs, likelihoodPtr,
				&inconsistent, &likelihood_resolved, resolvedPtr, visitedPtr,
				fglAlleleMatchPtr, CAN_CUT_SHORT); // resolved is -1 if it is not resolved and set to the allele code if it is resolved. visited is -1 if it is deterministic, -2 if it is unobserved, 0,1,2 if the FGL belongs to graph 1,2,or 3.

		//calculate_graph_likelihood(denseMarkers, pedigree, fglmatrix, cur_marker, alleleFreq, DEBUG, &num_graphs, likelihoodPtr,
		//fglAlleleTablePtr, &inconsistent, &likelihood_resolved, &allObservedFGLResolved,
		//resolvedPtr, visitedPtr , graphListPtr, fglAlleleMatchPtr); // resolved is -1 if it is not resolved and set to the allele code if it is resolved. visited is -1 if it is deterministic, -2 if it is unobserved, 0,1,2 if the FGL belongs to graph 1,2,or 3.
		//cout<<"after calculating likelihood\n";
		//  calculate_graph_likelihood(denseMarkers, pedigree, fglmatrix, cur_marker, alleleFreq, DEBUG, &num_graphs, likelihoodPtr, fglAlleleTablePtr, &consistent, &likelihood_resolved, &allObservedFGLResolved
		//, resolvedPtr, visitedPtr, graphList);

		inconsistency[cur_it][cur_marker] = inconsistent;

		double **likelihood = *likelihoodPtr;
		int **fglAlleleMatch = *fglAlleleMatchPtr;
		int *resolved = *resolvedPtr;
		int *visited = *visitedPtr;

		//list<list<int> > graphList = *graphListPtr;


		//impute:
		//D.finally, we do the imputation probabilistically...(output)
		//so in each iteration, we end up with several probabilities locus by locus {ie. AA, Aa, aA, aa}
		// use the probabilities as weights to calculate the  [estimate] of probabilitiy distribution of imputed genotypes for each individual
		//and we can choose to report the probability distribution or the most likely state

		//TESTING!!!!!! - DELETE WHEN /* */ is removed!
		//if(inconsistent==0){
		//num_consistent_iterations++;
		//}

		//only impute if observed markers are consistent with the FGLg
		if (inconsistent == 0) {
			//cout<<"inconsistent ==0\n";
			num_consistent_iterations++;
			for (int cur_ind = 0; cur_ind < numInd; cur_ind++) {
				//cout<<"imputing: cur_ind " << cur_ind <<endl;

				//int one = fglmatrix->getValue(cur_ind, 0, cur_marker);
				//int two = fglmatrix->getValue(cur_ind, 1, cur_marker);
				int one = fglmatrix->getValue(cur_ind, 0, 0); //fglm now only has 1 marker
				int two = fglmatrix->getValue(cur_ind, 1, 0);

				//cout <<resolved[one -1 ] <<"," << resolved[two -1 ] <<endl;

				if (DEBUG) {
					cout << "currently processing: " << pedigree->getPedMeio(
							cur_ind).id << ", whose first FGL is " << one
							<< ", second FGL is " << two << endl;
				}

				//both already resolved
				if (((resolved[one - 1] != -1) && (resolved[two - 1] != -1))) {
					if (DEBUG) {
						cout << pedigree->getPedMeio(cur_ind).id
								<< "->both resolved\n";
						cout << " --> " << resolved[one - 1] << " "
								<< resolved[two - 1] << endl;
					}

					//alleles need to be labeled as 1,2,3,4,5,.... in consecutive order
					//since it has been resolved, get it from the resolved table, NOT from the markers...
					int d1 = resolved[one - 1];
					int d2 = resolved[two - 1];
					int code = codeFunction(d1, d2);
					//cout<< "the code is: " <<code <<".  d1 is " <<d1 <<" and d2 is " <<d2 <<endl;
					g_dist[cur_ind][cur_marker][code] += 1; //add prob 1 as weight
				}
				//or less than 2 resolved
				else {
					if (DEBUG) {
						cout << "visited[one-1]=" << visited[one - 1] << endl;
						cout << "visited[two-1]=" << visited[two - 1] << endl;
					}
					//determined = -1 , in a graph, or singleton -2?
					if ((visited[one - 1] == visited[two - 1]) && (visited[one
							- 1] >= 0)) {
						if (DEBUG) {
							cout << pedigree->getPedMeio(cur_ind).id
									<< "->not resolved, both from same graph\n";
						}
						//from same graph
						//	cout<<"from same graph\n";
						//list<list<int> >::iterator myGList = graphList.begin();

						////roll to the right graph
						//for(int i=0; i< visited[one-1];i++){
						//++myGList;
						//}

						//list<int>::iterator iter=myGList->begin();
						int code;
						//int index1, index2, code;
						//int index=0;
						//for(list<int>::iterator iter=myGList->begin(); iter != myGList->end(); ++iter){
						////			cout << *iter <<" ";
						//if(*iter == (one-1)){
						//index1 = index;
						//}
						//if((*iter==(two-1))){
						//index2 = index;
						//}
						//index++;
						//}
						//cout<<endl;
						//if(DEBUG){
						//cout <<"index 1: "<<index1 <<" and index 2:" <<index2 <<endl;
						//}


						//both permutations would give the same genotype
						//int old_d1 = fglAlleleTable[visited[one-1]][0][index1];
						//int old_d2 = fglAlleleTable[visited[one-1]][0][index2];
						//int old_d1_2 = fglAlleleTable[visited[one-1]][1][index1];
						//int old_d2_2 = fglAlleleTable[visited[one-1]][1][index2];

						int d1 = fglAlleleMatch[0][one - 1];
						int d2 = fglAlleleMatch[0][two - 1];
						int d1_2 = fglAlleleMatch[1][one - 1];
						int d2_2 = fglAlleleMatch[1][two - 1];

						if (DEBUG) {
							cout << "d1:" << d1 << " and d2:" << d2 << endl;
							cout << "d1_2:" << d1_2 << " and d2_2:" << d2_2
									<< endl;
						}

						double sumLikelihood = likelihood[visited[one - 1]][0]
								+ likelihood[visited[one - 1]][1];

						if (DEBUG) {
							cout << "likelihood: " << likelihood[visited[one
									- 1]][0] << " and "
									<< likelihood[visited[one - 1]][1] << endl;
						}

						code = codeFunction(d1, d2);
						//add code
						if (likelihood[visited[one - 1]][0] != 0) {
							g_dist[cur_ind][cur_marker][code]
									+= likelihood[visited[one - 1]][0]
											/ sumLikelihood; //in this case we also know the genotype, even though unresolved.
						}

						//configuration 2
						code = codeFunction(d1_2, d2_2);
						//add code
						if (likelihood[visited[one - 1]][1] != 0) {
							g_dist[cur_ind][cur_marker][code]
									+= likelihood[visited[one - 1]][1]
											/ sumLikelihood; //in this case we also know the genotype, even though unresolved.
						}

						if (DEBUG) {
							cout << " add code " << code << " with prob "
									<< likelihood[visited[one - 1]][1]
											/ sumLikelihood << endl;
						}
					}

					else {
						//check what type of FGLs for this person, and an adapter for the probability of possible alleles
						//cout<<"RUN HERE\n";
						if (DEBUG) {
							cout << pedigree->getPedMeio(cur_ind).id
									<< "->not from the same graph \n";
						}

						int index1, index2, code;

						//int a1[countAllelicType[cur_marker]];
						//WHY IS IT ALLOWABLE? I THOUGHT IT MUST BE DECLARED AS A POINTER?
						double probOne[countAllelicType]; //adpater
						double probTwo[countAllelicType]; //adapter

						for (int i = 0; i < countAllelicType; i++) {
							probOne[i] = 0;
							probTwo[i] = 0;
						}

						//cout<<"ONE\n";
						//one
						if (visited[one - 1] >= 0) {
							//list<list<int> >::iterator myGList = graphList.begin();
							////roll to the right graph
							//for(int i=0; i< visited[one-1];i++){
							//++myGList;
							//}
							//list<int>::iterator iter=myGList->begin();
							//int index=0;
							//for(list<int>::iterator iter=myGList->begin(); iter != myGList->end(); ++iter){
							////		cout << *iter <<" ";
							//if(*iter == (one-1)){
							//index1 = index;
							//break;
							//}
							//index++;
							//}
							//cout<<endl;
							//cout<<index1 <<endl;
							double sumLikelihood =
									likelihood[visited[one - 1]][0]
											+ likelihood[visited[one - 1]][1];

							//				probOne[fglAlleleTable[visited[one-1]][0][index1] -1 ] = likelihood[visited[one-1]][0] / sumLikelihood;
							//				probOne[fglAlleleTable[visited[one-1]][1][index1] -1] = likelihood[visited[one-1]][1] / sumLikelihood;
							probOne[fglAlleleMatch[0][one - 1] - 1]
									= likelihood[visited[one - 1]][0]
											/ sumLikelihood;
							probOne[fglAlleleMatch[1][one - 1] - 1]
									= likelihood[visited[one - 1]][1]
											/ sumLikelihood;

							if (likelihood[visited[one - 1]][0]
									!= likelihood[visited[one - 1]][0]
									|| likelihood[visited[one - 1]][1]
											!= likelihood[visited[one - 1]][1]) {

								cout << "different!" << endl;
								exit(0);
							}
							//cout<<"probOne:" <<probOne[fglAlleleTable[visited[one-1]][0][index1]-1]  <<"," <<probOne[fglAlleleTable[visited[one-1]][1][index1]-1]  <<endl; ;
						} //close if
						//deterministic
						else if (visited[one - 1] == -1) {
							probOne[resolved[one - 1] - 1] = 1; //marker -1 offset
						}
						//follow population allele frequency
						else {
							for (int i = 0; i < countAllelicType; i++) {
								probOne[i] = alleleFreq[i];
								//cout << "here: " <<probOne[i] <<" ";
							}
							//cout <<endl;
						}

						//cout<<"TWO\n";
						//two
						if (visited[two - 1] >= 0) {
							//list<list<int> >::iterator myGList = graphList.begin();
							////roll to the right graph
							//for(int i=0; i< visited[two-1];i++){
							//++myGList;
							//}
							//list<int>::iterator iter=myGList->begin();
							//int index=0;
							//for(list<int>::iterator iter=myGList->begin(); iter != myGList->end(); ++iter){
							////cout <<"try " << *iter <<" ";
							//if(*iter == (two-1)){
							//index2 = index;
							//break;
							//}
							//index++;
							//}
							//cout<<endl;
							//cout<<index2 <<endl;
							double sumLikelihood =
									likelihood[visited[two - 1]][0]
											+ likelihood[visited[two - 1]][1];

							//probTwo[fglAlleleTable[visited[two-1]][0][index2] -1] = likelihood[visited[two-1]][0] / sumLikelihood;
							//probTwo[fglAlleleTable[visited[two-1]][1][index2] -1] = likelihood[visited[two-1]][1] / sumLikelihood;
							probTwo[fglAlleleMatch[0][two - 1] - 1]
									= likelihood[visited[two - 1]][0]
											/ sumLikelihood;
							probTwo[fglAlleleMatch[1][two - 1] - 1]
									= likelihood[visited[two - 1]][1]
											/ sumLikelihood;

						} //close if

						//deterministic
						else if (visited[two - 1] == -1) {
							probTwo[resolved[two - 1] - 1] = 1; //marker -1 offset
						}
						//follow population allele frequency
						else {
							for (int i = 0; i < countAllelicType; i++) {
								probTwo[i] = alleleFreq[i];
							}
						}

						//multiply probabilities
						for (int d1 = 1; d1 <= countAllelicType; d1++) {
							for (int d2 = 1; d2 <= countAllelicType; d2++) {
								int code = codeFunction(d1, d2);
								if (probOne[d1 - 1] * probTwo[d2 - 1] != 0) {
									g_dist[cur_ind][cur_marker][code]
											+= probOne[d1 - 1]
													* probTwo[d2 - 1]; //add prob 1 as weight
									//		   cout << g_dist[cur_ind][cur_marker][code] <<" " ;
								}
							}
						}

						if (DEBUG) {
							cout << "--------------------------------\n";
							cout << "FGL index: " << one - 1 << "," << two - 1
									<< "\n";
							cout << "Mar: " << denseMarkers->getValue(cur_ind,
									0, cur_marker) << "/"
									<< denseMarkers->getValue(cur_ind, 1,
											cur_marker) << "\n";
							cout << "Res: " << resolved[one - 1] << ","
									<< resolved[two - 1] << endl;
							cout << visited[one - 1] << " and " << visited[two
									- 1] << endl;
							//cout<<"P1: " << probOne[0] <<"," <<probOne[1] <<endl;
							//cout<<"P2: " << probTwo[0] <<"," <<probTwo[1] <<endl;
							cout << "P1: ";
							for (int type = 0; type < countAllelicType; type++) {
								cout << probOne[type] << ",";
							}
							cout << endl;
							cout << "P2: ";
							for (int type = 0; type < countAllelicType; type++) {
								cout << probTwo[type] << ",";
							}
							cout << endl;

							cout << "--------------------------------\n";
						}
					} //close else

				} //close less than 1 resolved


			} //close individuals loop

		}//inconsistent==0

		//cout<<"hi, de-allocate variables\n";

		//de-alllocate variables
		for (int i = 0; i < 2; i++) {
			delete[] fglAlleleMatch[i];
		}
		delete[] fglAlleleMatch;

		for (int i = 0; i < num_graphs; i++) {
			delete[] likelihood[i];
		}
		delete[] likelihood;
		delete[] visited;

		delete[] resolved;
		delete fglAlleleMatchPtr;
		delete likelihoodPtr;
		delete resolvedPtr;
		delete visitedPtr;

	}//over iterations


	//cout<<"average over iterations\n";

	//Genotype
	//averaging over iterations. dividing by the sum
	if (num_consistent_iterations > 0) {
		for (int per = 0; per < numInd; per++) {
			//cout<< pedigree->getPedMeio(per).id <<" " ;
			int numEntries = countAllelicType * (countAllelicType + 1) / 2;
			for (int cell = 0; cell < numEntries; cell++) {
				g_dist[per][cur_marker][cell] = g_dist[per][cur_marker][cell]
						/ num_consistent_iterations;
				//cout<< g_dist[per][cur_marker][cell] <<" ";
			}
			//cout<<endl;
		}
	}//num_consistent_iterations
	else {
		for (int per = 0; per < numInd; per++) {
			int numEntries = countAllelicType * (countAllelicType + 1) / 2;
			for (int cell = 0; cell < numEntries; cell++) {
				g_dist[per][cur_marker][cell] = 0;
			}
		}

	}
}

//alternative estimator of probability
void impute_genotypeProb_withDenseMarker(FGL_Matrix *denseMarkers,
		Ped *pedigree, FGL_Matrix_List *fglm, double *alleleFreq,
		int countAllelicType, MTRand *mtrand1, double ***g_dist,
		int **inconsistency, int cur_marker) {

	double denominator_likelihood = 0;
	int DEBUG = 0;
	int num_Iterations = fglm->n_Iterations;
	int numInd = pedigree->n_ind;

	int num_consistent_iterations = 0;

	for (int cur_it = 0; cur_it < num_Iterations; cur_it++) {
		//cout<<"cur_it = " <<cur_it <<"\n";
		FGL_Matrix *fglmatrix = fglm->getMatrix(cur_it);

		int num_graphs = 0;
		int inconsistent = 0;
		double likelihood_resolved = 1;

		double ***likelihoodPtr = new double**;
		int ***fglAlleleMatchPtr = new int**;
		int **resolvedPtr = new int*;
		int **visitedPtr = new int*;

		int CAN_CUT_SHORT = 0;
		calculate_graph_likelihood(denseMarkers, pedigree, fglmatrix,
				cur_marker, alleleFreq, DEBUG, &num_graphs, likelihoodPtr,
				&inconsistent, &likelihood_resolved, resolvedPtr, visitedPtr,
				fglAlleleMatchPtr, CAN_CUT_SHORT); // resolved is -1 if it is not resolved and set to the allele code if it is resolved. visited is -1 if it is deterministic, -2 if it is unobserved, 0,1,2 if the FGL belongs to graph 1,2,or 3.


		inconsistency[cur_it][cur_marker] = inconsistent;

		double **likelihood = *likelihoodPtr;
		int **fglAlleleMatch = *fglAlleleMatchPtr;
		int *resolved = *resolvedPtr;
		int *visited = *visitedPtr;

		//calculate observed likelihood of the marker
		double likelihood_observed = likelihood_resolved;
		//calculate the likelihood
		for (int i = 0; i < num_graphs; i++) {
			double sumLikelihood = likelihood[i][0] + likelihood[i][1];
			if (DEBUG) {
				cout << "likelihood of graph " << i << " is " << sumLikelihood
						<< endl;
			}
			likelihood_observed *= sumLikelihood;
		}

		denominator_likelihood += likelihood_observed;

		//impute:
		//D.finally, we do the imputation probabilistically...(output)
		//so in each iteration, we end up with several probabilities locus by locus {ie. AA, Aa, aA, aa}
		// use the probabilities as weights to calculate the  [estimate] of probabilitiy distribution of imputed genotypes for each individual
		//and we can choose to report the probability distribution or the most likely state

		//only impute if observed markers are consistent with the FGLg
		if (inconsistent == 0) {
			int numCode = countAllelicType * (countAllelicType + 1) / 2;
			//cout<<likelihood_observed <<endl;
			num_consistent_iterations++;
			for (int cur_ind = 0; cur_ind < numInd; cur_ind++) {
				double *add = new double[numCode];
				for (int i = 0; i < numCode; i++) {
					add[i] = 0;
				}
				int one = fglmatrix->getValue(cur_ind, 0, 0); //fglm now only has 1 marker
				int two = fglmatrix->getValue(cur_ind, 1, 0);

				if (DEBUG) {
					cout << "currently processing: " << pedigree->getPedMeio(
							cur_ind).id << ", whose first FGL is " << one
							<< ", second FGL is " << two << endl;
				}

				//both already resolved
				if (((resolved[one - 1] != -1) && (resolved[two - 1] != -1))) {
					if (DEBUG) {
						cout << pedigree->getPedMeio(cur_ind).id
								<< "->both resolved\n";
						cout << " --> " << resolved[one - 1] << " "
								<< resolved[two - 1] << endl;
					}

					//alleles need to be labeled as 1,2,3,4,5,.... in consecutive order
					//since it has been resolved, get it from the resolved table, NOT from the markers...
					int d1 = resolved[one - 1];
					int d2 = resolved[two - 1];
					int code = codeFunction(d1, d2);
					add[code] += 1; //add prob 1 as weight
				}
				//or less than 2 resolved
				else {
					if (DEBUG) {
						cout << "visited[one-1]=" << visited[one - 1] << endl;
						cout << "visited[two-1]=" << visited[two - 1] << endl;
					}
					//determined = -1 , in a graph, or singleton -2?
					if ((visited[one - 1] == visited[two - 1]) && (visited[one
							- 1] >= 0)) {
						if (DEBUG) {
							cout << pedigree->getPedMeio(cur_ind).id
									<< "->not resolved, both from same graph\n";
						}
						//from same graph
						int code;
						int d1 = fglAlleleMatch[0][one - 1];
						int d2 = fglAlleleMatch[0][two - 1];
						int d1_2 = fglAlleleMatch[1][one - 1];
						int d2_2 = fglAlleleMatch[1][two - 1];

						if (DEBUG) {
							cout << "d1:" << d1 << " and d2:" << d2 << endl;
							cout << "d1_2:" << d1_2 << " and d2_2:" << d2_2
									<< endl;
						}

						double sumLikelihood = likelihood[visited[one - 1]][0]
								+ likelihood[visited[one - 1]][1];

						if (DEBUG) {
							cout << "likelihood: " << likelihood[visited[one
									- 1]][0] << " and "
									<< likelihood[visited[one - 1]][1] << endl;
						}

						code = codeFunction(d1, d2);
						//add code
						if (likelihood[visited[one - 1]][0] != 0) {
							add[code] += likelihood[visited[one - 1]][0]
									/ sumLikelihood; //in this case we also know the genotype, even though unresolved.
						}

						//configuration 2
						code = codeFunction(d1_2, d2_2);
						//add code
						if (likelihood[visited[one - 1]][1] != 0) {
							add[code] += likelihood[visited[one - 1]][1]
									/ sumLikelihood; //in this case we also know the genotype, even though unresolved.
						}

						if (DEBUG) {
							cout << " add code " << code << " with prob "
									<< likelihood[visited[one - 1]][1]
											/ sumLikelihood << endl;
						}
					}

					else {
						//check what type of FGLs for this person, and an adapter for the probability of possible alleles
						if (DEBUG) {
							cout << pedigree->getPedMeio(cur_ind).id
									<< "->not from the same graph \n";
						}

						int index1, index2, code;

						//WHY IS IT ALLOWABLE? I THOUGHT IT MUST BE DECLARED AS A POINTER?
						double probOne[countAllelicType]; //adpater
						double probTwo[countAllelicType]; //adapter

						for (int i = 0; i < countAllelicType; i++) {
							probOne[i] = 0;
							probTwo[i] = 0;
						}
						//one
						if (visited[one - 1] >= 0) {
							double sumLikelihood =
									likelihood[visited[one - 1]][0]
											+ likelihood[visited[one - 1]][1];
							probOne[fglAlleleMatch[0][one - 1] - 1]
									= likelihood[visited[one - 1]][0]
											/ sumLikelihood;
							probOne[fglAlleleMatch[1][one - 1] - 1]
									= likelihood[visited[one - 1]][1]
											/ sumLikelihood;

							if (likelihood[visited[one - 1]][0]
									!= likelihood[visited[one - 1]][0]
									|| likelihood[visited[one - 1]][1]
											!= likelihood[visited[one - 1]][1]) {

								cout << "different!" << endl;
								exit(0);
							}
						} //close if
						//deterministic
						else if (visited[one - 1] == -1) {
							probOne[resolved[one - 1] - 1] = 1; //marker -1 offset
						}
						//follow population allele frequency
						else {
							for (int i = 0; i < countAllelicType; i++) {
								probOne[i] = alleleFreq[i];
							}
						}

						//two
						if (visited[two - 1] >= 0) {
							double sumLikelihood =
									likelihood[visited[two - 1]][0]
											+ likelihood[visited[two - 1]][1];
							probTwo[fglAlleleMatch[0][two - 1] - 1]
									= likelihood[visited[two - 1]][0]
											/ sumLikelihood;
							probTwo[fglAlleleMatch[1][two - 1] - 1]
									= likelihood[visited[two - 1]][1]
											/ sumLikelihood;
						} //close if

						//deterministic
						else if (visited[two - 1] == -1) {
							probTwo[resolved[two - 1] - 1] = 1; //marker -1 offset
						}
						//follow population allele frequency
						else {
							for (int i = 0; i < countAllelicType; i++) {
								probTwo[i] = alleleFreq[i];
							}
						}

						//multiply probabilities
						for (int d1 = 1; d1 <= countAllelicType; d1++) {
							for (int d2 = 1; d2 <= countAllelicType; d2++) {
								int code = codeFunction(d1, d2);
								if (probOne[d1 - 1] * probTwo[d2 - 1] != 0) {
									add[code] += probOne[d1 - 1] * probTwo[d2
											- 1]; //add prob 1 as weight
								}
							}
						}

						if (DEBUG) {
							cout << "--------------------------------\n";
							cout << "FGL index: " << one - 1 << "," << two - 1
									<< "\n";
							cout << "Mar: " << denseMarkers->getValue(cur_ind,
									0, cur_marker) << "/"
									<< denseMarkers->getValue(cur_ind, 1,
											cur_marker) << "\n";
							cout << "Res: " << resolved[one - 1] << ","
									<< resolved[two - 1] << endl;
							cout << visited[one - 1] << " and " << visited[two
									- 1] << endl;
							cout << "P1: ";
							for (int type = 0; type < countAllelicType; type++) {
								cout << probOne[type] << ",";
							}
							cout << endl;
							cout << "P2: ";
							for (int type = 0; type < countAllelicType; type++) {
								cout << probTwo[type] << ",";
							}
							cout << endl;

							cout << "--------------------------------\n";
						}
					} //close else

				} //close less than 1 resolved

				for (int code = 0; code < numCode; code++) {
					g_dist[cur_ind][cur_marker][code] += add[code]
							* likelihood_observed;
				}
				delete[] add;
			} //close individuals loop

		}//inconsistent==0

		//cout<<"hi, de-allocate variables\n";

		//de-alllocate variables
		for (int i = 0; i < 2; i++) {
			delete[] fglAlleleMatch[i];
		}
		delete[] fglAlleleMatch;

		for (int i = 0; i < num_graphs; i++) {
			delete[] likelihood[i];
		}
		delete[] likelihood;
		delete[] visited;

		delete[] resolved;
		delete fglAlleleMatchPtr;
		delete likelihoodPtr;
		delete resolvedPtr;
		delete visitedPtr;

	}//over iterations

	//Genotype
	//averaging over iterations. dividing by the sum
	if (num_consistent_iterations > 0) {
		for (int per = 0; per < numInd; per++) {
			int numEntries = countAllelicType * (countAllelicType + 1) / 2;
			for (int cell = 0; cell < numEntries; cell++) {
				g_dist[per][cur_marker][cell] = g_dist[per][cur_marker][cell]
						/ denominator_likelihood;
			}
			//cout<<endl;
		}
	}//num_consistent_iterations
	else {
		for (int per = 0; per < numInd; per++) {
			int numEntries = countAllelicType * (countAllelicType + 1) / 2;
			for (int cell = 0; cell < numEntries; cell++) {
				g_dist[per][cur_marker][cell] = 0;
			}
		}

	}
}



//change method 1 calling - if maxCode has probability of 0, call 0/0 instead.
void callThreshold_multiAllelic(FGL_Matrix *genotypeCall, int startWriteIndex,
		double ***g_dist, Ped *pedigree, int num_Markers,
		int *countAllelicType, int method, double threshold, double threshold2) {


	if (method == 1) {
		//cout<< "Call method : most likely genotype configuration.\n";
	}
	else if (method == 2) {
		//cout	<< "Call method: threshold-based calling: T1="<< threshold << " and T2=" << threshold2 << ".\n";
	} else {
		cout << "error - please choose either method 1 or method 2 for calling genotypes. Exit this function now\n";
	}

	int numInd = pedigree->n_ind;
	int num_denseMarkers = num_Markers;

	//preset code tables:
	int MAX_NUM_ALLELES = 10;
	int TABLE_SIZE = (MAX_NUM_ALLELES - 1) * MAX_NUM_ALLELES / 2
			+ MAX_NUM_ALLELES;
	int *bigAllele = new int[TABLE_SIZE];
	int *smallAllele = new int[TABLE_SIZE];
	int code = 0;
	for (int b = 1; b <= MAX_NUM_ALLELES; b++) {
		for (int s = 1; s <= b; s++) {
			bigAllele[code] = b;
			smallAllele[code] = s;
			code++;
		}
	}

	//checking:
	//for(int i=0; i < TABLE_SIZE; i++){
	//	cout<< "code=" <<i <<" b=" <<bigAllele[i] <<" and s=" <<smallAllele[i] <<endl;
	//}


	//	cout<<"Imputed Results\n";
	for (int cur_ind = 0; cur_ind < numInd; cur_ind++) {
		//  cout << pedigree->getPedMeio(cur_ind).id <<"(" <<cur_ind <<") " ;
		//		  cout << pedigree->getPedMeio(cur_ind).id <<" ";
		for (int m = 0; m < num_denseMarkers; m++) {

			//expand coding table if the marker has the total number of alleles that is higher than the current maximum:
			if (countAllelicType[m] > MAX_NUM_ALLELES) {
				MAX_NUM_ALLELES = countAllelicType[m];
				TABLE_SIZE = (MAX_NUM_ALLELES - 1) * MAX_NUM_ALLELES / 2
						+ MAX_NUM_ALLELES;
				delete[] bigAllele;
				delete[] smallAllele;
				bigAllele = new int[TABLE_SIZE];
				smallAllele = new int[TABLE_SIZE];
				int code = 0;
				for (int b = 1; b <= MAX_NUM_ALLELES; b++) {
					for (int s = 1; s <= b; s++) {
						bigAllele[code] = b;
						smallAllele[code] = s;
						code++;
					}
				}
			}//expand

			if (method == 1) {
				//A. Choose the max
				int maxCode = -1;
				double max = 0;
				int numEntries = countAllelicType[m]
						* (countAllelicType[m] + 1) / 2;
				for (int cell = 0; cell < numEntries; cell++) {
					if (g_dist[cur_ind][m][cell] > max) {
						max = g_dist[cur_ind][m][cell];
						maxCode = cell;
					}

				}

				//translating from the code to the genotype.
				//countAlleleType[m]
				if (max > 0) {
					int b, s;
					b = bigAllele[maxCode];
					s = smallAllele[maxCode];
					genotypeCall->write2Matrix(b, cur_ind, 0, startWriteIndex
							+ m);
					genotypeCall->write2Matrix(s, cur_ind, 1, startWriteIndex
							+ m);
				} else {
					genotypeCall->write2Matrix(0, cur_ind, 0, startWriteIndex
							+ m);
					genotypeCall->write2Matrix(0, cur_ind, 1, startWriteIndex
							+ m);
				}
			} //end of method 1
			// ===========================================


			else if (method == 2) {
				//	cout<<"threshold method with T1=" <<threshold <<" and T2=" <<threshold2 <<":\n";
				//	cout<<"Imputed Results\n";
				int D_index = -1;
				int numEntries = countAllelicType[m]
						* (countAllelicType[m] + 1) / 2;
				for (int cell = 0; cell < numEntries; cell++) {
					if (g_dist[cur_ind][m][cell] > threshold) {
						D_index = cell;
					}
				}
				//if I can call both alleles
				if (D_index != -1) {
					int b, s;
					b = bigAllele[D_index];
					s = smallAllele[D_index];
					genotypeCall->write2Matrix(b, cur_ind, 0, startWriteIndex
							+ m);
					genotypeCall->write2Matrix(s, cur_ind, 1, startWriteIndex
							+ m);
				}
				//see if I can call at least 1 allele
				else {
					int callSingle = 0;
					for (int allele = 1; allele <= countAllelicType[m]; allele++) {
						double sumAlleleProb = 0;
						for (int otherAllele = 1; otherAllele
								<= countAllelicType[m]; otherAllele++) {
							sumAlleleProb += g_dist[cur_ind][m][codeFunction(
									allele, otherAllele)];
						}
						if (sumAlleleProb > threshold2) {
							genotypeCall->write2Matrix(allele, cur_ind, 0,
									startWriteIndex + m);
							genotypeCall->write2Matrix(0, cur_ind, 1,
									startWriteIndex + m);
							callSingle = 1;
							break;
						}
					}
					if (callSingle == 0) {
						//				cout << "0 0 ";
						genotypeCall->write2Matrix(0, cur_ind, 0,
								startWriteIndex + m);
						genotypeCall->write2Matrix(0, cur_ind, 1,
								startWriteIndex + m);
					}

				}//close else

			} //marker
			else {
				//unknown method of calling genotypes
			}

			//		  cout <<endl;
		} //ind
	}//

	delete[] bigAllele;
	delete[] smallAllele;

}


void printImputedProbabilitiesToFile(char *probFile, double ***full_g_dist,
		Ped *pedigree, int numMarkers, int *atypeCount) {
	ofstream probStream;
	probStream.open(probFile);
	if (!probStream) {
		cout
				<< "Cannot open the output file for writing the imputed probabilities.\n";
		exit(1);
	}
	for (int per = 0; per < pedigree->n_ind; per++) {
		probStream << pedigree->getPedMeio(per).id << " ";
		for (int l = 0; l < numMarkers; l++) {
			int numEntries = atypeCount[l] * (atypeCount[l] + 1) / 2;
			for (int cell = 0; cell < (numEntries); cell++) {
				probStream << full_g_dist[per][l][cell] << " ";
			}
			probStream << " ";
		}
		probStream << endl;
	}//for


	probStream.close();

}

void printBEAGLEImputedProbs(double ***full_g_dist, Ped *pedigree,
		int *atypeCount, ofstream *probStream, char *markerName) {
	*probStream << markerName;
	*probStream << " 1 2 ";

	if (atypeCount[0] != 2) {
		cout
				<< "Can't use the BEAGLE genotype output format, because it assumes di-allelic marker!\n";
	}

	for (int per = 0; per < pedigree->n_ind; per++) {
		int numEntries = atypeCount[0] * (atypeCount[0] + 1) / 2;
		for (int cell = 0; cell < (numEntries); cell++) {
			*probStream << full_g_dist[per][0][cell];
			if (cell != (numEntries - 1)) {
				*probStream << " ";
			}
		}
		if (per != (pedigree->n_ind - 1)) {
			*probStream << " ";
		}
	}
	*probStream << "\n";
}

void printBEAGLEImputedGenotype(FGL_Matrix *genotypeCall, Ped *pedigree,
		ofstream *genoStream, char *markerName) {

	*genoStream << "M ";
	*genoStream << markerName << " ";

	for (int p = 0; p < genotypeCall->n_ind; p++) {
		*genoStream << genotypeCall->getValue(p, 0, 0) << " "
				<< genotypeCall->getValue(p, 1, 0);
		if (p != (pedigree->n_ind - 1)) {
			*genoStream << " ";
		}

		//		*genoStream <<endl;
	}

	*genoStream << "\n";

}

void printImputedProbabilitiesToFileByComponent(double ***full_g_dist,
		Ped *pedigree, int numMarkers, int *atypeCount, ofstream *probStream) {
	for (int per = 0; per < pedigree->n_ind; per++) {
		*probStream << pedigree->getPedMeio(per).id << " ";
		for (int l = 0; l < numMarkers; l++) {
			int numEntries = atypeCount[l] * (atypeCount[l] + 1) / 2;
			for (int cell = 0; cell < (numEntries); cell++) {
				*probStream << full_g_dist[per][l][cell] << " ";
			}
			*probStream << "\t";
		}
		*probStream << endl;
	}//for


}

//ONLY WORK FOR DI-ALLELIC MARKER
void printDosageToFileByComponent(double ***full_g_dist,
		Ped *pedigree, int numMarkers, ofstream *dosageStream) {
	for (int per = 0; per < pedigree->n_ind; per++) {
		*dosageStream << pedigree->getPedMeio(per).id << " ";
		for (int l = 0; l < numMarkers; l++) {
			double dosage = full_g_dist[per][l][0] + 0.5*full_g_dist[per][l][1];
			*dosageStream << dosage <<" ";
		}
		*dosageStream << endl;
	}//for

}



void printImputedGenotypeToFile(char *genotypeFile, FGL_Matrix *genotypeCall,
		Ped *pedigree) {
	ofstream genoStream;
	genoStream.open(genotypeFile);
	if (!genoStream) {
		cout
				<< "Cannot open the output file for writing the imputed genotypes.\n";
		exit(1);
	}

	for (int p = 0; p < genotypeCall->n_ind; p++) {
		genoStream << pedigree->getPedMeio(p).id << " ";
		for (int l = 0; l < genotypeCall->n_loci; l++) {
			genoStream << genotypeCall->getValue(p, 0, l) << " "
					<< genotypeCall->getValue(p, 1, l) << " ";
		}
		genoStream << endl;
	}
	genoStream.close();

}

void printImputedGenotypeToFileByComponent(FGL_Matrix *genotypeCall,
		Ped *pedigree, ofstream *genoStream) {

	for (int p = 0; p < genotypeCall->n_ind; p++) {
		*genoStream << pedigree->getPedMeio(p).id << " ";
		for (int l = 0; l < genotypeCall->n_loci; l++) {
			*genoStream << genotypeCall->getValue(p, 0, l) << " "
					<< genotypeCall->getValue(p, 1, l) << " ";
		}
		*genoStream << endl;
	}
}


// this is basically an integer analogue to readMarkerPos_v2
// this parses through white space AND \n instead of just a single white space
int *readIntegerListFile(char *markerFile, int *number_markers) {
	int num_pos = 0;

	ifstream inFile;
	inFile.open(markerFile);
	if (!inFile) {
		cout << "Unable to open the integer list file. Exit now.\n";
		exit(1); // terminate with error
	}
	//determine how many markers
	string line;
	while (inFile >> line) {
		num_pos++;
	}
	inFile.close();

	//cout <<"number of position is " <<num_pos <<endl;

	//read content
	inFile.open(markerFile);
	if (!inFile) {
		cout << "Unable to open the integer list file. Exit now.\n";
		exit(1); // terminate with error
	}

	int *markerPos = new int[num_pos];
	int curMarkerPos = 0;

	for (int curMarkerPos = 0; curMarkerPos < num_pos; curMarkerPos++) {
		inFile >> line;
		// cout <<"print: " <<line <<endl;
		char *markerPosStr = new char[line.length() + 1];
		strcpy(markerPosStr, line.c_str());
		markerPos[curMarkerPos] = atoi(markerPosStr);
		delete markerPosStr;
	}

	inFile.close();

	*number_markers = num_pos;
	return (markerPos);
}

//v2 - also return # of markers read
// this parses through white space AND \n instead of just a single white space
double *readMarkerPos_v2(char *markerFile, int *number_markers) {
	int num_pos = 0;

	ifstream inFile;
	inFile.open(markerFile);
	if (!inFile) {
		cout << "Unable to open the marker position file. Exit now.\n";
		exit(1); // terminate with error
	}
	//determine how many markers
	string line;
	while (inFile >> line) {
		num_pos++;
	}
	inFile.close();

	//cout <<"number of position is " <<num_pos <<endl;

	//read content
	inFile.open(markerFile);
	if (!inFile) {
		cout << "Unable to open the marker position file. Exit now.\n";
		exit(1); // terminate with error
	}

	double *markerPos = new double[num_pos]; //just to play safe.
	int curMarkerPos = 0;

	for (int curMarkerPos = 0; curMarkerPos < num_pos; curMarkerPos++) {
		inFile >> line;
		// cout <<"print: " <<line <<endl;
		char *markerPosStr = new char[line.length() + 1];
		strcpy(markerPosStr, line.c_str());
		markerPos[curMarkerPos] = atof(markerPosStr);
		if(curMarkerPos >0){
			if(markerPos[curMarkerPos] < markerPos[curMarkerPos-1]){
				cout<<"Markers must be sorted by ascending order. Error in file " << markerFile<< ".\nExit with error\n";
				exit(0);
			}
		}
		delete markerPosStr;
	}

	inFile.close();

	*number_markers = num_pos;
	return (markerPos);
}

// this parses through white space AND \n instead of just a single white space
double *readMarkerPos(char *markerFile, int num_pos) {
	ifstream inFile;
	inFile.open(markerFile);
	if (!inFile) {
		cout << "Unable to open the marker position file. Exit now.\n";
		exit(1); // terminate with error
	}
	string line;
	double *markerPos = new double[num_pos]; //just to play safe.
	int curMarkerPos = 0;

	for (int curMarkerPos = 0; curMarkerPos < num_pos; curMarkerPos++) {
		inFile >> line;
		// cout <<"print: " <<line <<endl;
		char *markerPosStr = new char[line.length() + 1];
		strcpy(markerPosStr, line.c_str());
		markerPos[curMarkerPos] = atof(markerPosStr);
		delete markerPosStr;
	}

	inFile.close();
	return (markerPos);
}


void printInconsistency(const char *inconsistencyFile, int **inconsistency,
		int n_iterations, int n_loci) {
	cout << "printing inconsistency to " << inconsistencyFile << endl;
	ofstream out;
	out.open(inconsistencyFile);
	if (!out) {
		cout
				<< "Cannot open the output file for writing the consistency file.\n";
		exit(1);
	}

	for (int i = 0; i < n_iterations; i++) {
		for (int j = 0; j < n_loci; j++) {
			out << inconsistency[i][j] << " ";
			//cout << inconsistency[i][j] <<" ";
		}
		out << "\n";
		//cout <<"\n";
	}
	//cout << n_iterations << ","<<n_loci;
	out.close();
}

void printNumberConsistency(const char *consistencyFile, int **inconsistency,
		int n_iterations, int n_loci) {
	cout << "printing the number of consistent realizations per marker  to "
			<< consistencyFile << endl;
	ofstream out;
	out.open(consistencyFile);
	if (!out) {
		cout
				<< "Cannot open the output file for writing the consistency file.\n";
		exit(1);
	}

	for (int j = 0; j < n_loci; j++) {
		int numCon = 0;
		for (int i = 0; i < n_iterations; i++) {
			if (inconsistency[i][j] == 0) {
				numCon++;
			}
		}
		out << numCon << "\n";
	}
	//cout << n_iterations << ","<<n_loci;
	out.close();
}


void printNumberConsistencyByComponent(int **inconsistency, int n_iterations,
		int n_loci, ofstream *out) {

	for (int j = 0; j < n_loci; j++) {
		int numCon = 0;
		for (int i = 0; i < n_iterations; i++) {
			if (inconsistency[i][j] == 0) {   
				numCon++;
			}
		}
		*out << numCon << "\n";
	}
	//cout << n_iterations << ","<<n_loci;
}


//read in the num CONSISTENCY directly
void printNumberConsistencyByComponentMod(int *numconsistency, int n_iterations,
		int n_loci, ofstream *out) {

	for (int j = 0; j < n_loci; j++) {
		int numCon = numconsistency[j];
		*out << numCon << "\n";
	}
	//cout << n_iterations << ","<<n_loci;
}

void printNumberConsistencyAcrossNeighbours(const char *inconsistencyFile,
		int **inconsistency, int n_iterations, int n_loci, int numNeighbours) {

	cout
			<< "printing the number of inconsistent realizations per marker across neighbours to "
			<< inconsistencyFile << endl;
	ofstream out;
	out.open(inconsistencyFile);
	if (!out) {
		cout
				<< "Cannot open the output file for writing the inconsistency file.\n";
		exit(1);
	}
	//very inefficient routine, but it is only for exploratory purpose
	for (int j = 0; j < n_loci; j++) {
		int numCon = 0;
		int lowerBound = j - numNeighbours;
		int upperBound = j + numNeighbours;
		if (lowerBound < 0) {
			lowerBound = 0;
		}
		if (upperBound > (n_loci - 1)) {
			upperBound = n_loci - 1;
		}

		for (int i = 0; i < n_iterations; i++) {
			int consistentAcross = 1;
			for (int k = lowerBound; k <= upperBound; k++) {
				// cout <<"j=" <<j << " working on k=" <<k <<endl;
				if (inconsistency[i][k] == 1) {
					consistentAcross = 0;
					break;
				}
			}
			if (consistentAcross == 1) {
				numCon++;
			}
		}
		out << numCon << "\n";
	}
	//cout << n_iterations << ","<<n_loci;
	out.close();
}

void printErrorDetectionOutput(const char *errorOutputFile,
		double *pctConsistentIterations, double *probNoGenotypeError,
		int *mostLikelyPerson_given_error, int *mostLikelyConf_given_error,
		double *MLE_given_error, int n_loci, Ped *pedigree,
		int *countAllelicType, double *original_likelihood,
		int *mostLikelyPerson, double *MLE_person) {
	cout << "printing error output to to " << errorOutputFile << endl;
	ofstream out;
	out.open(errorOutputFile);
	if (!out) {
		cout
				<< "Cannot open the output file for writing the imputed probabilities.\n";
		exit(1);
	}

	out
			<< "pctConsistent probNoErr ML_person_g_err ML_conf_a1_g_err ML_conf_a2_g_err ML_prob orig_likelihood ML_person ML_person_prob\n";
	for (int i = 0; i < n_loci; i++) {
		int numAlleles = countAllelicType[i];
		int *codeToAlleles = new int[numAlleles + 1];
		codeToAlleles[0] = 0;
		for (int k = 1; k <= numAlleles; k++) {
			codeToAlleles[k] = codeToAlleles[k - 1] + k;
		}

		out << pctConsistentIterations[i] << " " << probNoGenotypeError[i]
				<< " ";
		if (mostLikelyPerson_given_error[i] == -1) {
			//out << ". . . .\n";
			out << ". . . . " << original_likelihood[i] << " . ." << "\n";
		} else {

			out << pedigree->getPedMeio(mostLikelyPerson_given_error[i]).id
					<< " ";

			int code = mostLikelyConf_given_error[i];
			int b = 1;
			while (code >= codeToAlleles[b]) {
				b++;
			}
			int s = code - codeToAlleles[b - 1] + 1;
			//out << s <<" " << b <<" " <<MLE_given_error[i] <<"\n";
			out << s << " " << b << " " << MLE_given_error[i] << " "
					<< original_likelihood[i] << " " << pedigree->getPedMeio(
					mostLikelyPerson[i]).id << " " << MLE_person[i] << "\n";
		}
		delete[] codeToAlleles;
	}

	out.close();

}




void printImputedProbabilitiesToFileByComponent_markerT(double ***full_g_dist,	Ped *pedigree, int atypeCount, char **markerNamePtr, ofstream *probStream) {
	*probStream << *markerNamePtr <<" ";
	for (int per = 0; per < pedigree->n_ind; per++) {
			int numEntries = atypeCount * (atypeCount + 1) / 2;
			for (int cell = 0; cell < (numEntries); cell++) {
				*probStream << full_g_dist[per][0][cell] << " ";
			}
			*probStream << "\t";
	}//for
	*probStream << endl;

}


//ONLY WORK FOR DI-ALLELIC MARKER
void printDosageToFileByComponent_markerT(double ***full_g_dist, Ped *pedigree, char **markerNamePtr, ofstream *dosageStream) {
	*dosageStream << *markerNamePtr <<" ";
	for (int per = 0; per < pedigree->n_ind; per++) {
					double dosage = full_g_dist[per][0][0] + 0.5*full_g_dist[per][0][1];
			*dosageStream << dosage <<" ";
	}//for
	*dosageStream << endl;

}





void printImputedGenotypeToFileByComponent_markerT(FGL_Matrix *genotypeCall,Ped *pedigree, char **markerNamePtr, ofstream *genoStream) {
	*genoStream << *markerNamePtr <<" ";
	for (int p = 0; p < genotypeCall->n_ind; p++) {
		*genoStream << genotypeCall->getValue(p, 0, 0) << " " << genotypeCall->getValue(p, 1, 0) << " ";
	}
	*genoStream << endl;
}



//main imputation routine
int main(int argc, // Number of strings in array argv
		char *argv[], // Array of command-line argument strings
		char *envp[]) {


	cout<<"\n<<GIGI: Genotype Imputation Given Inheritance (v1.04)>>\n\n";

	//Setting flags
	int verbose = 0; //to display the progress of imputation to the console display
    int longParam = 0;
    int seed =1234;
    int PROGRESS_NUM_MARKERS = 2000;
    char outDir[500]="";
    int fileTranspose = 0;

	//Read in the filename of the  parameter file
	if (argv[1] == NULL) {
		cout << "Command-line requires an argument which specifies the location of the parameter file.\n";
	}

	char *imputeParameterFileName = argv[1];



	int curConsoleArgument = 0;

	for(curConsoleArgument =2; curConsoleArgument < argc; curConsoleArgument ++){

	 if (argv[curConsoleArgument] != NULL) {

		if (strcmp(argv[curConsoleArgument], "-oldParam") == 0 || strcmp(argv[curConsoleArgument], "-longParam") == 0) {
			longParam = 1;
			cout<<"The option for using the long parameter file is turned on. [See the documentation file for the appropriate format of the parameter file.]\n";
		}

		if (strcmp(argv[curConsoleArgument], "-verbose") == 0) {
			verbose = 1;
			cout<<"The verbose option is turned on, so the console output of GIGI will be more comprehensive.\n";
		}

	//	int stringLength = strlen(argv[curConsoleArgument]);
		char substring_arg[500];
		memcpy( substring_arg, argv[curConsoleArgument], 6 );

		substring_arg[6] = '\0';
		if(strcmp(substring_arg, "-seed=") == 0){
		  char seed_arg[6];
		  memcpy( substring_arg, &argv[curConsoleArgument][6], 10 );
		  seed = atoi(substring_arg);
		  if(seed < 0 | seed > 999999){
			cout<<"Seed should be an integer value >0 and less than 999999. Exit with error\n";
			exit(0);
		  }
		  cout<<"Seed is now set to " << seed <<".\n";
		}
		if(strcmp(substring_arg, "-prog=") == 0){
		  char seed_arg[6];
		  memcpy( substring_arg, &argv[curConsoleArgument][6], 10 );
		  PROGRESS_NUM_MARKERS = atoi(substring_arg);
		  if(PROGRESS_NUM_MARKERS < 0 ){
			cout<<"The progress indicator should be an integer greater than 0. Exit with error\n";
			exit(0);
		  }
		  cout<<"The progress indicator is now set to " << PROGRESS_NUM_MARKERS <<".\n";
		}
		if(strcmp(substring_arg, "-outD=") == 0){
		  char seed_arg[6];
		  memcpy( substring_arg, &argv[curConsoleArgument][6], 200 );
		  strcpy(outDir,substring_arg);
		  strcat(outDir, "/");
		  cout<<"The output directory is now set to " << outDir <<"\n";
		}

		if (strcmp(argv[curConsoleArgument], "-long") == 0) {
			fileTranspose = 1;
			cout<<"The option for using the transposed dense marker file is activated. Please refer to the documentation for the appropriate file format.\n";
		}

	 }

	}

	//Initiate the filenames and variables that the parameter file will read in:

	char **pedigreeFilePtr = new char*;
	char **MIMFilePtr = new char*;
	char **markerPanelPosFilePtr = new char*;
	char **denseMarkersPosFilePtr = new char*;
	char **denseMarkersFilePtr = new char*;
	char **afFilePtr = new char*;
	char **genotypeFilePtr = new char*;
	char **probFilePtr = new char*;
	char **consistencyFilePtr = new char*;
 	char **dosageFilePtr = new char*;

	int num_Ped;

	int num_Original_markers, num_positions;
	long num_Iterations;
	int method;
	double threshold, threshold2;

	int printDosage = 0;
	char dosageFile[300];

	//Read the parameter file:
	//readImputeParameterFile_v0_05(imputeParameterFileName, pedigreeFilePtr,	&num_Ped, denseMarkersFilePtr, MIMFilePtr, &num_Iterations, markerPanelPosFilePtr, denseMarkersPosFilePtr, afFilePtr, &seed, genotypeFilePtr, &method, &threshold, &threshold2, probFilePtr,consistencyPtr);
	if(longParam == 1){
	 readImputeParameterFile_GIGI(imputeParameterFileName, pedigreeFilePtr, &num_Ped,  denseMarkersFilePtr, MIMFilePtr, &num_Iterations, markerPanelPosFilePtr, denseMarkersPosFilePtr,  afFilePtr, &seed,  genotypeFilePtr, &method, &threshold, &threshold2, probFilePtr, consistencyFilePtr);
	}
	else{
	 //the newest format of parameter file
	 readImputeParameterFile_GIGI_v1_01(imputeParameterFileName, pedigreeFilePtr, &num_Ped,  denseMarkersFilePtr, MIMFilePtr, &num_Iterations, markerPanelPosFilePtr, denseMarkersPosFilePtr,  afFilePtr, &seed,  genotypeFilePtr, &method, &threshold, &threshold2, probFilePtr, consistencyFilePtr);
	 num_Ped = 1;
	    char genotypeFile[300];
	 	strcpy(genotypeFile, outDir);
	    strcat(genotypeFile, "impute.geno");
	    char probFile[300];
	    strcpy(probFile, outDir);
	    strcat(probFile, "impute.prob");
	    char consistencyFile[300];
	    strcpy(consistencyFile, outDir);
	    strcat(consistencyFile, "impute.consistentIV");
	    strcpy(dosageFile, outDir);
		*genotypeFilePtr =  genotypeFile;
		*probFilePtr =  probFile;
		*consistencyFilePtr =  consistencyFile;  //consistencyFilePtr should really be named as consistencyFilePtr

	}
	//the original longParam format doesn't have a line to specify the location of the dosage file. It will be printed in the current directory when GIGI is run.
        strcat(dosageFile, "impute.dosage");
	*dosageFilePtr = dosageFile;

	//==============================================================================================
	//Read the data from the numerous files needed for imputation
	cout<< "1. Reading the input files: " <<endl;


	//Read the pedigree
	cout << " a. Now reading the Pedigree Meiosis file from '" << *pedigreeFilePtr <<"'\n";

	int num_Meiosis, curPed;
	curPed = 0;
	num_Ped = 1;
	PedList *pedList = readPedigrees(*pedigreeFilePtr, num_Ped); //read the pedigree meiosis file
	Ped *pedigree = (pedList->getPed(curPed));
	num_Meiosis = pedigree->n_Meio;
	cout<<	"     Pedigree has " << pedigree->n_ind << " individuals, from Subject1="
			<< pedigree->getPedMeio(0).id <<" to Subject"
			<< pedigree->n_ind <<"=" <<pedigree->getPedMeio(pedigree->n_ind -1).id
			<<" and contains "<< num_Meiosis <<" meioses in the file.\n\n";
	
	if(num_Meiosis ==0){
	  cout <<" *The pedigree-meiosis file indicates that the IV file produced by gl_auto should be empty. Are you imputing on trios or on any other pedigree structures that we cannot infer recombinations using unphased genotypes?\n"
	       <<" You should be aware that on trios, imputing genotypes usmain(ing GIGI is equivalent to imputing genotypes conditional on the pedigree structure and minor allele frequencies. "
	       <<" Information about linkage-disequilibrium is potentially useful for genotype imputation on trios. Since the current version of GIGI does not leverage information from "
	      <<" linkage-disequilibrium, you may want to use existing linkage-disequilibrium-based software that supports imputation on trios.\n\n\n";
	}




  int **personIndexPtr = new int*;
  int nObserved = 0; //initialize this variable

  ifstream *transposeStream;
if(fileTranspose){
//  transposeStream = initializeBEAGLE_denseMarkersStream("/acct/cykc/software/GIGI_v1.04/example/dense.genotypes.t", personIndexPtr, pedigree, &nObserved);
  transposeStream = initializeBEAGLE_denseMarkersStream(*denseMarkersFilePtr, personIndexPtr, pedigree, &nObserved);
}

//FGL_Matrix *denseMarker = readBEAGLE_denseMarkers_oneMarker(transposeStream, pedigree, *personIndexPtr, 22, markerNamePtr); //read a marker



	//Read the Framework IV file
	cout << " b. Now reading the framework IV file from '"<< *MIMFilePtr <<"'\n";
	//stream for IVs file
	ifstream *oscorStream = new ifstream;
	oscorStream->open(*MIMFilePtr);
	if (!(*oscorStream)) {
		cout << "Unable to open the meiosis indicator matrix file";
		exit(1); // terminate with error
	}
	OSCOR *oscorData;
	long oscorReadPos = 0;
	oscorData = readOSCOR_byComponent(*MIMFilePtr, num_Iterations, num_Meiosis, oscorStream, &oscorReadPos);
	cout << "     The framework IV file contains " <<num_Iterations <<" sampled IVs.\n\n";


	cout<< " c. Now reading from the Framework marker position file from '" << *markerPanelPosFilePtr <<"'\n";
	double *markerPos = readMarkerPos_v2(*markerPanelPosFilePtr,	&num_Original_markers); //read the map positions of the framework marker file
	cout<< "     Framework marker positions (cM): " << num_Original_markers <<" markers in the range " << markerPos[0] <<" to " <<markerPos[num_Original_markers-1] <<"\n\n";

	cout<< " d. Now reading the Dense marker position file from '" << *denseMarkersPosFilePtr <<"'\n";
	double *positions = readMarkerPos_v2(*denseMarkersPosFilePtr, &num_positions); //read the map positions of the dense marker file
	cout<< "     Dense marker positions (cM): " << num_positions <<" markers in the range " << positions[0] <<" to " <<positions[num_positions-1] <<"\n\n";
	//cout<< " GIGI read in " << num_Original_markers << " framework markers and "<< num_positions << " dense markers to impute." <<endl;


	//Read the dense markers
	cout<<" e. Now reading the Dense genotype file from '" <<*denseMarkersFilePtr <<"'\n";
	cout<<"     reminder: allelic types should be coded as 1,2,... (0 for missing)\n\n";

	FGL_Matrix *denseMarkers;
	int num_denseMarkers  = num_positions; 

if(!fileTranspose){
	denseMarkers = readDenseMarkers_byComponent(*denseMarkersFilePtr, pedigree, num_positions);
	//num_denseMarkers = denseMarkers->n_loci;
}

	//read allele frequency
	cout << " f. Now reading the Allele frequency file from '"<< *afFilePtr  <<"'\n";
	//read the file once to count how many allelic type is in each marker first, so the right number of allelic type can be
	//initialized in afreq later
	int *atypeCount = new int[num_positions];
	readAllelicTypeCount(*afFilePtr, num_positions, atypeCount);
	int numAllelicTypes=0;
	int *allelicTypes = new int[100];
	int *allelicTypesCount = new int[100];
	for(int i=0; i < num_positions; i++){
		int foundNumAllelicTypes = 0;
		int cur_found = -1;
		for(int j=0; j< numAllelicTypes; j++){
			if(atypeCount[i] ==allelicTypes[j]){
				foundNumAllelicTypes = 1;
				cur_found = j;
				break;
			}
		}
		//increment the count of the appropriate allelic type
		if(foundNumAllelicTypes == 1){
			allelicTypesCount[cur_found]++;
		}
		else{
			//new number of allelic types detected
			allelicTypes[numAllelicTypes] = atypeCount[i];
			allelicTypesCount[numAllelicTypes] = 1;
			numAllelicTypes ++;
		}
	}
	cout << "     The file includes N markers with (M allelic types): ";
	for(int i=0; i < numAllelicTypes; i++){
		cout<<allelicTypesCount[i] <<"("<< allelicTypes[i]  <<") ";
	}
        cout<<"\n";
	cout<<"    *[Reminder!] In each line of the allele frequency file, the first number should correspond to the allele frequency of the allele labeled \"1\""
	     << " in the genotype file, the second number should correspond to the allele frequency of allele labeled \"2\". This is a common mistake!\n";
	cout<<"\n";


	if((numAllelicTypes == 1) && (allelicTypes[0]==2)){
	  cout<<"    *The markers to be imputed are all di-allelic markers. The dosage file will be generated along with the imputed genotype file and the probability file.\n";
	  cout<<"     Dosage is the expected percent of 1-allele in a genotype.\n      [eg. 1 if P(genotype is 1/1)=1]\n";
	  printDosage = 1;
	}

	double **afreq = new double*[num_positions];
	for (int dm = 0; dm < num_positions; dm++) {
      	   if(atypeCount[dm]>1){
		afreq[dm] = new double[atypeCount[dm]];
	    }
	   else if(atypeCount[dm]==1){
             cout<< "[Warning:] You only specified 1 allele frequency in marker " << dm+1 << ". I suppose this marker is monomorphic?\n";
        //in this case, it screws up some normal routine.. I think it screwed up the imputation routine, which 
        //expects that a certain marker has at least 2 alleles... as a quick patch, if a marker has 1 allele with allele frequency
        //equal to 1, I will change the monomorphic marker into diallelic marker with allele frequency 1 and 0.
 	//I will also provide a warning documenting that I have changed the value...

	    atypeCount[dm] = 2;
	    afreq[dm] = new double[2]; //I also updated the atypeCount here. It won't interfere the readAF routine, because
                     //readAF just keeps reading entries in the line until it reaches the end..(it doesn't depend on knowing atypeCount)
	     afreq[dm][1] = 0; //initialize it to 0.	     
	  }
	}
	readAF(*afFilePtr, num_positions, afreq, atypeCount); //here will change the atypeCount to 2 if this marker is specified as monomorphic //afreq would be 1 and 0
	


//	cout<<"finish first!\n";
//				exit(0);


	cout<<"\n2. Options:\n";
	cout<<" a. Call method: ";
	if(method==1){
		cout<<"most likely genotypes" <<endl;
	}
	else if(method==2){
		cout<<"confidence-based thresholds: t1=" << threshold <<", t2=" <<threshold2 <<endl;
	}
	else{ cout<<"The call method should be 0 or 1. Please check the parameter file.\n";
		exit(0);
	}
	

	MTRand *mtrand1 = new MTRand();  //random number generation
	mtrand1->seed(seed); //set the seed of the random number generator
	cout <<" b. Seed=" <<seed;
	if(seed==1234){
		cout <<" (default)\n";
	}
	cout <<"\n";




	//Declare the location of the output files:
	cout<<"\n3. The output files will be saved to:\n";
	cout<< " a. " <<*genotypeFilePtr <<" (called genotypes)\n";
	cout<< " b. " <<*probFilePtr<<" (estimated genotype probabilities)\n";
	cout<< " c. " <<*consistencyFilePtr <<" (number of consistent IVs by marker)\n";
	if(printDosage){
	cout<< " d. " <<*dosageFilePtr <<" (dosage file of imputed genotypes)\n";
	}


	ofstream *consistencyStream = new ofstream;
	consistencyStream->open(*consistencyFilePtr);
	if (!(*consistencyStream)) {
		cout << "Cannot open the output file for writing the consistency file."<<*consistencyFilePtr <<"\n";
		exit(1);
	}
	*consistencyStream << "Pedigree " << curPed + 1 << endl;

	ofstream *probStream = new ofstream;
	probStream->open(*probFilePtr);
	if (!(*probStream)) {
		cout
				<< "Cannot open the output file for writing the imputed probabilities." <<*probFilePtr <<"\n";
		exit(1);
	}

	ofstream *genoStream = new ofstream;
	genoStream->open(*genotypeFilePtr);
	if (!(*genoStream)) {
		cout
				<< "Cannot open the output file for writing the imputed genotypes."<<*genotypeFilePtr << "\n";
		exit(1);
	}

	ofstream *dosageStream = new ofstream;
	if(printDosage){
 	  dosageStream->open(*dosageFilePtr);
	  if (!(*dosageStream)) {
		cout<< "Cannot open the output file for writing the imputed genotypes."<<*dosageFilePtr << "\n";
		exit(1);
	  }
	}

if(fileTranspose){

//Header for the files:

*probStream << "id ";
for(int i=0; i < pedigree->n_ind; i++){
  *probStream << pedigree->getPedMeio(i).id <<" " << pedigree->getPedMeio(i).id <<" " << pedigree->getPedMeio(i).id <<"\t";
}
*probStream <<endl;

*genoStream << "id ";
for(int i=0; i < pedigree->n_ind; i++){
  *genoStream << pedigree->getPedMeio(i).id <<" " << pedigree->getPedMeio(i).id <<" ";
}
*genoStream <<endl;

*dosageStream << "id ";
for(int i=0; i < pedigree->n_ind; i++){
  *dosageStream << pedigree->getPedMeio(i).id <<" ";
}
*dosageStream << endl;

}

	//======================================================================================================================

	//OSCOR *oscorData;
	//int num_Meiosis, curPed; //moved to earlier
	//long oscorReadPos = 0;
	//Begin processing each pedigree:
//	for (curPed = 0; curPed < num_Ped; curPed++) {
		//Ped *pedigree = (pedList->getPed(curPed)); //moved to earlier
		//num_Meiosis = pedigree->n_Meio; //moved to earlier

		//cout << "Processing pedigree number " << curPed + 1 << " (" << pedigree->n_ind << " individuals)\n";
		//Read OSCOR file
		//oscorData = readOSCOR_byComponent(*MIMFilePtr, num_Iterations, num_Meiosis, oscorStream, &oscorReadPos);

		//read the available dense markers for imputation
		//FGL_Matrix *denseMarkers = readDenseMarkers_byComponent(*denseMarkersFilePtr, pedigree, num_positions);
		//int num_denseMarkers = denseMarkers->n_loci;

		//Declare variables to store the output ====================================================================
		FGL_Matrix *genotypeCall;
		double ***full_g_dist;
//		int **inconsistency;
		int *numconsistency;
if(!fileTranspose){
		genotypeCall = new FGL_Matrix_int;
		genotypeCall->FGL_Matrix_INIT(pedigree->n_ind, num_denseMarkers);

		full_g_dist = new double**[pedigree->n_ind];
		for (int per = 0; per < pedigree->n_ind; per++) {
			full_g_dist[per] = new double*[num_denseMarkers];
			for (int mar = 0; mar < num_denseMarkers; mar++) {
				int numEntries = atypeCount[mar] * (atypeCount[mar] + 1) / 2;
				full_g_dist[per][mar] = new double[numEntries];
				for (int cell = 0; cell < (numEntries); cell++) {
					full_g_dist[per][mar][cell] = 0;
				}
			}
		}

		numconsistency = new int[num_denseMarkers];
		for(int j=0; j < num_denseMarkers; j++){
		  numconsistency[j]=0;
		}
//		inconsistency = new int*[num_Iterations];
//		for (int i = 0; i < num_Iterations; i++) {
//			inconsistency[i] = new int[num_denseMarkers];
//			for (int j = 0; j < num_denseMarkers; j++) {
//				inconsistency[i][j] = 0;
//			}
//		}
} //fileTranspose


		//============================================================================================================


		MIM_List *mim_list = NULL;
		MIM_List *miml_pos = NULL;
		FGL_Matrix_List *fglm;
		double currentPos;
		double *leftRightFlankingMarkerPos;
		int curSparseMarkerIndex = 0;
		int readNewMIML = 0;

		//read the IVs for a pedigree
		mim_list = readMIM_partial(oscorData, num_Iterations, num_Meiosis, 1, NULL, 0, 0); //assume previousMIM read in order -each marker is read?


		cout<<"\n4. Start imputing genotypes: [Progress shown every "<<PROGRESS_NUM_MARKERS <<"th markers]\n";
		//Impute each dense marker one by one
		for (int cur_l = 0; cur_l < num_denseMarkers; cur_l++) {
			if(cur_l%PROGRESS_NUM_MARKERS == (PROGRESS_NUM_MARKERS-1)){
				cout<<" progress: imputed "<<cur_l+1 << " markers\n";
			}

			//To impute dense marker, we first have to infer IV at an arbitrary position: [the next many lines process this information]
			readNewMIML = 0;
			int previousStartingIndex = curSparseMarkerIndex - 1;

			if (verbose) {
				cout << " imputing marker " << cur_l + 1 << "\t";
			}

			currentPos = positions[cur_l];

			//determine whether we should read in the position of the next sparse marker
			while (markerPos[curSparseMarkerIndex] <= currentPos) { //> ?
				if (curSparseMarkerIndex == num_Original_markers) {
					break; //how to deal?
				}
				curSparseMarkerIndex++;
				readNewMIML = 1; //expect to read new MIML
			}
			int numSparseMarkers;
			//an impilcit assumption is that there should be more than 1 framework marker?
			//right boundary case? (only left flanking marker)
			if (curSparseMarkerIndex == num_Original_markers) {
				leftRightFlankingMarkerPos = new double[1];
				leftRightFlankingMarkerPos[0] = markerPos[curSparseMarkerIndex	- 1];
				//leftRightFlankingMarkerPos[1] = -1;
				numSparseMarkers = 1;
				if(verbose){
					cout << "Right Boundary Case\t";
				}
			}
			//left boundary case (only right flanking marker)
			else if (curSparseMarkerIndex == 0) {
				leftRightFlankingMarkerPos = new double[1];
				leftRightFlankingMarkerPos[0] = markerPos[0];
				numSparseMarkers = 1;
				if(verbose){
					cout << "Left Boundary Case\t";
				}
			} else {
				leftRightFlankingMarkerPos = new double[2];
				leftRightFlankingMarkerPos[0] = markerPos[curSparseMarkerIndex	- 1];
				leftRightFlankingMarkerPos[1] = markerPos[curSparseMarkerIndex];
				numSparseMarkers = 2;
			}

			//this should read in the flanking markers
			if (readNewMIML == 1) {
				MIM_List *previousMIM = mim_list;
				int newSparse_startingIndex = curSparseMarkerIndex - 1;
				//leftboundary case
				if (curSparseMarkerIndex == 0) {
					newSparse_startingIndex = 0;
				}
				if (verbose) {
					//cout<<"about the mim_list, newSparse_startingIndex=" <<newSparse_startingIndex <<" , previousStartingIndex=" << previousStartingIndex <<endl;
				}
				mim_list = readMIM_partial(oscorData, num_Iterations,
						num_Meiosis, numSparseMarkers, previousMIM,
						newSparse_startingIndex, previousStartingIndex); //assume previousMIM read in order -each marker is read?
				delete previousMIM;
			}
			if (verbose) {
				cout <<"pos=" << currentPos <<"\t";
				cout << "flanking FW pos=(";
				for (int i = 0; i < numSparseMarkers; i++) {
					cout << leftRightFlankingMarkerPos[i] ;
					if(i != (numSparseMarkers -1)){
						  cout <<" " ;
					}
				}
				cout<<")  \t";
				//cout << ") cur locus: " << currentPos << "\t";
			}
			miml_pos = MIM_pos(mim_list, num_Iterations, 1, num_Meiosis,mtrand1, &currentPos, leftRightFlankingMarkerPos, numSparseMarkers);
			//convert to FGLM
			fglm = MIM2FGL(miml_pos, pedigree);  //convert the IV into FGLs
			delete miml_pos; //after convertion to fglm, miml_pos is not needed anymor
			delete[] leftRightFlankingMarkerPos;
			//=================================================================================================================================
			//finished reading in the IVs at the position of this marker



			//allele frequency of this marker-------------------------- impute_genotypeProb requires us to read supply the af of the current marker only
			int *atypeCountMarker = new int[1];
			atypeCountMarker[0] = atypeCount[cur_l];
			double **afreqMarker = new double*[1];
			afreqMarker[0] = new double[atypeCountMarker[0]];
			for (int i = 0; i < atypeCountMarker[0]; i++) {
				afreqMarker[0][i] = afreq[cur_l][i];
			}


if(!fileTranspose){
			//MTRand ranNumGen;
			//impute_genotypeProb(denseMarkers, pedigree, fglm, afreqMarker[0], *atypeCountMarker, &ranNumGen, full_g_dist, consistency, cur_l);
			impute_genotypeProbMod(denseMarkers, pedigree, fglm, afreqMarker[0], *atypeCountMarker, mtrand1, full_g_dist, numconsistency, cur_l);

			//alternative estimator of probability
			//impute_genotypeProb_withDenseMarker(denseMarkers,pedigree,fglm, afreqMarker[0], *atypeCountMarker, &ranNumGen, full_g_dist, consistency, cur_l);

			//calculate the number of consistent realizations
			int numCon = numconsistency[cur_l];
			//for (int i = 0; i < num_Iterations; i++) {
			//	if (inconsistency[i][cur_l] == 0) {
			//		numCon++;
			//	}
			//}
			
			if (verbose) {
				cout << " # consistent IVs=" << numCon << endl;
				cout << endl;
			}

}//!fileTranspose
else{

		//initialize the g_dist_marker and consistency_marker
		double ***g_dist_marker = new double**[pedigree->n_ind];
		for (int per = 0; per < pedigree->n_ind; per++) {
			g_dist_marker[per] = new double*[1]; // 1 marker only
				int numEntries = *atypeCountMarker * (*atypeCountMarker + 1) / 2;
				g_dist_marker[per][0] = new double[numEntries];
				for (int cell = 0; cell < (numEntries); cell++) {
					g_dist_marker[per][0][cell] = 0;
				}
		}

		//int **inconsistency_marker = new int*[num_Iterations];
		//for (int i = 0; i < num_Iterations; i++) {
		//	inconsistency_marker[i] = new int[1]; //1 marker 
		//	inconsistency_marker[i][0] = 0;
		//}
		int *numconsistency_marker = new int[1];
		numconsistency_marker[0] = 0;

		char **markerNamePtr = new char*; //name of the marker is specified in the long file
			//read a marker at a time
			FGL_Matrix *denseMarker_one = readBEAGLE_denseMarkers_oneMarker(transposeStream, pedigree, *personIndexPtr, 22, markerNamePtr); //read a marker

			//impute
			impute_genotypeProbMod(denseMarker_one, pedigree, fglm, afreqMarker[0], *atypeCountMarker, mtrand1, g_dist_marker, numconsistency_marker, 0);			



			//calculate the number of consistent realizations
			//int numCon = 0;
			//for (int i = 0; i < num_Iterations; i++) {
			//	if (inconsistency_marker[i][0] == 0) {
			//		numCon++;
			//	}
			//}
			int numCon = numconsistency_marker[0];

//for(int i=0; i < pedigree->n_ind; i++){
// int numEntries = *atypeCountMarker * (*atypeCountMarker + 1) / 2;
// for(int j=0; j < numEntries; j++){
//   cout << g_dist_marker[i][0][j] <<" ";
// }
// cout <<endl;
//}

//cout<<"numCon=" <<numCon <<endl;
			//write
			*consistencyStream << numCon <<endl; //consistency file 

		FGL_Matrix *genotypeCall_marker = new FGL_Matrix_int;
		genotypeCall_marker->FGL_Matrix_INIT(pedigree->n_ind, 1);
		callThreshold_multiAllelic(genotypeCall_marker, 0, g_dist_marker, pedigree, 1, atypeCountMarker, method, threshold, threshold2);
		printImputedGenotypeToFileByComponent_markerT(genotypeCall_marker, pedigree, markerNamePtr, genoStream);
		printImputedProbabilitiesToFileByComponent_markerT(g_dist_marker, pedigree, *atypeCountMarker, markerNamePtr, probStream);
		if(*atypeCountMarker > 2){
		  cout <<"Warning: marker " << cur_l + 1 <<" has more than 2 alleles, so it will print probabilities from more than 3 possible genotype configurations.\n"
 			<< "The header assumes that all the markers have only 3 possible genotype configurations, so the output of this marker will not match the header.\n";
		}
		if(printDosage == 1){
		  printDosageToFileByComponent_markerT(g_dist_marker, pedigree, markerNamePtr, dosageStream); //this routine only works for di-allelic markers
		}


			//delete
			delete denseMarker_one;
			delete genotypeCall_marker;
			//delete g_dist_marker and inconsistency_marker also
			for (int i = 0; i < pedigree->n_ind; ++i) {
				for (int j = 0; j < 1; ++j) {
					delete[] g_dist_marker[i][j];
				}
				delete[] g_dist_marker[i];
			}
			delete[] g_dist_marker;
			//for (int i = 0; i < num_Iterations; i++) {
			//	delete[] inconsistency_marker[i];
			//}
			//delete[] inconsistency_marker;
			delete[] numconsistency_marker;

			delete[] markerNamePtr[0];
			delete[] markerNamePtr;
}
			delete[] atypeCountMarker;
			delete[] afreqMarker[0];
			delete[] afreqMarker;

			//Deallocate component specific variables
			//it is so weird i am reading in pedigree by pedigree
			delete fglm; //called a destructor


		}//end of processing the current marker
		delete mim_list; //delete the last mim

		//cout << endl;
		cout << " Done calculating genotype probabilities for " << num_positions  <<" markers!\n";
		//Printing output
		cout << "\n5. Now calling genotypes.\n";
if(!fileTranspose){
		callThreshold_multiAllelic(genotypeCall, 0, full_g_dist, pedigree,	num_denseMarkers, atypeCount, method, threshold, threshold2);
		//callThreshold(genotypeCall, 0, full_g_dist, pedigree, denseMarkers->n_loci, atypeCount, method,threshold, threshold2);
		//printNumberConsistencyByComponent(inconsistency, num_Iterations, num_denseMarkers, consistencyStream);
		printNumberConsistencyByComponentMod(numconsistency, num_Iterations, num_denseMarkers, consistencyStream);
		printImputedGenotypeToFileByComponent(genotypeCall, pedigree, genoStream);
		printImputedProbabilitiesToFileByComponent(full_g_dist, pedigree, num_denseMarkers, atypeCount, probStream);
		if(printDosage == 1){
		  printDosageToFileByComponent(full_g_dist, pedigree, num_denseMarkers, dosageStream); //this routine only works for di-allelic markers
		}
} //!fileTranspose
else{
	 closeBEAGLE_denseMarkersStream(transposeStream);
}
		cout<<" Done!\n";
		cout<<"\n";


		oscorStream->close();
		consistencyStream->close();
		genoStream->close();
		probStream->close();
		if(printDosage){
		 dosageStream->close();
		}

if(!fileTranspose){
		//Deallocate variables
		for (int i = 0; i < pedigree->n_ind; ++i) {
			for (int j = 0; j < num_denseMarkers; ++j) {
				delete[] full_g_dist[i][j];
			}
			delete[] full_g_dist[i];
		}
		delete[] full_g_dist;
		//for (int i = 0; i < num_Iterations; i++) {
		//	delete[] consistency[i];
		//}
		//delete[] consistency;
		delete[] numconsistency;
		delete denseMarkers;
		delete genotypeCall;
}//!fileTranspose
		delete oscorData;

//	} //end of processing the current component


	delete[] atypeCount;
	for (int i = 0; i < num_positions; i++) {
		delete[] afreq[i];
	}
	delete[] afreq;



	delete pedList;

	return 0;
}
